import { qGrid } from "../../grid";
import dxDataGrid from 'devextreme/ui/data_grid';
import { cxGridIncluirFiltroLookup } from "../../nLookUp";

/**
 * 
 * @param {Object} _AData Data with necessary configuration to build a grid 
 * @param {Object} options Additional configuration
 * @param {Object} options.selection Selection style
 * @param {Object} options.targetButton Button trigger of search modal
 */
export function BuildModalGrid(_AData, { selection, targetButton, gridLookupHandler } = {}) {
	let selectionType = {
		selection: {
			allowSelectAll: true,
			selectAllMode: "page",
			mode: "single",
			...selection
		}
	}; 	
	
	let consulta = { ...JSON.parse(_AData.consulta), ...selectionType };
	let dataSource = createdxDatasource({
		_AClassName: _AData.Class, 
		_AFuncaoFiltroCX: cxGridIncluirFiltroLookup,
		options: { targetButton, gridLookupHandler }
	});
	let paramsConfig = 	{
		Class: _AData.Class,
		GridName: consulta.nomegrid,
		DataSource: dataSource
	};

	evaluateGridConfig(consulta);

	let config = prepareGridConfig(GridConfigModal.config(paramsConfig), consulta);

	createGrid("dataGridModal", config);

	$('#myModal').find('.modal-title').text('Consulta '+ _AData.Caption);
}

/**
 * Grid configuration for modal
 */
class GridConfigModal {
	static config({ DataSource, Class, GridName }) {
		const rowDoubleClick = function () {
			RetornarData();
			$("#myModal").modal('hide');
		}
		
		const stateStoring = {
			enabled: true,
			type: "custom",
			customLoad: function () { return qGrid.loadGridState(Class + GridName); },
			customSave: function (state) { qGrid.saveGridState(Class + GridName, state); }
		}
		
		const onToolbarPreparing = function (e) {
			var toolbarItems = e.toolbarOptions.items;
		
			toolbarItems.push({
				widget: "dxButton",
				locateInMenu: "auto",
				showText: "inMenu",
				options: {
					icon: "refresh",
					text: "Atualizar",
					hint: "Atualizar",
					onClick: function () { dxDataGrid.getInstance("#dataGridModal").refresh(); }
				},
				location: "after"
			});
			
			toolbarItems.push({
				location: "after",
				widget: "dxDropDownButton",
				options: {
					dropDownOptions: {
						width: 140
					},
					displayExpr: "text",
					icon: "las la-th",
					hint: "Opções do grid",
					items: [
						{ text: "Restaurar colunas", action: "restaurar"},
						{ text: "Selecionar colunas", action: "selecionar"}
					],
					onItemClick(data) {
						if (data.itemData.action == 'restaurar'){
							dxDataGrid.getInstance('#dataGridModal').state({});			
						} else {
							dxDataGrid.getInstance("#dataGridModal").showColumnChooser();
						}
					}
				}
			});	    		
		}

		return {
			onRowDblClick: rowDoubleClick,
			onRowRemoving: ConsultaAjaxDeleteReg,
			dataSource: DataSource,
			showRowLines: false,
			stateStoring: stateStoring,
			onToolbarPreparing: onToolbarPreparing
		}
	}
}