import template from './ConciliacaoBancaria.html';
import style from './ConciliacaoBancaria.scss';
import WidgetBase from '../../Widgets/Widget.base';
import FormManager from '../../../FormManager';
import { AbrirCadastroModal } from '../../../nGrid';
import { onSelectionLancamentos, onSelectionExtratos } from './dataGridOnSelection';

export const buildWidgets = () => ConciliacaoBancaria.trackChanges();

class ConciliacaoBancaria extends WidgetBase {
	static trackChanges() {
		let form = $('#frmTNCTBDPCONCILIACAOBANCARIA');
		let target = $('#Conteudo', form).exists();
		FormManager.byname("TNCTBDPCONCILIACAOBANCARIA").afterCreateGridConteudo.subscribe((nameGrid) => {
			if (nameGrid == 'dataGridConteudoCdsExtratoBancario') {
				ConciliacaoBancaria.setAdditionalGridOptions(target);

				let content = ConciliacaoBancaria.getElements(target);
				let buttons = ConciliacaoBancaria.getButtonsRow(target);
				let row = ConciliacaoBancaria.createRow(content);

				target
					.html('')
					.append([buttons, ConciliacaoBancaria.removeSizeClassFromCard(row)]);
			}
		})
	}

	static setAdditionalGridOptions(container) {
		const setSelectedRowTextColor = (datagridId) => {
			if (!$(`#${datagridId}Style`).exists()) {
				$('body').append(`
					<style id='${datagridId}Style'>
						#${datagridId} .dx-datagrid .dx-datagrid-rowsview .dx-selection.dx-row.conciliado>td {
							color: blue 
						}
					</style>
				`);
			}
		}
		let gridElLancamentos = $('#dataGridConteudoCdsLctosContabeis', container).exists();
		let gridElExtratos = $('#dataGridConteudoCdsExtratoBancario', container).exists();

		if (gridElExtratos) {
			var gridExtratos = dxDataGrid.getInstance(gridElExtratos);
		}

		if (gridElLancamentos) {
			var gridLancamentos = dxDataGrid.getInstance(gridElLancamentos);
		}

		if (gridElLancamentos) {
			let rowDblClick = (e) => {
				AbrirCadastroModal({
					_AClasse: 'TnCtbDMLctoCtb',
					AFiltros: rowDataValues(e, 'CODIGO;CODIGOEMPRESA', 'CHAVELCTOCTB;CODIGOEMPRESA'),
					AGrid: gridLancamentos,
					ANovo: false
				});
			};

			let originalOnRowPrepared = gridExtratos.option('onRowPrepared');

			setSelectedRowTextColor(gridElLancamentos.attr('id'))

			gridLancamentos.option('onRowDblClick', rowDblClick);
			gridLancamentos.option('selection.mode', 'multiple');
			gridLancamentos.option('selection.allowSelectAll', false);
			gridLancamentos.option('keyExpr', ['ID'])
			gridLancamentos.option('onSelectionChanged', onSelectionLancamentos(gridExtratos));
			gridLancamentos.option('onRowPrepared', function (rowInfo) {
				originalOnRowPrepared(rowInfo);

				if (rowInfo.rowType == "data") {
					if (rowInfo.data.CONCILIADO == "1") {
						rowInfo.rowElement.addClass('conciliado');
					}
				};
			})

			gridLancamentos.on('selectionChanged', ({ component, selectedRowsData }) => {
				component.option('selectedRowsQuestor', selectedRowsData);
			});
		}

		if (gridExtratos) {
			let originalOnRowPrepared = gridExtratos.option('onRowPrepared');

			setSelectedRowTextColor(gridElExtratos.attr('id'))

			gridExtratos.option('selection.mode', 'multiple');
			gridExtratos.option('selection.allowSelectAll', false);
			gridExtratos.option('keyExpr', ['ID'])
			gridExtratos.option('onSelectionChanged', onSelectionExtratos(gridLancamentos))
			gridExtratos.option('onRowPrepared', function (rowInfo) {
				originalOnRowPrepared(rowInfo);

				if (rowInfo.rowType == "data") {
					if (rowInfo.data.CONCILIADO == "1") {
						rowInfo.rowElement.addClass('conciliado');
					}
				};
			})

			gridExtratos.on('selectionChanged', ({ component, selectedRowsData }) => {
				component.option('selectedRowsQuestor', selectedRowsData)
			});
		}
	}

	static getElements(target) {
		let cards = $(target).find('.twMiniCardValueDisplayer-group');
		let [gridLancamentos, gridExtratos] = $(target).find('[id*=dataGridConteudo]');
		let [cardsLancamentos, cardsExtratos] = [cards.slice(0, 4), cards.slice(4)];

		return [
			{
				title: 'Extrato Bancário',
				cards: cardsExtratos,
				grid: gridExtratos
			},
			{
				title: 'Lançamentos Contábeis',
				cards: cardsLancamentos,
				grid: gridLancamentos
			}
		];
	}

	static getButtonsRow(target) {
		return $('.col-12', target).first();
	}

	static createRow(contentColumns) {
		let row = $('<div class="row">')

		contentColumns.forEach(column => row.append(ConciliacaoBancaria.createCol(column)));

		return row;
	}

	static createCol({ cards, grid, title }) {
		let wrapper = $(ConciliacaoBancaria.prototype.render({ template, style }));

		wrapper.find('.card-title').append(title);
		wrapper.find(`.${style['miniCardValueDisplayer-wrapper']}`).append(cards);
		wrapper.find('.card-body').append(grid);

		return wrapper;
	}

	static removeSizeClassFromCard(row) {
		row
			.find('.twMiniCardValueDisplayer')
			.removeClass(['col-lg-2', 'col-xl-1', 'col-sm-3'])
			.addClass('col-sm-3')

		return row;
	}
}