import FormManager from '../../FormManager';
import dxButton from 'devextreme/ui/button';
import { createdxDatasource, MontarGridCustomToolButtons, AddColumnButtonsEdicao } from '../../nGrid';
import { cxGridIncluirFiltroLookup } from '../../nLookUp';
import { evaluateGridConfig, prepareGridConfig, createGrid } from '../../gridConfigurator';
import { ButtonTypes } from '../../grid/ButtonTypes';

export function runEndPageScript(args) {
	if (FormManager.afterCreateAllForms.isStopped) ScriptConsulta.script(args);
	else FormManager.afterCreateAllForms.subscribe(() => ScriptConsulta.script(args));
}

class ScriptConsulta {
	static script({ scope }) {
		ScriptConsulta.handleFiltros(scope);
		ScriptConsulta.handleData(scope);

		PrepararMaskDefault();

		ScriptConsulta.handleGrid(scope);
	}

	static handleFiltros({ Filtros, Class } = {}) {
		if (Filtros) {
			FormManager.loadCadastro({_ACadastro: Class, _AAlias: '', _AFiltros: Filtros});
		}
	}

	static handleData({ data } = {}) {
		if (data) {
			CarregarDataCadastro(data);
		}
	}

	static handleGrid(scope) {
		if (scope.DefinicaoObjetos) {
			GridConsulta.buildGrid(scope);
		}
	}
}

class GridConsulta {
	static buildGrid({ Class, DefinicaoObjetos }) {
		const parseConsulta = (consulta) => { try { return JSON.parse(consulta) } catch { return {} } };
		
		let consulta = parseConsulta(DefinicaoObjetos.consulta);
		let configConsulta = GridConsulta.getConfigFromConsulta({ Class, DefinicaoObjetos, gridConfig: consulta });
		
		evaluateGridConfig(consulta);

		AddColumnButtonsEdicao(consulta.columns, Class, consulta.keyExpr.join(';'), consulta.DisableButtons);

		let config = prepareGridConfig(configConsulta, consulta);

		createGrid('dataGrid', config);
	}
	
	static getConfigFromConsulta({ Class, DefinicaoObjetos, gridConfig }) {
		return {
			form: {
				Class: DefinicaoObjetos.Class,
				Alias: DefinicaoObjetos.Alias
			},
			onRowRemoving: ConsultaAjaxDeleteReg,
			dataSource: GridConsulta.getDataSource(DefinicaoObjetos),
			stateStoring: GridConsulta.getStateStoring({ Class, DefinicaoObjetos }),
			onToolbarPreparing: GridConsulta.buildOnToolbarPreparing(DefinicaoObjetos, gridConfig)
		}
	}

	static getDataSource({ Class }) {
		return createdxDatasource({
			_AClassName: Class,
			_AFuncaoFiltroCX: cxGridIncluirFiltroLookup
		});
	}

	static getStateStoring({ Class, DefinicaoObjetos }) {
		let key = `${Class}${DefinicaoObjetos.consulta.nomegrid}`;

		return {
			enabled: true,
			type: 'custom',
			ignoreColumnOptionNames: [],
			customLoad: async () => await qGrid.loadGridState(key),
			customSave: (state) => qGrid.saveGridState(key, state) 
		}
	}

	static buildOnToolbarPreparing(DefinicaoObjetos, gridConfig) {
		return (e) => {
			let toolbarItems = e.toolbarOptions.items;
			
			MontarGridCustomToolButtons({
				Alias: DefinicaoObjetos.Alias,
				Class: DefinicaoObjetos.Class,
				gridComponent: e,
				gridToolbarItems: toolbarItems,
			});
			
			let toolButtons = GridConsulta.GetButtonsFromToolbarButtons(e, DefinicaoObjetos);
			let DefButtonsFirst = GridConsulta.GetDefaultToolButtons(e, DefinicaoObjetos, gridConfig);
			
			e.toolbarOptions.items = [...DefButtonsFirst, ...toolbarItems, ...toolButtons];
		}
	}

	static GetButtonsFromToolbarButtons(gridEvent, { ToolButtons } = {}) {
		let toolbarItems = [];

		ToolButtons.forEach((toolButton, index) => {
			if (index >= 2) {
				let { CleanCaption, Hint, NameEvent} = toolButton;

				toolbarItems.push({
					template: function (conf, index, el) {
						let btnConfig = {
							stylingMode: "contained",
							locateInMenu: "auto",
							showText: "inMenu",
							text: CleanCaption,
							hint: Hint,
							elementAttr: { "data-posttype": "0" },
							onClick: function () { window[NameEvent](this); }
						};
							
						(new dxButton('<div data-posttype="0">', btnConfig))
							.element()
							.addClass('btn btn-secondary text-white')
							.appendTo(el);
					},
					widget: "dxButton",
					location: "before"
				})
			}
		});

		return toolbarItems;
	}

	static GetDefaultToolButtons(gridEvent, DefinicaoObjetos, { DisableButtons = [] }) {
		return [
			GridConsulta.addbutton(DefinicaoObjetos, DisableButtons.includes(ButtonTypes.actDataSetInsert)),
			GridConsulta.refreshButton(),
			GridConsulta.exportButton(gridEvent),
			GridConsulta.gridOptionsButton(),
		];
	}

	static gridOptionsButton() {
		return {
			location: "after",
			widget: "dxDropDownButton",
			options: {
				dropDownOptions: {
					width: 140
				},
				displayExpr: "text",
				icon: "las la-th",
				hint: "Opções do grid",
				items: [
					{ text: "Restaurar colunas", action: "restaurar"},
					{ text: "Selecionar colunas", action: "selecionar"}
				],
				onItemClick(data) {
					if (data.itemData.action == 'restaurar'){
						dxDataGrid.getInstance($("#dataGrid")).state({});
					} else {
						dxDataGrid.getInstance($("#dataGrid")).showColumnChooser(); 
					}
				}
			}
		}
	}

	static refreshButton() {
		return {
			widget: "dxButton", 
			locateInMenu: "auto",
			options: { 
				icon: "refresh", 
				hint: "Atualizar", 
				onClick: () => dxDataGrid.getInstance($("#dataGrid")).refresh()
			},
			location: "after"
		}
	}

	static addbutton({ CadCons }, disabled) {
		return {
			disabled,
			widget: "dxButton", 
			locateInMenu: "auto",
			options: { 
				icon: "add", 
				hint: "Novo Registro", 
				elementAttr: {
					'data-block-page-on-click': '',
				},
				onClick: () => (window.location.href = `Cadastro?Scope={Class:"${CadCons}", new:true}`),
			},
			location: "after"
		}
	}

	static exportButton(gridEvent) {
		return {
			location: "after",
			widget: "dxDropDownButton",
			options: {
				dropDownOptions: {
					width: 140
				},
				displayExpr: "text",
				icon: "la la-download",
				hint: "Exportar",
				items: [
					{ text: "Exportar para Excel", action: "excel"},
					{ text: "Exportar para PDF", action: "pdf"}
				],
				onItemClick(data, ...args) {
					exporter.export(gridEvent, data.itemData.action);
				}
			}
		}
	}
}