import FormManager from "../FormManager";

export class FieldChooser {
	static init() {
		let fieldChooser = $('[data-id=fieldChooser]').exists();

		if (!fieldChooser) return;

		fieldChooser.each((index, el) => {
			if (!FieldChooser.createList($('ul', el))) {
		        $(el).remove();
			}
		})

		$('.dropdown-menu', fieldChooser).on('click', e => e.stopPropagation());
		$('input[type=checkbox]', fieldChooser).on('click', FieldChooser.onClick);
	}

	static createList(ul) {
		let { className, alias, fields, pages } = ul.data();
		
		if (pages) pages.map((page) => fields = fields.concat(page.Fields));
		
		if (!fields) fields = [];

		let hasFieldInChooser = fields.reduce((has, { Required, Lookup, Visible }) => (has || !Required && Visible && !Lookup), false);
	    if (!hasFieldInChooser) {
			return false;
		} 

		const setCheckedFieldsHidden = () => {
			fields.forEach(({ Name, Required, Visible, TabStop, Lookup }) => {
				if (!(Required || !Visible || Lookup)) {
					let form = FormManager.byname(className, alias);

					if (form) form.setvisible(Name, TabStop)
				}
			});
		}
		let newUl = $(`
			<ul class="kt-nav">
				${
					fields.map(({ DisplayLabel, Name, Required, Visible, TabStop, Lookup }) => {
						return (Required || !Visible || Lookup) ? '' 
							: `<li class="kt-nav__item">
									<a class="kt-nav__link">
										<div class="checkbox-wrapper">
											<label class="checkbox checkbox-outline checkbox-dark mb-2">
												<input type="checkbox" data-class-name="${className}" data-alias="${alias}" data-name="${Name}" ${TabStop ? '' : 'checked'}>
												<span></span>
												<div class="ml-2 mr-5" {{^nvValor}} style="font-size:14px!important;" {{/nvValor}} >${DisplayLabel}</div>
											</label>
										</div>
									</a>
								</li>`;
						}).join('')
				}
		`).css({
			'list-style': 'none',
			'padding': '2rem 1rem',
		});

		ul.replaceWith(newUl);
		
		setCheckedFieldsHidden();

		setTimeout(() => {
			KTUtil.scrollInit(newUl.get(0), {height: 300})
		});
		return true;
	}

	static async onClick() {
		let { name, className, alias } = this.dataset;
		let form = FormManager.byname(className);

		try {
			form.setvisible(name, !this.checked);

			await FieldChooser.save({
				p: {
					fieldName: name,
					className: className + alias,
					hidden: this.checked
				}
			});
		}
		catch (e) {
			qNotifier.error("Ocorreu um erro ao salvar o campo");
			form.setvisible(name, this.checked);
			this.checked = !this.checked;
		}
	}

	static async save(data) {
		return await axios.post('nHttpServicos/UpdateVisibleFields', data);
	}
}