import $ from 'jquery';

export const qGrid = (() => {
	var mergeValues = {};
	var mergeCells = {};			
			
    const exporter = (() => {

        const excel = e => {
            e.component.exportToExcel(true);
        };

        const pdf = e => {
            const rows = [];
            const widths = [];

            const columns = e.component.getVisibleColumns().filter(item => !item.command);
            const data = e.component.getSelectedRowsData();

            rows.push(columns.map(c => {
                return {
                    text: c.caption,
                    style: "tableHeader"
                };
            }));

            for (let i = 0; i < data.length; i++) {
                const j = i;

                rows.push($.map(columns,
                    d => {
                        let text = _.get(data[j], d.dataField);

                        if (typeof text === "boolean" || text instanceof Boolean) {
                            text = (text) ? "Ativo" : "Inativo";
                        }

                        // strip html from columns
                        if (typeof text === "string" || text instanceof String) {
                            text = text.replace(/<(?:.|\n)*?>/gm, "").trim();
                        }

                        return {
                            text,
                            alignment: d.alignment,
                            style: ((j % 2) > 0) ? "tableBodyEven" : "tableBodyOdd"
                        };
                    }));
            }

            $.map(columns, () => widths.push("auto"));

            const doc = {
                content: [
                    {
                        table: {
                            headerRows: 1,
                            widths,
                            body: rows
                        },
                        layout: "noBorders"
                    }
                ],
                styles: {
                    tableHeader: {
                        bold: true,
                        fontSize: 11,
                        color: "white",
                        fillColor: "#2d4154",
                        alignment: "center"
                    },
                    tableBodyEven: {},
                    tableBodyOdd: {
                        fillColor: "#f3f3f3"
                    },
                    tableFooter: {
                        bold: true,
                        fontSize: 11,
                        color: "white",
                        fillColor: "#2d4154"
                    },
                    title: {
                        alignment: "center",
                        fontSize: 15
                    },
                    message: {}
                },
                defaultStyle: {
                    fontSize: 10
                }
            };

            const fileName = e.component.option().EntityDescription;

            pdfMake.createPdf(doc).download(fileName);
        };

        return {
            export(e, action) {
                if (action !== "excel" && action !== "pdf")
                    return null;

                if (e.component.getSelectedRowKeys().length === 0) {
                    return qNotifier.warning("Selecione ao menos um registro para exportar.");
                }

                return (action === "excel") ? excel(e) : pdf(e);
            }
        };

    })();

    const showAction = data => {
        let title = "Visualizar Registro";

        const titleTag = $(data).find(".entity-title");
        if (titleTag.length > 0) {
            title = titleTag.text();
        }

        const dialog = bootbox.dialog({
            title: title,
            message: data,
            size: "large",
            className: "register-showing-modal",
            backdrop: true,
            onEscape: true,
            buttons: {
                cancel: {
                    label: "Fechar",
                    className: "btn-secondary"
                }
            }
        });

        dialog.removeAttr("tabindex");

        dialog.on("shown.bs.modal",
            () => {
                mApp.initComponents();
                qForm.initPlugins();
            });
    };

    const createShowButton = e => {
        const options = e.component.option();

        if (!options.AllowShowing)
            return;

        $("<a href=\"#\" class=\"dx-link dx-link-show dx-icon-search\"></a>")
            .appendTo(e.cellElement)
            .on("click", clickEvent => {
                clickEvent.preventDefault();

                qAjax.get(`${e.component.option("UrlShow")}?id=${e.key}`)
                    .then(response => showAction(response.data))
                    .catch(() => qNotifier.error("Falha a abrir a visualiza��o do registro."));
            });
    };

    const prepareDataForAction = (actionData, rowData) => {
        const returnData = {};

        for (let property in actionData) {
            if (actionData.hasOwnProperty(property)) {
                const value = actionData[property];
                const matches = value.match(/{(.*?)}/g);

                if (matches) {
                    if (matches.length === 1 && matches[0] === value) {
                        const dataProperty = matches[0].replace("{", "").replace("}", "");

                        if (rowData.hasOwnProperty(dataProperty)) {
                            returnData[property] = rowData[dataProperty];
                        }
                    } else {
                        matches.forEach(match => {
                            const dataProperty = match.replace("{", "").replace("}", "");

                            if (rowData.hasOwnProperty(dataProperty)) {

                                const valueToReplace = typeof rowData[dataProperty] === "object"
                                    ? JSON.stringify(rowData[dataProperty])
                                    : rowData[dataProperty];

                                returnData[property] = actionData[property].replace(match, valueToReplace);
                            }
                        });
                    }
                }
            }
        }

        return returnData;
    };

    const executeAction = (action, data) => {
        if (!action.Ajax) {
            const params = $.param(data, true);

            const url = `${action.Url}?${params}`;

            if (action.TargetBlank) {
                window.open(url, "_blank");
            } else {
                window.location = url;
            }

            return;
        }

        $.ajax({
            method: action.Method.toLowerCase(),
            url: action.Url,
            data
        }).done(response => {
            const successCallback = eval(action.OnSuccessCallback);

            if (successCallback) {
                successCallback(response);
            } else {
                qAjaxHandler.onSuccess(response);
            }
        }).fail(error => {
            const failureCallback = eval(action.OnFailureCallback);

            if (failureCallback) {
                failureCallback(response);
            } else {
                qAjaxHandler.onFailure(error);
            }
        });
    };

    const executeMassAction = (action, selectedRows) => {
        const getDataCallback = eval(action.BeforeExecute);

        Promise.resolve((getDataCallback) ? getDataCallback(selectedRows) : {})
            .then(requestData => {
                const data = {
                    ids: selectedRows,
                    data: requestData
                };

                executeAction(action, data);
            })
            .catch(() => {
                // ignored
            });

        return true;
    };

    const createContextActionButtons = e => {

        const contextActions = e.component.option().ContextActions;

        if (contextActions && contextActions.length) {

            const actions = [
                {
                    text: "A��es",
                    disabled: true
                }
            ];

            contextActions.forEach(action => {
                actions.push({
                    text: action.Title,
                    action
                });
            });

            $("<div>").appendTo(e.cellElement).dxMenu({
                items: [
                    {
                        icon: "preferences",
                        items: actions
                    }
                ],
                showFirstSubmenuMode: "onClick",
                hideSubmenuOnMouseLeave: true,
                onItemClick({ itemData }) {
                    if (itemData.items)
                        return;

                    const data = prepareDataForAction(itemData.action.Data, e.data);

                    executeAction(itemData.action, data);
                }
            });
        }

    };
	
    const createMassActions = (e, toolbarItems) => {

        const massActions = e.component.option().MassActions;

        if (massActions && massActions.length) {

            toolbarItems.unshift({
                location: "after",
                widget: "dxButton",
                options: {
                    type: "success",
                    icon: "fa fa-play",
                    hint: "Executar a��o",
                    onClick() {
                        const action = $(e.element).data("selected-mass-action");

                        if (!action) {
                            return qNotifier.warning("Selecione uma a��o para ser executada.");
                        }

                        const selectedRows = e.component.getSelectedRowKeys();

                        if (action.NeedsSelect && !selectedRows.length) {
                            return qNotifier.warning("Selecione ao menos um registro para executar a a��o.");
                        }

                        return executeMassAction(action, selectedRows);
                    }
                }
            });

            toolbarItems.unshift({
                location: "after",
                widget: "dxSelectBox",
                options: {
                    dataSource: massActions,
                    displayExpr: "Title",
                    valueExpr: "Title",
                    placeholder: "Selecione uma a��o...",
                    onSelectionChanged(event) {
                        $(e.element).data("selected-mass-action", event.selectedItem);
                    }
                }
            });
        }
    };

    return {
        init() {

            $('a[data-toggle="tab"]').on("shown.bs.tab", () => {
                window.dispatchEvent(new Event("resize"));
            });

        },

        onCellPrepared(e) {
            e.component._options.noDataText = (e.component.pageSize() <= 0)
                ? "Carregue os registros e utilize os filtros para visualizar os itens desejados"
                : "Não existem dados";

            const options = e.component.option();

            if (e.column.noBorder) {
                $(e.cellElement).css("border", "none");
            }

            if (e.rowType == "header" && e.column.command == "edit") {  
                $(e.cellElement).text(e.column.caption || "");
            } 

            if (e.rowType === "data" && e.column.command === "edit") {
                const $links = $(e.cellElement).find(".dx-link");
                if (!options.AllowUpdating)
                    $links.filter(".dx-icon .las .la-pen").remove();

                createShowButton(e);
                createContextActionButtons(e);

                $links.filter(".dx-icon-save").addClass("dx-link-save")
                    .attr("title", $links.filter(".dx-link-save").text());

                $links.filter(".dx-icon-revert").addClass("dx-link-cancel")
                    .attr("title", $links.filter(".dx-link-cancel").text());

                $links.filter(".dx-icon-edit").addClass("dx-icon las la-pen")
                    .attr("title", $links.filter(".dx-icon .las .la-pen").text());

                $links.filter(".dx-icon-trash").addClass("dx-link-delete las la-trash")
                    .attr("title", $links.filter(".dx-link-delete").text());


                $links.text("");
            }
			
			if (isNaN(e.rowIndex))  
				return;  
		  
			if (e.column.mergeCell && e.rowType == 'data') {  
				if (e.rowIndex == 0 || mergeValues[e.columnIndex] != e.value) {  
					mergeValues[e.columnIndex] = e.value;  
					$(e.cellElement).css("vertical-align", "middle");
					mergeCells[e.columnIndex] = $(e.cellElement);
					$(mergeCells[e.columnIndex]).attr("rowspan", 1);
				} else {  
					if (!$(mergeCells[e.columnIndex]).attr("rowspan")){
						$(mergeCells[e.columnIndex]).attr("rowspan", 1);
					}
					else{
					  var rowsp = $(mergeCells[e.columnIndex]).attr("rowspan");
					  rowsp = parseInt(rowsp)+1;
						$(mergeCells[e.columnIndex]).attr("rowspan", rowsp);
					}
					$(e.cellElement).attr("remove", "true");              
                } 
            }
        },
        onInitNewRow(e) {
            const insertingCallback = e.component.option().InsertingCallback;

            if (insertingCallback && eval(insertingCallback)) {
                eval(insertingCallback)();

                return;
            }

            const urlParams = new URLSearchParams(window.location.search);
            urlParams.delete("id");

            window.location = `${e.component.option("UrlAdd")}?${urlParams.toString()}`;
        },

        onEditingStart(e) {
            e.cancel = true;

            const editingCallback = e.component.option().EditingCallback;

            if (editingCallback && eval(editingCallback)) {
                eval(editingCallback)(e.key);

                return;
            }

            const urlParams = new URLSearchParams(window.location.search);
            urlParams.delete("id");
            urlParams.append("id", e.key);

            window.location = `${e.component.option("UrlEdit")}?${urlParams.toString()}`;
        },

        onRowRemoving(e) {
            e.cancel = true;

            qAjax.post(`${e.component.option("UrlRemove")}?id=${e.key}`)
                .then(response => {
                    qAjaxHandler.onSuccess(response.data);
                })
                .catch(() => {
                    qAjaxHandler.onFailure();
                });
        },

        onToolbarPreparing(e) {
            const dtGrid = e.component;

            const toolbarItems = e.toolbarOptions.items;

            toolbarItems.unshift({
                location: "after",
                widget: "dxButton",
                options: {
                    icon: "refresh",
                    hint: "Carregar registros",
                    onClick() {
                        dtGrid.option("paging.pageSize", 10);
                        dtGrid.refresh();
                    }
                }
            });

            toolbarItems.unshift({
                location: "after",
                widget: "dxButton",
                options: {
                    icon: "revert",
                    hint: "Restaurar",
                    onClick() {
                        dtGrid.state({});
                    }
                }
            });

            toolbarItems.unshift({
                location: "after",
                widget: "dxMenu",
                options: {
                    hint: "Exportar",
                    items: [
                        {
                            icon: "la la-download",
                            items: [
                                { text: "Exportar para Excel", action: "excel" },
                                { text: "Exportar para PDF", action: "pdf" }
                            ]
                        }
                    ],
                    onItemClick(data) {
                        exporter.export(e, data.itemData.action);
                    }
                }
            });

            createMassActions(e, toolbarItems);
        },

        onContentReady(e) {
            mergeValues = {};//limpando
            mergeCells = {};//limpando
            var celulas = $($(e.element)).find("td");
            for (var i = 0; i < celulas.length; i++) {
                if ($(celulas[i]).attr('remove')) {
                    $(celulas[i]).remove();
                }
            }
            e.component.updateDimensions();
        },

        onExporting(e) {
            e.fileName = e.component.option().EntityDescription;
            $(e.element).data("original-filters", e.component.getCombinedFilter());
            e.component.clearFilter();
        },

        onExported(e) {
            e.component.filter($(e.element).data("original-filters"));
            $(e.element).removeData("original-filters");
        },

        customizeExportData(_, rows) {
            rows.forEach(row => {
                for (let i = 0; i < row.values.length; i++) {
                    if (typeof row.values[i] === "string" || row.values[i] instanceof String) {
                        row.values[i] = row.values[i].replace(/<(?:.|\n)*?>/gm, "").trim();
                    }
                }
            });
        },

        async loadGridState(gridName) {
						const deferred = new $.Deferred;

						let gridEl = $(`[data-class-nomegrid=${gridName}]`);

						const instance = gridEl.length ? dxDataGrid.getInstance(gridEl) : null;

            $.ajax({
                url: "nHttpServicos/GetGridConfig?_ANome=" + gridName,
                method: "post",
                dataType: "json",  
                cache: false,              
                success(data) {
									deferred.resolve(data.Result);
									if (instance && instance.option('onStateReady'))
										instance.option('onStateReady').next();												
                },
                fail() {
                    console.log('GetGridConfig - erro');
                    deferred.reject();
                }
            });

            return deferred.promise();
        },

        saveGridState(gridName, gridState) {
            gridState.columns.forEach(column => {
                delete column.filterValue;
                delete column.selectedFilterOperation;
            });
            delete gridState.filterValue;
            delete gridState.searchText;
            delete gridState.selectedRowKeys;
            return $.ajax({
                 url: "nHttpServicos/UpdateGridConfig?_ANome=" + gridName + '&_AConfig='+ JSON.stringify(gridState),
                 method: "post",
                 dataType: "json",  
                 cache: false,              
                 fail() {
                     toastr.error("Falha ao salvar o estado da grid no perfil.");
                 }
            });
        },

        calculateFilterExpression() {
            this.dataField = this.calculateSortValue;

            return this.defaultCalculateFilterExpression(...arguments);
        },

        onEditorPreparing(e) {
            if (e.parentType === "filterRow") {
                e.editorOptions.onEnterKey = () => {
                    $(e.element).find(".dx-apply-button").trigger("dxclick");
                };
            }
        },

        onDeleteSuccess(e) {
            if (!e) {
                qNotifier.error("Ocorreu um erro ao remover o registro, entre em contato com a administra��o.");
                return;
            }

            if (e.url) {
                window.location.href = e.url;
                return;
            }

            if (e.mensagem) {
                (e.sucesso)
                    ? qNotifier.success(e.mensagem)
                    : qNotifier.error(e.mensagem);
            } else {
                qNotifier.success("O registro foi removido com sucesso.");
            }

            qGrid.refreshAllGrids();
        },

        findAllGrids() {
            return $.map($(".dx-widget"), item => $(item).data("dxDataGrid")).filter(item => item);
        },

        refreshAllGrids() {
            const $grids = $("[id*='grid_']");

            $grids.each((_, grid) => {
                if ($(grid).data("dxDataGrid") !== undefined) {
                    dxDataGrid.getInstance($(grid)).refresh();
                }
            });
        },

        loadDetail(container, info) {
            container.html(`Carregando...`);

            container.resize(() => {
                info.component.updateDimensions();
            });

            const url = `${info.component.option("UrlDetail")}?id=${info.data.id}`;

            $.get(url)
                .done(response => {
                    container.html(response);
                })
                .fail(() => {
                    qNotifier.error("Falha ao buscar detalhamentos do registro.");
                });
        }
    };

})();