import { formatWithDataType } from "../nGrid";

export class GridExpressions {
	static onChange({ gridComponent, editorField, data, rowIndex, expressionExecutor }) {
		const isEmpty = (val) => [undefined, null, ''].includes(val);

		return async (e) => {
			let { component: editorComponent, event, previousValue, value, element } = e;

			if (isEmpty(value) && isEmpty(previousValue)) return;	/// dont recompute expression for changes on values ['', null, undefined]. Ex: DateBox onChange event fired for values null or ''

			try {
				let rowData = GridExpressions.formatData({ data, editorField, gridComponent, editorComponent, expressionExecutor });

				for (let { dataField, Expressoes, value } of rowData) {
					if (Expressoes) {
						if ((!value || Expressoes.ForcarValor) && Expressoes.ExpressaoValue) {
							let expressionValue = await expressionExecutor.execute(Expressoes.ExpressaoValue);

							gridComponent.cellValue(rowIndex, dataField, expressionValue);
						}
						if (Expressoes.ExpressaoClear) {
							let expressionValue = await expressionExecutor.execute(Expressoes.ExpressaoClear);

							if (expressionValue) gridComponent.cellValue(rowIndex, dataField, '');
						}
					}
				}

				setTimeout(() => { // prevent conflict error with jquery async focus eventHandler on method leverageNative of metronic6 vendors.bundle.js
					gridComponent.cellValue(rowIndex, editorField, editorComponent.option('value'));
				})
			}
			finally {
				KTApp.unblockPage();
			}
		}
	}
	
	static formatData({ data, editorField, gridComponent, editorComponent, expressionExecutor }) {
		return Object
			.entries(data)
			.map(([key, value]) => {
				let { Expressoes, dataField, dataType } = gridComponent.columnOption(key);
				let formatedValue = { [key]: value };

				formatWithDataType(dataType, formatedValue, key, value);

				let valueForExpression = editorField === dataField ? editorComponent.option('value') : formatedValue[key];

				expressionExecutor.registrarValor(dataField, valueForExpression);

				return { dataField, Expressoes, value: valueForExpression };
			});
	}

   static onHandleSelection({gridComponent, editorField, data, rowIndex, expressionExecutor, valueField}){
		return async (e) => {
			try {
				let rowData = GridExpressions.formatDataHandleSelection({ data, editorField, gridComponent, valueField, expressionExecutor});

				for (let { dataField, Expressoes, value } of rowData) {
					if (Expressoes) {
						if ((!value || Expressoes.ForcarValor) && Expressoes.ExpressaoValue) {
							let expressionValue = await expressionExecutor.execute(Expressoes.ExpressaoValue);

							gridComponent.cellValue(rowIndex, dataField, expressionValue);
						}
						if (Expressoes.ExpressaoClear) {
							let expressionValue = await expressionExecutor.execute(Expressoes.ExpressaoClear);

							if (expressionValue) gridComponent.cellValue(rowIndex, dataField, '');
						}
					}
				}

			}
			finally {
				KTApp.unblockPage();
			}
		}
	};

	static formatDataHandleSelection({ data, editorField, gridComponent, valueField, expressionExecutor }) {
		return Object
			.entries(data)
			.map(([key, value]) => {
				let { Expressoes, dataField, dataType } = gridComponent.columnOption(key);
				let formatedValue = { [key]: value };

				formatWithDataType(dataType, formatedValue, key, value);

				let valueForExpression = editorField === dataField ? valueField : formatedValue[key];

				expressionExecutor.registrarValor(dataField, valueForExpression);

				return { dataField, Expressoes, value: valueForExpression };
			});
	}
}