import { CarregarTreeList } from "../../../treeview/treeview";

export function handleResponseBox(grid, { MsgTreeLista, MsgLista } = {}) {
	const getComponent = (grid) => {
		try {
			return typeof grid === 'string' ? dxDataGrid.getInstance(`#dataGridConteudo${grid}`) : grid;
		}
		catch {
			return null;
		}
	};

	let gridComponent = getComponent(grid); 


	if (gridComponent) {
		handleTreeViewResponseBox(gridComponent, MsgTreeLista);
		handleListResponseBox(gridComponent, MsgLista);
	}
}

function handleTreeViewResponseBox(gridComponent, treeview) {
	let gridEl = gridComponent.element();
	let responseActionEl = gridEl.parent().find('div[data-action-response]').exists();
	let { Tree } = (treeview || {});

	if (responseActionEl) {
		responseActionEl.jstree('destroy');

		if (!treeview) {
			responseActionEl.closest('div[data-treeview-wrapper]').remove();

			return;
		}
		
		responseActionEl.data('tree-content', Tree);
		CarregarTreeList(null, responseActionEl);
	}
	else {
		if (!treeview) return;
		
		let treeviewWrapper = $(
			`<div class="row" data-treeview-wrapper="true">
				<div class="card card-custom kt-portlet--bordered kt-portlet--rounded kt-portlet--unair kt-portlet--head-sm mt-4">
					<div class="card-header">
						<h3 class="h2 card-title">
							<span class="mr-2">
								<i class="fa fa-tag">
								</i>
							</span>
							Resultados
						</h3>
					</div>
					<div class="card-body">
						<div data-action-response="true"></div>
					</div>
				</div>
			</div>`
		);
		let treeviewEl = treeviewWrapper.find('div[data-action-response');

		treeviewEl.data('tree-content', Tree);

		gridEl.append(treeviewWrapper);

		CarregarTreeList(null, treeviewEl);
	}
}

function handleListResponseBox(gridComponent, MsgLista) {
	const createTh = (Columns) => Columns.map((col) => `<th>${col}</th>`).join('');
	const createTd = (tr) => tr.map((td) => `<td>${td}</td>`).join('');
	const createTrBody = (Items) => Items.map((tr) => `<tr> ${createTd(tr)} </tr>`).join('');
	const createTemplate = ({ Columns, Items }) => $(
		`<table data-action-response="true" class="display table kt-table kt-table--head-no-border">
			<thead> <tr> ${createTh(Columns)} </tr> </thead>	  
			<tbody> ${createTrBody(Items)} </tbody>
		</table>`
	);
	const initTable = (elToAppend, { Columns, Items, datatablesettings }, { replace } = {}) => {
		let template = createTemplate({ Columns, Items });

		replace ? elToAppend.replaceWith(template) : elToAppend.append(template);

		if (datatablesettings) template.DataTable(datatablesettings);
	}

	let gridEl = gridComponent.element();
	let actionReponseEl = gridEl.parent().find('table[data-action-response]').exists();

	if (actionReponseEl) {
		if (!MsgLista) {
			actionReponseEl.DataTable().destroy();
			actionReponseEl.remove();

			return;
		}

		initTable(actionReponseEl, MsgLista, { replace: true });
	}
	else {
		if (!MsgLista) return;

		initTable(gridEl, MsgLista);
	}
}