import dxMenu from 'devextreme/ui/menu'; 
import { DateConfigurator } from './DateConfigurator';
import { createGridLookup } from './GridLookup';

export function buttonCellTemplateRef(el, options) {
	const getDataEvent = (options, event) => ({
		column: options.column,
		component: options.component,
		element: options.component.$element(),
		event: event,
		row: options.row
	});

	const isVisible = (btn) => {
		if (typeof btn.visible == "function") {
			return btn.visible(options);
		}

		return btn.visible;
	}

	/**
	 * Its necessary to return the sum of hidden and visible buttons based on btn.<visible> property, 
	 * which could used the condExpressions to hide/show the button depending on the value of another column.
	 */
	const createVisibleButtons = () => {
		let notHiddenButtonCount = options.column.buttons
			.filter(i => !i.inDropDown && isVisible(i))
			.map(btn => {
				let btnContainer;

				if (btn.icon) btnContainer = $(`<img src=${btn.icon} class="dx-icon">`);
				else btnContainer = $('<div class="dx-link" style="background-color: red">')

				btnContainer.on('click', function (event) {
					btn.onClick.call(this, getDataEvent(options, event));
				})
					.appendTo(el);
			})
			.length;

		let hiddenButtonCount = options.column.buttons
			.filter(i => !i.inDropDown && !isVisible(i))
			.length;

		return notHiddenButtonCount + hiddenButtonCount; 
	}

	const createInDropDownButtons = () => {
		let menuItem = [];

		options.column.buttons
			.filter(i => i.inDropDown && isVisible(i))
			.map(btn => {
				menuItem.push({
					onClick: function (e) {
						btn.onClick.call(this, getDataEvent(options, event));
					},
					text: btn.text
				})
			});

		if (menuItem.length) {
			(new dxMenu('<div>', {
				items: [{
					icon: "overflow",
					items: menuItem
				}],
				elementAttr: { style: "vertical-align: middle;" },
				showFirstSubmenuMode: 'onClick',
				hideSubmenuOnMouseLeave: true,
			}))
			.element()
			.appendTo(el);
		}
		return menuItem.length;
	}

	const createButtons = () => {
		let visibleButtons = createVisibleButtons();
		let inDropDownButtons = createInDropDownButtons();
		let buttonsWidth = (inDropDownButtons ? 50 : 0) + (20 * visibleButtons);

		options.component.columnOption(options.column.caption, 'width', Math.max(50, buttonsWidth));
		el.css("text-align", "center");
	}

	createButtons();
}

export function getEditCellTemplate(configColumn) {
	const isLookup = ({ Lookup, LinkLookupDataSetNome } = {}) => (!Lookup && ![undefined, null, '', []].includes(LinkLookupDataSetNome));
	const { Lookup } = configColumn;

	if (isLookup(Lookup)) {
		return createGridLookup();
	}
	else {
		return null;
	}
}

export function configureFormat(column) {
	if (column.dataType === 'date') {
		column.format = function (value) {
			return DateConfigurator.valueToDisplay(column, value);
		};
	}
	else if (column.dataType === 'string') {
		if (column.mask) {
			column.calculateDisplayValue = function (rowData) {
				let value = rowData[column.dataField];

				return maskedValueToDisplay(column, value);
			}
		}
	}
}

function maskedValueToDisplay(column, value) {
	let { mask, maskreverse } = column;

	if (mask) {
		let input = $(`<input value="${value}" />`);

		input.mask(mask, {
			reverse: maskreverse
		});

		return input.val();
	}

	return value;
}