import $ from 'jquery';
import { QuickSearch } from './quickSearch';
import { pesquisarModulos } from './searchModulos';
import FormManager from './FormManager';
import moment from 'moment';
import { DynamicImport } from './Utils/DynamicImport';
import { CarregarTreeList } from './treeview/treeview';
import { togglePortletFullscreen } from './fullscreen';
import { FieldChooser } from './form/FieldChooser';
import { createContextMenu } from './ContextMenu';
import { qApp } from './app';


export default class Initializer {
	static init() {
		$(Initializer._initializerWrapper);
	}

	static _initializerWrapper() {

		Initializer._initMenuEvents();

		Initializer._initSuporteOnline();

		Initializer._initQuickSearch();

		Initializer._initSearchModulos();

		Initializer._initSubheaderDate();

		Initializer._initVerticalTabs();

		Initializer._initSweetAlert();
		
		Initializer._initQApp();

		Initializer._createForms();

		Initializer._initPosicionarWizards();

		Initializer._initClassActive();

		Initializer._initTreeview();

		Initializer._initFieldChooser();

		Initializer._initContextMenuChecklist();

		Initializer._initDynamicDependency();

		Initializer._initBlockPageEvents();

		Initializer._initCustomFileInput(); // Deixar por último para executar os handles para arquivos na ordem correta
	}

	static _initMenuEvents() {		

		function abrirMenu() {
			$('#main-menu-search').val('');
			$("li.menu-item.menu-item-submenu.searchable").removeClass("menu-item-open");
			$("li.menu-item.searchable.menu-item[data-menu-search-only]").removeClass("menu-item-open").addClass("d-none");
			if ($('body').hasClass('aside-minimize-hover')) {
				$('body').removeClass('aside-minimize-hover');
				$('body').addClass('aside-minimize');
			} else {
				$('body').removeClass('aside-minimize');
				$('body').addClass('aside-minimize-hover');
			};
		}

		$('.toggle-menu-button').on('click', function () {
			abrirMenu();	
		})

		$('.aplicacoes-menu-button').on('click', function () {
			if ($('body').hasClass('aside-minimize-hover')) {
				$(".aplicacoes-menu").addClass("menu-item-open");
			}
			abrirMenu();
		})

		$('.favoritos-menu-button').on('click', function () {
			if ($('body').hasClass('aside-minimize-hover')) {
				$(".favoritos-menu").addClass("menu-item-open");
			}
			abrirMenu();
		})
	}

	static _initSuporteOnline() {
		let suporteOnlineEl = $('#suporte-online').exists();
		if (suporteOnlineEl) {				
			suporteOnlineEl.click(function(e) {
				KTApp.blockPage();
				axios.get('nHttpServicos/SuporteOnline')
					.then(
						function(response) {
							KTApp.unblockPage();
							const suporteOnline = response.data.Result;
							if (suporteOnline.HabilitarSuporteOnline) {
								if (suporteOnline.Error) {
									qNotifier.error(suporteOnline.Error, 'Suporte Online');
								} else {
									window.open(suporteOnline.URL);
								}
							}	else {
								qNotifier.warning('Movidesk não está habilitado!', 'Suporte Online');
							}
						});
			});
		} 
	}

	static _initSearchModulos() {				
		$('#search-modulos').on('keyup', (e) => pesquisarModulos());
	}

	static _initQuickSearch() {
		if (KTUtil.getById('kt_quick_search_d')) {
			QuickSearch().init(KTUtil.getById('kt_quick_search_d'));
		}
	}

	static _initSubheaderDate() {
		let subheader = $('#todaySubheader').exists();

		if (subheader) {
			moment.locale('pt-br');
			let now = moment().format('dddd, DD [de] MMMM [de] YYYY');
			let user = subheader.data('user');

			subheader.html(`<span class="font-size-sm">${user.charAt(0).toUpperCase() + user.slice(1)}, ${now.toLowerCase()}. </span>`);
		}
	}

	static _initVerticalTabs() {
		Initializer._onShowBsTabs();
		Initializer._afterShowBsTabs();
	}

	static _onShowBsTabs() {
		$('.kt-wizard__head a[data-toggle="tab"]').on('show.bs.tab', function (e) {
			let dataId = $(e.target).closest('.nav-item').data('id');

			$('#kt_content [data-id="tabitens"]').find('.active,.kt-nav__item--active').removeClass('active kt-nav__item--active show');
			$(`#kt_content [data-id="${dataId}"]`).addClass('kt-nav__item--active show')
		});
	}

	static _afterShowBsTabs() {
		const updateTabMenuHeight = (elem) => {
			let href = $(elem || '[data-id="tabitens"] li:first-child a').attr('href');
			let closest = $(elem).closest('.kt-scroll');
			let maxHeight = $('#kt_aside_menu').height() - ($('#kt_header').height() + 55) - 30;

			closest.css('max-height', $(href).height() > maxHeight ? maxHeight : '500px');
		}

		$('[data-id="tabitens"] a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
			const jQdata = prop => form.data(prop);
			var href = $(this).attr('href');
			var pHeadElem = `${href} .card-header`;
			var form = $(`${href} form`);

			$('#kt_content [data-duplicate-portlet-page="true"]').hide();
			$(href).closest('#kt_content [data-duplicate-portlet-page="true"]').show();

			if ($(pHeadElem).length) qUtils.scrollTo(pHeadElem);

			updateTabMenuHeight(this);
		});

		updateTabMenuHeight();
	}

	static _initSweetAlert() {
		window.swal = swal.mixin({
			width: 400,
			heightAuto: false,
			padding: "2.5rem",
			buttonsStyling: false,
			confirmButtonClass: "btn btn-success m-btn m-btn--custom",
			confirmButtonColor: null,
			cancelButtonClass: "btn btn-outline-dark m-btn m-btn--custom",
			cancelButtonColor: null,
			confirmButtonText: 'Sim',
			cancelButtonText: 'Não'
		});
	}

	static _createForms() {
		FormManager.createAllForms();
		setarevento();
	}

	static _initPosicionarWizards() {
		//Carregando treeLists
		$('.data-tree').each(function (index, element) {
			CarregarTreeList(element.id);
		});
		//Posicionando wizards
		$('[data-wizard="true"]').each((i, e) => {
			let startStep = $(e).find('[data-wizard-state="current"]').attr("data-step-index");
			new KTWizard(e, { startStep: startStep || 1 });
			VoltarEnabled(e.dataset['progressposition'] > 2);
			AvancarEnabled(e.dataset['progressposition'] != 100);
		});

		$('[data-fullscreen="true"').on('click', (el) => togglePortletFullscreen(el.delegateTarget));
	}

	static _initClassActive() {
		$(".twMenu").on("click", function () {
			$(".twMenu").removeClass("active");
			this.classList.add("active");
		})
	}

	static _initTreeview() {
		CarregarTreeList();
	}

	static _initContextMenuChecklist() {
		createContextMenu();
	}
	
	static _initDynamicDependency() {
		let dependencies = Initializer._searchDependencies();

		Initializer._requestDynamicDependency(dependencies);
	}

	static _searchDependencies(context) {
		const extractData = ({ moduleName, importName, execute, ...params }) => ({ moduleName, importName, execute, params });
		const convertData = (index, el) => extractData($(el).remove().data());

		return $('[data-dynamic-dependency="true"]', context).map(convertData).toArray();
	}

	static async _requestDynamicDependency(dependencies = []) {
		let promises = dependencies.map(({ moduleName, importName, execute, params }) => {
			return execute ?
				DynamicImport.execute(DynamicImport.Import(moduleName, importName), params) :
				DynamicImport.Import(moduleName, importName);
		});

		Promise
			.all(promises)
			.catch((e) => console.error(e));
	}

	static _initCustomFileInput() {
		bsCustomFileInput.init();
	}

	static _initFieldChooser() {
		FieldChooser.init()
	}

	static _initQApp() {
		qApp.init();
	}

	static _initBlockPageEvents() {
		$(document).on('click', '[data-block-page-on-click]', ({ target, ctrlKey, shiftKey }) => {
			const isLink = target.tagName.match(/a/i) || target.closest('a');
			
			if (isLink && (ctrlKey || shiftKey)) return;

			KTApp.blockPage()
			console.log('blockPage Event', event)
			
		});
	}
}