import dxAutocomplete from 'devextreme/ui/autocomplete';
import FormManager from './FormManager';

export class lookupAlphaNumeric {
	constructor(config) {
		this._form = config.form;
		this._createComponent(config.element);
	}

	_createComponent(element) {
		this._options = this._extractOptions(element);
		this._value = null;

		let config = {
			dataSource: this._buildDataSource(this._options),
			inputAttr: this._generateInputAttr(this._options),
			searchExpr: this._options.searchExpr.split(';'),
			...this._handlersConfig(this._options)
		}

		this._autocompleteInstance = this._buildAutocomplete(element, config);
		this._configureFirstLoad();
	}

	_extractOptions(jQElement) {
		let jQData = prop => jQElement.data(prop);

		return {
			alias: jQData('alias'),
			displayExpr: jQData('linklookupdescshowfields'),			
			searchExpr: jQData('linklookupdescfindfield'),
			searchClassName: jQData('linklookupdatasetnome'),
			dmClassName: jQData('dmclassname'),
			linkkeyfields: jQData('linkkeyfields'),
			fieldName: jQData('fieldname'),
			isMultiColumn: jQData('ismulticolumn'),
			linklookupresultfield: jQData('linklookupresultfield'),
			linklookupkeyfields: jQData('linklookupkeyfields'),
			initialValue: jQData('initialvalue'),
			linklookupFieldOrdination: jQData('linklookupfieldordination')
		}
	}
	
	_buildDataSource(options) {
		let that = this;
		function getSort() {
			if (options.linklookupFieldOrdination !== '') {
				return [{ "selector": options.linklookupFieldOrdination, "desc": false }];
			} else {
				return '';
			};
		}

		const requestConfig = (filter) =>
			[
				'nHttpServicos/DadosConsulta2',
				{
					_AClass: options.searchClassName,
					filter: JSON.stringify(filter),
					sort: JSON.stringify(getSort()),
					skip: 0,
					take: 10
				}
			];
		
		const uniqBy = function(array, fieldname) {
			let set = new Set();
			return array.filter(item => fieldname && !set.has(item[fieldname]) && set.add(item[fieldname]));
		}

		return {
			load: function () {
				var d = new $.Deferred();
				let params = that._createFilter(options);
				let promises = [];

				for (let param of params) {
					promises.push($.getJSON(...requestConfig(param)));
				}
				promises.reverse();
				Promise.all(promises)
					.then((responses) => {
						let result = [];
						
						responses = responses
							.map(r => (r.data.items || []))
							.reduce((arrayBuild, currentItem) => arrayBuild.concat(...currentItem), []);
						
						result = uniqBy(responses, options.linklookupkeyfields.split(';').pop());

						d.resolve(result);
					})
				return d.promise();
			}
		}
	}

	_createFilter(options) {
		let that = this;
		let form = this._form;
		let linkkeyfields = options.linkkeyfields.split(';');
		let linkLookupKeyFields = options.linklookupkeyfields.split(';');
		let fieldNameLookupKeyField = linkLookupKeyFields.pop();
		let fieldNameLookupfield = linkkeyfields.pop();
		let codeSearch = [fieldNameLookupKeyField, '=', this._value];
		let searchFields = options.searchExpr.split(';');
		let searchArray = [];

		if (linkLookupKeyFields.length) {
			linkLookupKeyFields.forEach((keyField, index) => {
				let value = form.getvalue(keyField) || form.getvalue("P" + keyField) || form.getvalue(linkkeyfields[index]) || form.getvalue("P" + linkkeyfields[index]);

				searchFields.forEach(searchField => { 	
					searchArray.push([[keyField, '=', value], 'and', [searchField, 'contains', that._value]]);
				});
			});
		}
		else {
			searchFields.forEach((searchField) => {
				searchArray.push([searchField, 'contains', that._value])
			});
		}

		return [codeSearch, ...searchArray];
	}

	_generateInputAttr(options) {
		return {
			style: "padding: 0; font-family: poppins; font-size: 1rem;",
			class: "cad-control cad-logchange",
			"data-classname": options.dmClassName,
			"data-alias": options.alias
		}
	}

	_handlersConfig(options) {
		const that = this
		const onItemClick = function (data) {
			if (data.itemData) {
				const fieldName = options.linklookupkeyfields.split(';').pop();
				const lookupDesc = data.itemData[that._options.linklookupresultfield.split(';').pop()].replace(/&nbsp/g, ' ')
				const fieldComponent = data.element.siblings('#ctrl' + options.fieldName);

				if (!options.isMultiColumn) {
					$("#ctrl" + options.fieldName + 'RESULTTEXT').text(lookupDesc);
				}

				that._value = (data.itemData[fieldName] || data.itemData[options.fieldName] || null);

				fieldComponent
					.val(that._value)
					.data({
						lookup: lookupDesc,
						value: that._value,
					});
			}
		} 

		const valueExpr = function(item) {
			if (item && options.isMultiColumn) {
				return options.linklookupkeyfields
					.split(';')
					.pop()
					.split()
					.concat(options.linklookupresultfield.split(';'))	
					.map(fieldExpr => item[fieldExpr])
					.join(' - ')
					.replace(/&nbsp/g, ' ');
			}
			return ((item) || [])[options.fieldName];
		}

		const itemTemplate = function(item) {
			var text = options.displayExpr.split(';').map(fieldExpr => item[fieldExpr]).join(' - ').replace(/&nbsp/g, ' ');
			var search = that.component.option('value');
			var resElement = $('<span>').text(text);
			if (search.length) {								 
				var startPos = text.toLowerCase().indexOf(search.toLowerCase());
				var itemText = "";
					if (startPos >= 0) {
						itemText = [
								text.slice(0, startPos), 
								"<span class='lookup-search-highlight'>", 
								text.slice(startPos, startPos + search.length), 
								"</span>", 
								text.slice(startPos + search.length)
						].join('');
						resElement = $('<span>').html(itemText);
				}     				
			}
			return resElement; 
		}

		const onInput = function ({ element, event }) {
			const fieldComponent = element.siblings('#ctrl' + options.fieldName);
			that._value = event.currentTarget.value;
			fieldComponent.val(that._value);
		}

		const onFocusOut = function (...args) {
			const hasValue = (val) => !['', null, undefined].includes(val);
			const fieldComponent = that._form.fieldbyName(that._options.fieldName).componente
			const { lookup, value } = fieldComponent.data();
			const mustSearch = hasValue(value) && hasValue(that._value) && that._value != value;

			fieldComponent.data('value', that._value);
			fieldComponent.val(that._value);

			FindLookupValue(options.fieldName, options.dmClassName, options.alias, mustSearch)
				.then((data) => {
					if (!data && ![''].includes(that._value)) {
						that._setLookupDesc({
							[`${that._options.fieldName}RESULT`]: lookup,
						});
					}
				})
				.catch((e) => {
					fieldComponent.data('lookup', 'Falha ao buscar lookup...');

					that._setLookupDesc({
						[`${that._options.fieldName}RESULT`]: 'Falha ao buscar lookup...',
					});
				});
		}

		const onFocusIn = function() {
			that.component.option('value', that._value);
		}

		const onOpened = ({ component: { _list }}) => {
			queueMicrotask(() => { 
				_list.option('focusedElement', _list.getItemElementByFlatIndex(0));
			});
		};

		return { 
			onInput, 
			itemTemplate, 
			valueExpr, 
			onItemClick,
			onFocusOut,
			onFocusIn,
			onOpened
		}
	}

	_buildAutocomplete(HTMLElement, options) {
		return HTMLElement ? new dxAutocomplete(HTMLElement, options) : null;
	}

	_configureFirstLoad() {
		let that = this;
		this._form.AfterLoadDataSubject.subscribe((data) => {
			if (data.data[that._options.fieldName]) {
				that._setLookupDesc(data.data);
			}
		});
		if (this._options.initialValue) {
			this.component.option('value', this._options.initialValue);
			this._value = this._options.initialValue;
		}
	}
	
	get component() {
		return this._autocompleteInstance;
	}

	get options() {
		return this._options;
	}

	_setLookupDesc(data) {
		if (this._options.isMultiColumn) {
			this.component.option('value', `${this._value} - ${data[this._options.fieldName + 'RESULT']}`);
			this.component
				.element()
				.siblings("#ctrl"+ this._options.fieldName)
				.val(this._value)
		}
		else {
			$("#ctrl"+this._options.fieldName+'RESULTTEXT').text(data[`${this._options.fieldName}RESULT`]);
		}
	}

	/**
	 *	Gets the input from nFormulario to track changes by formulario.loadfromdata().
	 * 	ex: select a lookup item from modal
	 * 
	 * @param {JQuery} fieldComponent 
	 */
	configureInputPropagation(fieldComponent) {
		const that = this;
		
		fieldComponent.on('change', function(event){
			let valueForInput;

			that._value = $(event.target).data('value');

			if (that._value == '') valueForInput = ''
			else if (that._options.isMultiColumn) valueForInput = (that._value + ' - ' + that._form.getvalue(`${that._options.fieldName}RESULT`));
			else valueForInput = that._value;
			
			that.component.option('value', valueForInput);
		});
	}
	
	/**
	 * 
	 * @param {Field} fieldInstance
	 */
	setQField(fieldInstance) {
		this._autocompleteInstance.element().find('input').data('q-field', fieldInstance);
	}
}
