const removeSpaces = e => e.replace(/\s/g, '');
const removeSpecialCharacter = e => e.replace(/[^a-zA-Z0-9]+/g, '');

export const createAccordion = function (formFields, multiColumn) {
    let accordions = extractFormAccordion(formFields);
    
    insertAccordion(accordions, multiColumn);

    $(".collapse").collapse({ toggle: false });
}

function extractFormAccordion(formFields) {
    const accordions = formFields.map((field) => ({
        cssClass: removeSpecialCharacter(removeSpaces(field.divider)),
        name: _.trim(field.divider),
        required: field.required === "true",
        formClass: field.formClass
    }));

    const requiredAccordions = accordions.filter(({ required }) => required);

    return _.uniqBy(accordions, 'cssClass').map((accordion) => ({
        ...accordion,
        required: accordion.required || requiredAccordions.find(({ cssClass }) => cssClass == accordion.cssClass),
    }));
}

function insertAccordion(accordions, multiColumn) {
    if (accordions) {
        accordions.forEach(accordion => {
            if (accordion.cssClass) {
                const formFieldsRow = $(`.form-fields.anchor-accordion${accordion.formClass}`);
                const accordionStructure =
                    `<div class="col-lg-12 col-xl-12 mt-3">
						<div class="accordion accordion-solid accordion-toggle-arrow" id="main_accordion_Credito">
						<div class="card">
							<div class="card-header" id="main_accordion_item_${accordion.cssClass}">
								<div class="card-title collapsed" data-toggle="collapse" data-target="#accordion-body-${accordion.cssClass}" aria-expanded="false">
									<div>
										<span>${accordion.name}</span>
										${accordion.required ? "<h6>* Contém campos obrigatórios </h6>" : ""}
									</div>
								</div>
							</div>
						</div>
						</div>
					</div>`;

                
                if (formFieldsRow.length) {
                    const formFields = $('.input-block.form-group.kt-form__group.' + accordion.cssClass);

                    $(accordionStructure).insertAfter(formFields.eq(0));

                    const accordionContent =
                        `<div id="accordion-body-${accordion.cssClass}" data-accordion="true" class="collapse" aria-labelledby="Accordion Item" data-parent="#main_accordion_item_${accordion.cssClass}">
								<div class="card-body ${multiColumn ? "d-flex flex-wrap" : ""}">
								</div>
						</div>`;

                    $('.input-block.form-group.kt-form__group.' + accordion.cssClass).wrapAll(accordionContent);
                    $('#accordion-body-' + accordion.cssClass).insertAfter('#main_accordion_item_' + accordion.cssClass);
                }
            }
        })
    }
}

export const addAccordionClassField = (id, accordion) => $(id).addClass(removeSpecialCharacter(removeSpaces(accordion)));