import moment from "moment";
import {nStringToDate} from "./nStringToDate";
import {nStringToValue} from "./nStringToValue";
import { RequestExecutarExpression } from "./nRequests";

export class nExpressionExecutor{
    
    constructor(){
        this.valores = {};   
        this.cache = {};
        this.dateValidator = new nStringToDate();        
        this.valueValidator = new nStringToValue();        
    }

    registrarValor(Nome, Valor) { 
        const _checkbox_ =  '_checkbox_';
        const matchCheckBox = Nome.match(_checkbox_);

        if (matchCheckBox) {
            const field = Nome.slice(0, matchCheckBox.index);
            const checkVal = Nome.slice(matchCheckBox.index + _checkbox_.length);
            
            if (!Valor) {
                if (!this.valores[field]) this.valores[field] = '';
                if (!this.valores[field].includes(checkVal)) return;

                const values = this.valores[field];
                const valueIndex = values.indexOf(checkVal);
                const firstHalf =  values.slice(0, valueIndex);
                const secondHalf = values.slice(valueIndex + 1);

                this.valores[field] = `${firstHalf}${secondHalf[0] === ';' ? (secondHalf.slice(1) || '') : secondHalf}`;
            }
            else {
                if (this.valores[field].includes(checkVal)) return;

                const values = this.valores[field] || '';
    
                this.valores[field] = values.concat(`${values.length ? ';' : ''}${checkVal}`);
            }
        }
        else {
            this.valores[Nome] = typeof Valor === 'object' ? '' : Valor;
        }
    }

    clear() {
        this.valores = {};
    }

    checkConvertValue(data) {
        if (this.dateValidator.isDate(data)){
          return this.dateValidator.toDate(data);
        }   
        if (this.valueValidator.isValue(data)){
          return this.valueValidator.toValue(data);
        }    
        return data;
    }

    async funcIF(args){
        let [Condicao, valorTrue, valorFalse] = args;
        Condicao = await Condicao;
        valorTrue = await valorTrue;
        valorFalse = await valorFalse;
        if (Condicao){
            return valorTrue
        }else{
            return valorFalse
        }
    }
    
    async funcSOMAR(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
        valor2 = await valor2;
        if (Array.isArray(valor2)){
          let valores = valor2;
          valor2 = valores.reduce((acum, val) => acum + val, 0);
        }
        return this.checkConvertValue(valor1) + this.checkConvertValue(valor2);        
    }

    async funcSUBTRAIR(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
        valor2 = await valor2;
        if (Array.isArray(valor2)){
          let valores = valor2;
          valor2 = valores.reduce((acum, val) => acum + val, 0);
        }
        return this.checkConvertValue(valor1) - this.checkConvertValue(valor2);      
    }

    async funcMULTIPLICAR(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
        valor2 = await valor2;
        if ((!valor1) || (!valor2)){
            return 0;
        }
        return this.checkConvertValue(valor1) * this.checkConvertValue(valor2);   
    }

    async funcDIVIDIR(args){
        let [Dividendo, Divisor] = args;
        Dividendo = await Dividendo;
        Divisor = await Divisor;
        if ((!Dividendo) || (!Divisor)){
            return 0;
        }
        return this.checkConvertValue(Dividendo) / this.checkConvertValue(Divisor);  
    }    
    
    async funcIGUAL(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
				valor2 = await valor2;
        return Boolean(valor1 == valor2);  
    }
    
    async funcMAIORQUE(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
        valor2 = await valor2;
        return Boolean(this.checkConvertValue(valor1) > this.checkConvertValue(valor2)); 
    }

    async funcMENORQUE(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
        valor2 = await valor2;
        return Boolean(this.checkConvertValue(valor1) < this.checkConvertValue(valor2)); 
    }

    async funcMAIORIGUAL(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
        valor2 = await valor2;
        return Boolean(this.checkConvertValue(valor1) >= this.checkConvertValue(valor2)); 
    }

    async funcMENORIGUAL(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
        valor2 = await valor2;
        return Boolean(this.checkConvertValue(valor1) <= this.checkConvertValue(valor2)); 
    }

    async funcDIFERENTE(args){
        let [valor1, valor2] = args;
        valor1 = await valor1;
        valor2 = await valor2;
        return Boolean(this.checkConvertValue(valor1) != this.checkConvertValue(valor2)); 
    }

    getDateFormat(data){
        return this.dateValidator.getDateFormat(data);       
    }

    async funcINCDAY(args){
        let [data, incremento] = args;  
        data = await data;
        incremento = await incremento;      
        return moment(data).add(incremento, 'days').format(this.getDateFormat(data));
    }

    async funcINCMONTH(args){
        let [data, incremento] = args;
        data = await data;
        incremento = await incremento;      
        return moment(data).add(incremento, 'months').format(this.getDateFormat(data));
    }

    async funcINCYEAR(args){
        let [data, incremento] = args;
        data = await data;
        incremento = await incremento;      
        return moment(data).add(incremento, 'years').format(this.getDateFormat(data));
    }

    async funcIN(args){
        let [item, lista] = args;
        item = await item;
        for (var i = 0; i < lista.length; i++) {
			lista[i] = await lista[i];
		}		
		var itemtype = typeof item;
		var listtype = typeof lista[0];
		
		if (listtype === itemtype) {
			return lista.includes(item);
		}
		else{
			for (var i = 0; i < lista.length; i++) {
				try{
				  lista[i] = String(lista[i]);
				}catch{
					
				}; 
			}
			return lista.includes(String(item));
		}					
    }
	
	async funcCASE(args){
        let [item, opcoes, resultados, padrao] = args;
        item   = await item;
		
        for (var i = 0; i < opcoes.length; i++) {
			opcoes[i] = await opcoes[i];
		}		
		var itemtype = typeof item;
		var listtype = typeof opcoes[0];
		
		if (listtype === itemtype) {
			var indice = opcoes.indexOf(item);
		}
		else{
			for (var i = 0; i < opcoes.length; i++) {
				try{
				  opcoes[i] = String(opcoes[i]);
				}catch{
					
				}; 
			}
			var indice = opcoes.indexOf(String(item));
		}
		
		if (indice == -1){
			return await padrao;
		}
		else{
			return await resultados[indice];
		}        		
    }

    async funcENTRE(args){
        let [item, inicial, final] = args;
        item = await item;
        inicial = await inicial;
        final = await final;        
        return (this.checkConvertValue(item) >= this.checkConvertValue(inicial))&&
               (this.checkConvertValue(item) <= this.checkConvertValue(final));        
    }

    async funcAND(args){
        let [lista] = args;        
        for(let item of lista){
            item = await item;
            if(!item){
                return false;
            }
        }
        return true;
    }

    async funcOR(args){
        let [lista] = args;        
        for(let item of lista){
            item = await item;
            if(item){
                return true;
            }
        }
        return false;
    }

    async funcNOT(args){
        let [item] = args;        
        item = await item;
        return !item;
    }

    async funcEXTERNO(args){
        let [funcao, params] = args;        
        funcao = await funcao;
        var chave = funcao;
        for (let i = 0; i < params.length; i++){
            params[i] = String(await params[i]);
            chave = chave + '_' + params[i];
            if (params[i] == null){
              return null;
            }
        }
        if (this.cache.hasOwnProperty(chave)){
            return this.cache[chave]
        }else{            
            this.cache[chave] = null;
            KTApp.blockPage();
            var request = new RequestExecutarExpression(funcao, params);
            var resultado = await request.executar();
            this.cache[chave] = resultado;                        
            return resultado;            
        }
		}
		
	async funcVAZIO(args){
			let [param] = args;
			param = await param;

			if (typeof param === 'string')
				param = param.trim();
			
			return [undefined, null, ''].includes(param);
	}

	async funcFORMSTATE(args) {
		let [className] = args;
		let { estado } = FormManager.byname(await className);
		
		return estado;
	}

	async funcCONTAINS(args) {
		let [astring, asubstring] = args;
		astring = await astring;
		asubstring = await asubstring;
		
		return astring.includes(asubstring);
	}

	async funcCOPY(args) {
		let [astring, aindex, acount] = args;	
		astring = await astring;
		aindex = await aindex;
		acount = await acount;
		
		astring = astring + "";
		aindex = aindex - 1;
		acount = acount + aindex;
		
		return astring.substring(aindex, acount);
	}

    async execute(Expression){
        const IF            = async (...args) => await this.funcIF(args);
        const AND           = async (...args) => await this.funcAND(args);
        const OR            = async (...args) => await this.funcOR(args);
        const NOT           = async (...args) => await this.funcNOT(args);
        const SOMAR         = async (...args) => await this.funcSOMAR(args);
        const SUBTRAIR      = async (...args) => await this.funcSUBTRAIR(args);
        const MULTIPLICAR   = async (...args) => await this.funcMULTIPLICAR(args);
        const DIVIDIR       = async (...args) => await this.funcDIVIDIR(args);    
        const IN            = async (...args) => await this.funcIN(args);
		const CASE          = async (...args) => await this.funcCASE(args);
        const ENTRE         = async (...args) => await this.funcENTRE(args);
        const IGUAL         = async (...args) => await this.funcIGUAL(args);
        const MAIORQUE      = async (...args) => await this.funcMAIORQUE(args);
        const MENORQUE      = async (...args) => await this.funcMENORQUE(args);
        const MAIORIGUAL    = async (...args) => await this.funcMAIORIGUAL(args);
        const MENORIGUAL    = async (...args) => await this.funcMENORIGUAL(args);
        const DIFERENTE     = async (...args) => await this.funcDIFERENTE(args);
        const INCDAY        = async (...args) => await this.funcINCDAY(args);
        const INCMONTH      = async (...args) => await this.funcINCMONTH(args);
        const INCYEAR       = async (...args) => await this.funcINCYEAR(args);
				const EXTERNO       = async (...args) => await this.funcEXTERNO(args);
				const VAZIO					= async (...args) => await this.funcVAZIO(args);
				const FORMSTATE			= async (...args) => await this.funcFORMSTATE(args);
				const CONTAINS      = async (...args) => await this.funcCONTAINS(args);
				const COPY          = async (...args) => await this.funcCOPY(args);

				const testValue = (value) => {
					// test for NaN, workaround for isNaN(' ') = false (wich ' ' is converted to 0) ;
					return isNaN(value) || (typeof value === 'string');
				} 

				let JSLocal = '';
				
        for (let prop in this.valores) {
            let isDate = this.dateValidator.isDate(this.valores[prop]);
            let isString = testValue(this.valores[prop]);

            if (isDate || isString) {
                JSLocal += ` var ${prop} = \`${this.valores[prop]?.replace(/\\/g, '\\\\')}\`;`;
            }
            else {
                if (this.valores[prop]) {
                    JSLocal += ' var ' + prop + ' = ' + this.valueValidator.toValue(this.valores[prop]) + ';';
                }
                else {
                    JSLocal += ' var ' + prop + ' = null;';
                }
            }
        }
        JSLocal += Expression;
        try {
        //console.log(JSLocal);
        let result = await eval(JSLocal);
        //console.log(Expression +' = '+ result);
		    return result;
		}
		catch (e) {   
            console.log(JSLocal);
            console.log(e);         
			return '';
		}
    }

}