﻿import { itsToUseLookupInside } from './nLookUp';
import nAAction from './form/actions/nActions';
import { FraControlHistCt } from './fraControlHistCtb';
import { BehaviorSubject, Subject } from 'rxjs';
import { nExpressionExecutor } from './nExpressionExecutor';
import { FormState, getBtnSalvar } from './nCadastro';
import FormManager from './FormManager';
import { fixRowValues } from './nGrid';
import { nFraControl } from './nFraControl';
import { getFormatOptions } from './setDatepicker';

export const ControlType = {
	ntwEditComLabel: 'ntwEditComLabel',
	ntwDiaComData: 'ntwDiaComData',		
	ntcGrid: 'ntcGrid',
	ntcEdit: 'ntcEdit',
	ntcActBuscarCEP: 'ntcActBuscarCEP',
	ntcLinkEdit: 'ntcLinkEdit',
	ntcLink: 'ntcLink',			
	ntcComboBox: 'ntcComboBox',
	ntcCheckListBox: 'ntcCheckListBox',
	ntcCheckBox: 'ntcCheckBox',
	ntcMemo: 'ntcMemo',
	ntcRichEdit: 'ntcRichEdit',			
	ntcDBRichEditEditor: 'ntcDBRichEditEditor',
	ntcHistorio: 'ntcHistorio',
	ntcImage: 'ntcImage',	
	ntcActBlob: 'ntcActBlob',
	ntcCheckTreeView: 'ntcCheckTreeView',
	ntcCheckListBoxScroll: 'ntcCheckListBoxScroll',
	ntcCheckListBoxHorizontal: 'ntcCheckListBoxHorizontal',
	ntcLinkAnnotation: 'ntcLinkAnnotation',
	ntcMemoAnnotation: 'ntcMemoAnnotation',
};

export class Field {
	constructor({ nome, componente, linkbutton, label, resultcomponente, componenteimage, resultcomponentetext, visible, index, isMultiColumn, isGroupColumn, expressoes, lookupAlpha, controlType, InstanciaFormulario }) {
		this.nome = nome;
		this.componente = componente;
		this.resultcomponente = resultcomponente;
		this.resultcomponentetext = resultcomponentetext;
		this.ultimocontrolechave = false;
		this.autoincrementar = componente.data('autoincrementar');
		this.valorfixo = null;
		this.linkbutton = linkbutton;
		this.label = label;
		this.componenteimage = componenteimage;
		this.imagevalue = '';
		this.filebase64 = '';
		this.enabled = true;
		this.visible = (visible == undefined) ? true : visible;
		this.index = index;
		this.classe = '';
		this.multiColumn = isMultiColumn;
		this.isGroupColumn = isGroupColumn; 
		this.loaded = new Subject();
		this.loadExpression = new Subject();
		this.componente.data('q-field', this);

		this.isFile = componente.attr('type') == "file";
		this.lookupAlpha = lookupAlpha;
		this.expressoes = expressoes;
		this.controlType = controlType;
		this.InstanciaFormulario = InstanciaFormulario;
		this.isFormRepeater = controlType == ControlType.ntcGrid;

		this.formRepeater = null;

		this.isPrimaryKey = Boolean(InstanciaFormulario.chaveprimaria?.find((key) => key === nome));

		if (this.haspicker()) {
			this.getDatepickerElement().on('blur', this.onChange.bind(this)); 
			this.getDatepickerElement().on('focus', this.onFocus.bind(this)); 
		}
		else {
			this.componente.on('change', this.onChange.bind(this)); 
			this.componente.on('focus', this.onFocus.bind(this));
		} 

		if (this.componente.data("haslinklookup")) {
			this.isRequestingLookup = false;
			this.componente.on('blur', this.onBlurLokkupAtualizarValue.bind(this));
		}

		if (linkbutton) {
			linkbutton.data('q-field', this);
		}
		if (controlType == ControlType.ntcCheckBox) {
			this.componente.siblings('input').data('q-field', this);
		}

		this.setLookupAlpha();
		this.configureByName();
		this.applyOrRefreshMask();
	}

	remove() {
		this.InstanciaFormulario.removeField(this);
	}

	configureByName() {
		if (this.nome === 'CODIGOHISTCTB') {
			console.log('configure Histórico Contábil');
			this._fraControlHistCtb = new FraControlHistCt(this);
		}
	}

	setLookupAlpha() {
		if (this.lookupAlpha) {
			this.loaded.subscribe(data => this.lookupAlpha._setLookupDesc(data));

			this.lookupAlpha.configureInputPropagation(this.componente);
			this.lookupAlpha.setQField(this);
		}
	}
    onBlurLokkupAtualizarValue(){
      this.componente.data('value', this.componente.val());
    }

	async onChange(event, originalEvent) {
		if (!this.componente.data('value') || !this.componente.val()) {
			this.onBlurLokkupAtualizarValue();
		}
		const executor = this.InstanciaFormulario.ExpressionExecutor;
	
		try {
			for (let loopfield of this.InstanciaFormulario.fields) {
				executor.registrarValor(loopfield.nome, loopfield.getExpressionValue());
				
				if (!loopfield.expressoes) continue;
	
				if (loopfield.expressoes.ExpressaoEnabled) {
					const AEnabled = await executor.execute(loopfield.expressoes.ExpressaoEnabled);
					loopfield.setenabled(AEnabled);
				}
				if (loopfield.expressoes.ExpressaoRequired) {
					const AEnabled = await executor.execute(loopfield.expressoes.ExpressaoRequired);
					loopfield.setrequired(AEnabled);
				}
				if (loopfield.expressoes.ExpressaoVisible) {
					const AEnabled = await executor.execute(loopfield.expressoes.ExpressaoVisible);
					loopfield.setvisible(AEnabled);
				}
				if (loopfield.expressoes.ExpressaoMask) {
					const AMascara = await executor.execute(loopfield.expressoes.ExpressaoMask);
	
					if (![null, undefined].includes(AMascara)) {
						if (AMascara.indexOf('#') != -1) {
							await this.mudarMascaraValorMonetario(loopfield, AMascara);
						}
						else {
							await this.mudasMascaraValorTexto(loopfield, AMascara);
						}
					}
				}
				if (FormManager.podeEventoChangeFields) {
					if (loopfield.expressoes.ExpressaoClear) {
						const AClear = await executor.execute(loopfield.expressoes.ExpressaoClear);
						if (AClear) {
							loopfield.setvalue('');
						}
					}
					if ((!loopfield.getvalue() || loopfield.expressoes.ForcarValor)
						&& (!loopfield.expressoes.PermiteAlterarValorCloud ||
							(event != undefined &&
								originalEvent == undefined &&
								(event.currentTarget.name == loopfield.componente.prop('name') && 
								 event.currentTarget.value == '')  ||
								(event.currentTarget.name != loopfield.componente.prop('name') && 
								loopfield.expressoes.ExpressaoValue.includes(event.currentTarget.id.replace("ctrl", "")))
							)
						   )
						) {
						const executor = loopfield.InstanciaFormulario.ExpressionExecutor;
	
						if (loopfield.expressoes.ExpressaoValue) {
							const sugestao = await executor.execute(loopfield.expressoes.ExpressaoValue);
	
							if (![null, undefined, ''].includes(sugestao)) {
								loopfield.setvalue(sugestao);

								executor.registrarValor(loopfield.nome, loopfield.getExpressionValue());
							}
						}
					}
				}
	
			}
			this.loadExpression.next({ event, originalEvent });
		} finally {
			KTApp.unblockPage();
		}
	}
	
	async PegarDecimais(_AValor){
		var valores = _AValor.split(",");
		if (valores.length > 1){
			return valores[1];
		}else{
			return valores[0];
		}
	}
	
	async ValorSemDecimais(_AValor){
		return _AValor.split(",")[0];
	}
	
	async mudarMascaraValorMonetario(loopfield, AMascara){
		AMascara = AMascara.replace(',', '!').replace('.', ',').replace('!', '.');		
		
		var decimaisValor = await this.PegarDecimais(loopfield.getvalue());
		var decimaisMask  = await this.PegarDecimais(AMascara.replace(/\D/g, ''));
		
		var valor = await this.ValorSemDecimais(loopfield.getvalue()) + ",";
		if (decimaisValor.length > decimaisMask.length){
			valor = valor + decimaisValor.slice(0, decimaisMask.length-1);
		}else{
			valor = valor + decimaisValor + "0".repeat(decimaisMask.length-decimaisValor.length-1);
		}		
		
		try{loopfield.componente.unmask();}catch{}
		if (AMascara.length > 0){
		  loopfield.componente.attr('data-mask', AMascara);						  						  						  
		  $.applyDataMask(loopfield.componente);
		}else{
		  loopfield.componente.removeAttr('data-mask');
		}		
		if (loopfield.getvalue()){
			loopfield.setvalue(valor);								
		}
	}

    async mudasMascaraValorTexto(loopfield, AMascara){
		var avalor = loopfield.getvalue().replace(/\D/g, '');
		var atamanhomask = AMascara.replace(/\D/g, '').length;
		if (atamanhomask > 0){
		  if (avalor.length < AMascara.replace(/\D/g, '').replace(/9/g,'').length){
			  avalor = "0".repeat(atamanhomask - avalor.length) + avalor;
		  }
		  else{
			  if(avalor.length > atamanhomask){
				  avalor = avalor.slice(avalor.length-atamanhomask);
			  }
		  }
		}						
		try{loopfield.componente.unmask();}catch{}
		if (atamanhomask > 0){
		  loopfield.componente.attr('data-mask', AMascara);						  						  						  
		  $.applyDataMask(loopfield.componente);
		}else{
		  loopfield.componente.removeAttr('data-mask');
		}							
		loopfield.setvalue(avalor);						                    			
	}
	
	async onFocus(){
        
    }

	haspicker() {
		return this.componente.hasClass('hasdatepicker');
	}

	hasPickerValid() {
		return moment($(this.componente).val()).isValid();
	}

	getDatepickerElement() {
		if (this.haspicker()) {
			return $(this.componente[0].previousElementSibling).find('input');
		}
		return null;
	}

	required() {
		if (this.componente.type == 'checkbox' || this.componente.length == 0) {
			return false;
		}

		if (this.haspicker()) {
			var pickerEl = this.getDatepickerElement();
			return (!this.oculto() && pickerEl.attr('required'));
		}

		return (!this.oculto() && this.componente[0].hasAttribute('required'));
	}

	oculto() {
		if (this.haspicker()) {
			let pickerEl = this.getDatepickerElement();

			return (pickerEl.attr('type') == 'hidden') || (!this.visible);
		}

		if (this.controlType === ControlType.ntcCheckTreeView) {
			return !this.visible;
		}

		return (this.componente.attr('type') == 'hidden') || (!this.visible);
	}

	hasvalue() {
		const isCheckList = [ControlType.ntcCheckListBox, ControlType.ntcCheckListBoxHorizontal, ControlType.ntcCheckListBoxScroll].includes(this.controlType);
		
		if (isCheckList) {
			const getFieldName = ({ nome }) => nome.replace(/_checkbox_\S+/i, '');
			const fieldName = getFieldName(this);

			return this.InstanciaFormulario
				.fields
				.filter((field) => getFieldName(field) === fieldName)
				.reduce((hasValue, field) => field.getvalue() || hasValue, false);
		}

		return !['', null, undefined].includes(this.getvalue());
	}

	hasatencao() {
		if (this.componente) {
			var parent = this.componente.parents('.form-group');
			if (parent.length > 0)
				return parent.hasClass('has-danger');
		}
		return false;
	}

	setatencao() {
		if (this.haspicker()) this.componente.parent().find('input').addClass('is-invalid');
		else if (this.componente) this.componente.addClass('is-invalid');
	}

	removeatencao() {
		if (this.haspicker()) this.componente.parent().find('input').removeClass('is-invalid');
		else if (this.componente) this.componente.removeClass('is-invalid');
	}

	getvalue() {
		if (this.isFormRepeater) {
			return this.formRepeater
				.getItemsArray()
				.map((instance) => {
					return instance.fields
						.reduce((fieldValues, field) => ({
							...fieldValues,
							[field.nome]: field.getvalue(),
						}), {});
				});
		}
		if (this.filebase64 != ''){			
		    var result = {};
			result.data = this.filebase64;
			result.filename = this.componente.val();
			return result;
		}
		if (this.componenteimage != null) {
			return (this.componente.val() != '') ? this.imagevalue : null;
			//Não mudou imagem, não necessita enviar de volta						
		}
		if (this.componente.prop("type") == 'checkbox') {
			return this.componente.prop('checked') == true;
		} else if ((this.componente.prop("type") == 'datetime-local')&&(this.componente.val())) {
			return moment(this.componente.val(), 'YYYY-MM-DDTHH:mm:ss').format('YYYY-MM-DD HH:mm:ss');
		}

		if (this.componente.data('haslinklookup')){
			switch ($(this.componente).css("text-transform")) {
				case 'uppercase':
					return this.getValFromLookup('toUpperCase'); 
				case 'lowercase':
					return this.getValFromLookup('toLowerCase'); 
				default:
					return this.getValFromLookup();
			}
		}
		else {
			let value = this.fixDicValue(this.componente.val(), this.dicionario());
			switch ($(this.componente).css("text-transform")) {
				case 'uppercase':
					return value.toUpperCase(); 
				case 'lowercase':
					return value.toLowerCase(); 
				default:
					return value;
			}
		}
	}
	
	fixDicValue(value, dic){
		if (['CODIGOIMPOSTO', 'CODIGOCFOP'].includes(dic)){
			return String(value).replace(/\./g, '');
		} else {
			return value;
		}
	}

	getExpressionValue(){
		return this.fixDicValue(this.getvalue(), this.dicionario());
	}

	getValFromLookup(textTransform) {
		let value = this.fixDicValue(String((this.componente.data('value') || "")).trim(), this.dicionario());
		return textTransform ? value[textTransform]() : value;   	
	}

	getDescFromLookup(textTransform) {
		 let value = String((this.componente.data('lookup') || "")).trim();
		return textTransform ? value[textTransform]() : value;   	
	}

	getvalueasnumeric() {
		if ((this.getvalue() == '') || (this.getvalue() == null)) {
			return 0;
		}

		return this.getvalue().replace(new RegExp('[.]', 'g'), '').replace(',', '.');
	}

	setvalue(value) {
		const exists = (val) => !['', null, undefined].includes(val);

		if (this.isFormRepeater) {
			if (typeof value === 'string') {
				try {
					value = JSON.parse(value);
				}
				catch {
					value = [];
				}
			}

			this.formRepeater.loadData(value);
		}
		else if (this.componenteimage != null) {
			this.componente.val('');
			this.componenteimage.attr('src', value);
		} else if (this.haspicker()) {
			const el = $(this.getDatepickerElement());
			if (el.length > 0 && $.data(el.get(0), 'datepicker')) { // se já foi inicializado com o datepicker, pois o evento change dos fields é chamado antes do setarevento
				if (value) {
					const formatoData = value.match(/\d{2}\/\d{2}\/\d{4}/g) ? 'DD/MM/YYYY' : 'YYYY-MM-DD';
				
					const dateValue = moment(value, formatoData).toDate();		
					el.datepicker('setDate', dateValue);									 
				} else {
					this.getDatepickerElement().val('').datepicker('update');
				}
			}
		} else {
			const mask = this.componente.data('mask')?.mask;

			if (exists(mask) && exists(value)) {
				const mascaraDecimal = '#0,'; //#.##0,
				
				if (mask.search(mascaraDecimal) != -1) {
					if (typeof value == 'number') {
						value = value.toString();
					}

					if ((value.search(/\./) != -1) && (value.search(',') == -1)) { // ex: value = '123.12'
						value = Number(value);
					}
					else { // ex: value = '1.123,12' | '123'
						value = Number(value.replace(/\./g, '').replace(',', '.'));
					}

					const decimalPrecision = mask.substring(mask.indexOf(mascaraDecimal) + mascaraDecimal.length).length;

					value = value.toLocaleString('br', {
						minimumFractionDigits: decimalPrecision,
						maximumFractionDigits: decimalPrecision,
					});
				}
			}

			if ((this.componente.attr('type') === 'datetime-local') && (value)) {
				value = moment(value, 'YYYY-MM-DD HH:mm:ss').format('YYYY-MM-DDTHH:mm:ss');				
			} 

			if (this.controlType == ControlType.ntcCheckBox) {
				this.componente
					.siblings()
					.first()
					.prop('checked', value == 1);
			}
			this.componente.val(value);

			if (exists(mask)) {
				$(this.componente).unmask();
				$.applyDataMask(this.componente);
			}

			if (this.componente.data('haslinklookup')) this.componente.data('value', value);
		}
		if (((value == null) || (value == ''))&& this.limpalookup()) {
			this.resultcomponente.val('');
			this.resultcomponentetext.text('Nenhum item Selecionado');
		}
	}

    limpalookup(){
      return (this.resultcomponentetext)&&(this.resultcomponentetext)&&
             (this.resultcomponentetext.length > 0)&&(this.resultcomponentetext.length > 0);
    }

	disabled() {
		return this.componente.prop('disabled');
	}

	dicionario() {
		return this.componente.data('dicionario');
	}

	nomesvalores() {
        return (this.linkbutton == null) ? '' : this.linkbutton.data('nomesvalores');
	}

	type() {
		return this.componente.prop('type');
	}

	setarmascara(mascara) {
		$(this.componente).mask(mascara);
	}

	removermascara() {
		$(this.componente).unmask();
	}

	setrequired(value) {
		// var AStyle = (value) ? 'bold' : 'normal';
		// this.label.attr('style', 'font-weight:' + AStyle);
		// this.label.attr('data-bold', AStyle);
		this.componente.closest("[id*=formField]").find('[data-span-required]').toggle(Boolean(value));
		this.componente.attr('required', value);
	}

	setenabled(value) {
		if (this.isPrimaryKey && this.InstanciaFormulario.estado != FormState.cadInsert) value = false;

		this.enabled = value;

		var componente = this.componente;

		if (this.haspicker()) {
			componente = $(componente[0].previousElementSibling).find('input');
		}

		if (this.controlType == ControlType.ntcCheckBox) {
			componente = componente.siblings().first();
		}

		if (value) {
			componente.removeAttr('disabled');
			this.linkbutton.removeAttr('disabled');
			this.linkbutton.attr('data-toggle', 'modal');
			this.linkbutton.siblings().removeAttr('disabled');
		} else {
			componente.attr('disabled', 'disabled');
			this.linkbutton.attr('disabled', 'disabled');
			this.linkbutton.siblings().attr('disabled',	true);
			this.linkbutton.removeAttr('data-toggle');
		}
	}
	
	setvisible(value) {
		this.visible = value;
		if (this.isGroupColumn) {
			let groupField = this.componente.closest('[id*=formField]');
			let groupName = this.componente.closest('[data-group-name]').data('group-name');
			let fieldWrapper = groupField.find(`[data-group-name="${groupName}"]`);

			if (value) fieldWrapper.removeClass('hide');
			else fieldWrapper.addClass('hide');

			if (!groupField.find(':not([class*=hide])[data-group-name]').length) groupField.addClass('hide');
			else groupField.removeClass('hide');
		}
		else if (this.componente.prop('type') == "checkbox") {
			let el = this.componente.closest(".kt-checkbox");
			value ? el.show() : el.hide();
		}
		else if (this.nome) {
			let el = this.componente.parents('#formField' + this.nome + this.classe);
			if (value) el.removeClass('hide');
			else el.addClass('hide');
		}
	}

	iscombobox() {
		return (this.componente[0].options != null);
	}

	combovaluefromdescr(descr) {
		var options = this.componente[0].options;
		for (var i = 0; i < options.length; i++) {
			if (options[i].text == descr) {
				return options[i].value;
			}
		}
		return '';
	}

	combocheckvalue(value) {
		var options = this.componente[0].options;
		for (var i = 0; i < options.length; i++) {
			if (options[i].value == value) {
				return options[i].value;
			}
		}
		return '';
	}

	focus() {
		let AComponent;
		if (this.lookupAlpha) {
			this.lookupAlpha.component.focus();
		}
		else {
			if (this.haspicker()) {
				AComponent = this.getDatepickerElement();
			} else {
				AComponent = this.componente;
			}
			AComponent.focus();
			AComponent.trigger('focus');
			AComponent.select();
		}
	}

	settextolabeldireita(texto, cor) {
		var label = $('#ctrl' + this.nome + 'text2');
		if (label.length > 0) {
			label.html(texto);
			if (cor != null) {
				label.prop('style').color = cor;
			}
		}
	}

	settextolabelesquerda(texto, cor) {
		var label = $('#ctrl' + this.nome + 'TEXT');
		if (label.length > 0) {
			label.html(texto);
			if (cor != null) {
				label.prop('style').color = cor;
			}
		}
	}

	setlabel(text) {
		if (this.label.length) this.label.find('[data-text]').text(text);
	}

	setdatadireita(data) {
		var ctrldata = $('#ctrl' + this.nome + 'data');
		if (ctrldata.length > 0) {
			ctrldata.val(data);
		}
	}

	canFocus() {
		const isVisible = (field) => !field.componente.closest('.form-group').hasClass('hide');
		const isntResult = (field) => !field.nome.includes('RESULTTEXT');
		
		return (!this.disabled() && isVisible(this) && isntResult(this) && this.classe != 'TnDMSelecao');
	}

	getFormRepeaterFields(uuid) {
		const item = this.formRepeater.getItem(uuid);

		return (item || {}).fields;
	}

	getFormRepeaterField(uuid, name = '') {
		const nameLower = name.toLowerCase();
		const fields = this.getFormRepeaterFields(uuid);

		return fields 
			? fields.find((field) => field.nome.toLowerCase() === nameLower) 
			: null;
	}

	clearFormRepeaterFields(uuid) {
		const fields = this.getFormRepeaterFields(uuid);

		FormManager.podeEventoChangeFields = false;

		try {
			qNotifier.clear();

			fields.forEach(field => {
				field.removeatencao();

				if (field.componenteimage != null) field.setvalue('http://placehold.it/136x105');
				else field.setvalue('');
			})
		}
		finally {
			FormManager.podeEventoChangeFields = true;
		}
	}

	applyOrRefreshMask() {
		const hasMask = (el) => Boolean(el.data('mask'));
		const hasMaskActive = (el) => {
			const { mask } = (el?.data('mask') || {});

			return Boolean(mask);
		};

		const el = this.haspicker() ? this.getDatepickerElement() : this.componente;

		if (hasMaskActive(el)) el.unmask();
		if (hasMask(el)) $.applyDataMask(el);
	}

	setFormRepeater(instance) {
		this.formRepeater = instance;
	}
}

export class Formulario {
	constructor({classe, alias, formdetalhe, chavePrimaria, masterclass, masterfields, detailfields, filtrofixo, multiColumn, caption, element, idrequest, tipoObjeto, configextra, isFormRepeater, formRepeaterInstance}) {
		this.element = element;
		this.idrequest = idrequest;
		this.tipoObjeto = tipoObjeto;
		this.configextra = configextra || null;
		this.alias = '';
		if (masterclass != null) {
			this.masterclass = masterclass;
			this.masterfields = masterfields;
			this.detailfields = detailfields;
			this.filtrofixo = filtrofixo;
		}
		this.chaveprimaria = chavePrimaria;
		if (alias != null) {
			this.alias = alias;
		}
		this.classe = classe;
		this.nome = classe + this.alias;
		this.ehdetalhe = formdetalhe != null;
		this.fields = [];
		this.record = null;
		this.multiColumn = multiColumn;
		this.estado = null;
		this.caption = caption;
		this.grid = null;
		this.AfterDelete = null;
		this.AfterPost = null;
		this.BeforeDelete = null;
		this.BeforePost = null;
		this.AfterLoadData = null;
		this.AfterModal = null;
		this.AfterLoadDataSubject = new Subject();
		this.AfterGoToWizardItem = new Subject();
		this.AfterSave = new Subject();
		this.afterDelete = new Subject();
		this.OnWizardNext = new Subject();
		this.AfterExecuteActionProcess = new Subject();
		this.afterCreateGridConteudo = new Subject();
		this.gridContentReady = new Subject();
		this.ExpressionExecutor = new nExpressionExecutor();
		this.afterCreateGridConteudoPreserveLast = new BehaviorSubject();
		this._actions = new Map();
		this.isFormRepeater = isFormRepeater;
		this.formRepeaterInstance = formRepeaterInstance;
		this.element.data('q-form', this);
		this.afterCreateStaticFields = new Subject();
		this.splitReportParam = null;

		this.afterCreateGridConteudo.subscribe((v) => { this.afterCreateGridConteudoPreserveLast.next(v) });

		this.setCaptionComponent();
		this.changeState(FormState.cadNavegando);
	}
	
	setCaptionComponent() {
		this._captionComponent = $(`span[data-form-state-text][data-class-name="${this.classe}"][data-alias="${this.alias}"]`); 
	}

	addfield({nome, componente, linkbutton, label, resultcomponente, componenteimage, resultcomponentetext, visible, index, isMultiColumn, isGroupColumn, expressoes, lookupAlpha, controlType}) {
		var ANewField = this.newField({
			lookupAlpha: lookupAlpha,
			nome: nome,
			componente: componente,
			linkbutton: linkbutton,
			label: label,
			resultcomponente: resultcomponente,
			componenteimage: componenteimage,
			resultcomponentetext: resultcomponentetext,
			visible: visible,
			index: index,
			isMultiColumn: isMultiColumn,
			isGroupColumn: isGroupColumn,
			expressoes: expressoes,
			controlType: controlType,
			InstanciaFormulario: this,
		});
		
		this.fields[this.fields.length] = ANewField;
	}

	addFieldFormRepeater(options, additionalOptions) {
		var ANewField = this.newField(options);

		this.fields[this.fields.length] = ANewField;
	}

	newField(options) {
		const isLast = (arr, value) => (arr ? (arr[arr.length - 1] == value) : false);

		let newField = new Field({
			...options,
			InstanciaFormulario: this
		});

		newField.ultimocontrolechave = isLast(this.chaveprimaria, options.nome);
		newField.classe = this.classe;

		if (this.filtrofixo?.hasOwnProperty(options.nome)) {
			newField.valorfixo = this.filtrofixo[options.nome];
		}

		return newField
	}

	removeField(field) {
		let fieldIndex = this.fields.indexOf(field);

		this.fields.splice(fieldIndex, 1);
	}

	fieldbyName(Nome) {
		if (Nome != null && Nome != "") {
			for (var i = 0; i < this.fields.length; i++) {
				if (this.fields[i].nome.toUpperCase() == Nome.toUpperCase()) {
					return this.fields[i];
				}
			}
		}
		return null;
	}

	focarfield(Nome) {
		var resultfield = this.fieldbyName(Nome);
		if (resultfield == null) {
			resultfield.focus();
		}
	}

	fieldbyNamerDic(NomeDic) {
		var resultfield = this.fieldbyName(NomeDic);
		if (resultfield == null) {
			for (var i = 0; i < this.fields.length; i++) {
				if (this.fields[i].dicionario() == NomeDic.toUpperCase()) {
					resultfield = this.fields[i];
					break;
				}
			}
		}
		return resultfield;
	}

	fieldbyComponent(component) {
		for (var i = 0; i < this.fields.length; i++) {
			if (this.fields[i].componente == component) {
				return this.fields[i];
			}
		}
		return null;
	}

	getvalue(fieldname) {
		var Afield = this.fieldbyName(fieldname);
		if (Afield != null) {
			var avalue = Afield.getvalue();
		}
		if (avalue != undefined){
			return avalue;
		} else if ((this.masterclass)&&(FormManager)){
			var Afield = FormManager.byname(this.masterclass).fieldbyName(fieldname);
			if (Afield){
			  return Afield.getvalue();
			}
		}
		return '';
	}

	getvalueasnumeric(fieldname) {
		var Afield = this.fieldbyName(fieldname);
		if (Afield != null) {
			return Afield.getvalueasnumeric();
		}
		return 0;
	}

	getcomponent(fieldname) {
		var Afield = this.fieldbyName(fieldname);
		if (Afield != null) {
			if (Afield.haspicker()) {
				return $(Afield.componente[0].previousElementSibling).find('input');
			} else {
				return Afield.componente;
			}			
		}
		return null;
	}

	getvalueByNameOrDic(fieldname) {
		var Afield = this.fieldbyNamerDic(fieldname);
		if (Afield != null) {
			return (Afield.valorfixo != null) ? Afield.valorfixo : Afield.getvalue();
		}
		return '';
	}

	setvalue(fieldname, value) {
		var Afield = this.fieldbyName(fieldname);
		if (Afield != null) {
			return Afield.setvalue(value);
		}
		return '';
	}

	setfieldvalue(nomefield, value) {
		var afield = this.fieldbyName(nomefield);
		if (afield) {
			var componente = afield.componente;
			if (componente.prop("type") == 'checkbox') {
				componente.prop('checked', value);
			} else if (componente.prop("type") != 'file') {
				afield.setvalue(value);
				if (afield.getvalue() != value) componente.change();
			};
		}
	}

	setenabled(fieldname, value) {		
		var Afield = this.fieldbyName(fieldname);
		if (Afield != null) {
			Afield.setenabled(value);
			if (!value) {
				Afield.removeatencao();
			}
		}
	}

	setenabledCheckListBox(fieldname, value) {		
		this.fields.forEach((field, index) => {
			if (field.nome.indexOf(fieldname + '_checkbox') > -1) {
				field.setenabled(value);
			}
		});
	}
	
	getCheckListBoxValues(checkListBoxId) {
		var checkListArray = {};
		var checkListBox = document.getElementById(checkListBoxId).getElementsByTagName('input');
		checkListBox.forEach((checkbox) => {
		  checkListArray[checkbox.id] = checkbox.checked;
		});
		return checkListArray;
	}
	
	setvisibleCheckListBox(fieldName, value) {
		this.fields.forEach((field, index) => {
			if (field.nome.indexOf(fieldName + '_checkbox') > -1) {
				field.setvisible(value);
			}
		})
	}

	enabled(fieldname) {
		var Afield = this.fieldbyName(fieldname);
		return (Afield != null) ? Afield.enabled : false;
	}
	
	setvisible(fieldname, value) {
		var Afield = this.fieldbyName(fieldname);
		if (Afield != null) {
			Afield.setvisible(value);
		}
	}

	setrequired(fieldname, value) {
		var Afield = this.fieldbyName(fieldname);
		if (Afield != null) {
			Afield.setrequired(value);
		}
	}

	setenabledrequired(fieldname, value) {
		var Afield = this.fieldbyName(fieldname);
		if (Afield != null) {
			Afield.setenabled(value);
			Afield.setrequired(value);
			if (!value) {
				Afield.removeatencao();
			}
		}
	}

	loadfromdata(data, multiColumn) {
		const getResultText = (/** @type {Field} */ field, value = '') => {
			return field.dicionario().includes('PERIODOAQUIS_DATAINICIAL') && value.match(/\d{4}-\d{2}-\d{2}/)
				? moment(value, 'YYYY-MM-DD').format('DD/MM/YYYY')
				: value;
		};
		
		const reducer = (acc, index) => {
			if (index.match(/result/ig)) {
				acc.push(index.replace(/result$/i, ''));
			}
			return acc;
		};

		let dataMappedResult = Object.keys(data).reduce(reducer, []); 

		dataMappedResult.forEach((fieldName) => {
			if (data.hasOwnProperty(fieldName) && !data[fieldName])
				data[`${fieldName}RESULT`] = null;
		});

		if (data.configextra != null) {
			this.configextra = data.configextra;
		}
		FormManager.lookupToFix = [];

		if (data.EhLookup != "sim"){
			if (this.grid){
				if (!data.totalcount){
					data.totalcount = 0;					
				}
				this.grid.refresh();
				this.grid.state({});			
			}
            this.record = data;
		}

		for (var i = 0; i < this.fields.length; i++) {
			var afield = this.fields[i];
			var componente = afield.componente;
			var componenteimage = afield.componenteimage;
			var nomefield = afield.nome;
			var multiColumn = afield.multiColumn;
			
			if (data.hasOwnProperty(nomefield)) {
				var value = data[nomefield];
				if (componente.prop("type") == 'checkbox') {
					componente.prop('checked', (value == 'true' || value == true));
				} else if (componente.prop("type") != 'file') {
					var resultReg = /result$/ig;
					if (resultReg.exec(nomefield)) {
						if (!data.hasOwnProperty(nomefield + 'TEXT')) {
							const field = this.fieldbyName(nomefield.replace(/result$/i, ''));
							const resultTextField = this.fieldbyName(`${nomefield}TEXT`);						

							resultTextField.componente.text(getResultText(field, value));
						}
					}

					if ((afield.controlType === ControlType.ntcLinkEdit || data.EhLookup) && itsToUseLookupInside(multiColumn)) { 
						const setValueLookup = (field, value = '', lookup = '') => {
							const hasFocus = (element) => element.is(':focus');
							const text = (value === '') ? '' : `${value} - ${lookup}`;

							if (field.haspicker()) {
								const datePickerEl = field.getDatepickerElement();
								
								field.componente.val(value);

								if (!hasFocus(datePickerEl)) {
									const { mask } = datePickerEl.data('mask');
									const { momentFormat } = getFormatOptions(mask);
									const date = (value == '') ? '' : moment(value, 'YYYY-MM-DD').format(momentFormat);  
									const description = date ? `${date} - ${lookup}` : '';

									datePickerEl.val(description);
								}
							}
							else if (!hasFocus(field.componente)) field.componente.val(text);

							field.componente.data({ lookup, value });
							field.resultcomponente.val(lookup);
						};

						if (data.EhLookup) {						
							const fieldName = afield.nome.replace("RESULT", "");
							const inputField = afield.InstanciaFormulario.fieldbyName(fieldName);

							setValueLookup(inputField, inputField.getvalue(), data[afield.nome]);
						}
						else { // default load
							const lookupFieldName = afield.nome + "RESULT";
							 
							setValueLookup(afield, value, data[lookupFieldName]);
							
							FormManager.lookupToFix.push(afield);
						}

						$.data(afield.componente[0], "original-value", value);
					}
					else {
						if ($(afield.componente).hasClass('hasdatepicker')) {
							const componentDatePicker = afield.componente.prev('.input-group').find('input.datepicker');
							if (value){
								if (componentDatePicker.hasClass('multidate')) {
										componentDatePicker.val(value);
								}								
								else{
									let dataMask = componentDatePicker.data('mask');
									var formato = 'DD/MM/YYYY';
									if ((dataMask.mask || dataMask) == '00/0000') {
										formato = 'MM/YYYY';
									} else if ((dataMask.mask || dataMask) == '0000') {
										formato = 'YYYY';
									}

									componentDatePicker.datepicker('update', moment(value).format('DD-MM-YYYY'));
									componentDatePicker.val(moment(value, 'YYYY-MM-DD').format(formato));
							  }
							}
							else{
								componentDatePicker.val('');
							}
						} else if (value && afield.componente.prop('type') === 'datetime-local') {
							value = moment(value, 'YYYY-MM-DD HH:mm:ss').format('YYYY-MM-DDTHH:mm:ss');
						} else {
							if (afield.resultcomponentetext && afield.resultcomponentetext.length > 0) {
								let lookupFieldName = afield.nome + "RESULT";

								if (data[afield.nome] && data[afield.nome].length) {
									afield.resultcomponente.val(data[lookupFieldName] || data[lookupFieldName.toUpperCase()]);
									afield.resultcomponentetext.text(data[lookupFieldName] || data[lookupFieldName.toUpperCase()]);
								}
								afield.componente.data('value', value);
							}
						}

						afield.setvalue(value);
						afield.componente.data('value', value);

						$.data(afield.componente[0], "original-value", value);

						if (afield.nome.match(/result$/i)) {
							const fieldControl = this.fieldbyName(afield.nome.replace(/result$/i, ''))

							fieldControl?.componente?.data('lookup', data[afield.nome]);
						}
					}
					componente.trigger('change', {loadFromData: true})
				} else if (componenteimage != null && value != "") {
					afield.setvalue(value);
					componente.trigger('change', {loadFromData: true})
				};
			} else { 
				if (data._AOrigem != 'lookup' && data._AOrigem != null) {
					afield.setvalue('');
					componente.trigger('change', {loadFromData: true});
				}
			}

			if (this.isFormRepeater && (afield.controlType !== ControlType.ntcLinkEdit)) {
				afield.applyOrRefreshMask();
			}
		};
		
		if (this.fields.length >= 1){
			this.fields[0].onChange();
		}
		
		this.AfterLoadDataSubject.next({data: data, multiColumn: multiColumn});

		if (this.AfterLoadData) {
			this.AfterLoadData(data, multiColumn);
		}
	}

	clearfields() {
		FormManager.podeEventoChangeFields = false;
		try {
			//FormStateText('');
			qNotifier.clear();
			for (var i = 0; i < this.fields.length; i++) {
				if (this.fields[i].componenteimage != null)
					this.fields[i].setvalue('http://placehold.it/136x105');
				else
					this.fields[i].setvalue('');
			}
            if (this.grid){                
                this.grid.refresh();
            }
		} finally {
			FormManager.podeEventoChangeFields = true;
		}
    this.record = null;
	}

    hasRecord(){
      return this.record != null;
    }

	putvaluesin(destino) {
		return {
			...destino,
			...this.getValues(),
		};
	}

	getValues() {
		return this.fields.reduce((values, field) => {
			const value = field.getvalue();
			
			return {
				...values,
				[field.nome.toUpperCase()]: value === '' ? null : value,
			};
		}, {});
	}

    fieldsvalues(_AFields, _ADestNames) {
        var destino = {};
		for (var i = 0; i < _AFields.length; i++) {
			destino[_ADestNames[i].toUpperCase()] = (this.getvalue(_AFields[i]) == '') ? null : this.getvalue(_AFields[i]);
		}
		return destino;
	}

	putmastervaluesin(master, destino) {
		if (this.masterfields) {
			const masterForm = FormManager.byname(this.masterclass);
			const AMasterFields = this.masterfields;
			const Adetailfields = this.detailfields;

			if (masterForm.grid) {
				const selectedRows = masterForm.grid.getSelectedRowsData();
				
				if (selectedRows.length) {
					const row = fixRowValues(masterForm.grid, selectedRows[0]);
					const fields = AMasterFields.reduce((fields, current, index) => {
						const field = Adetailfields.length ? Adetailfields[index] : current;

						return {
							...fields,
							[field.toUpperCase()]: row[current.toUpperCase()],
						};
					}, {});
					
					return {
						...destino,
						...fields,
					};
				}
				else {
					return destino;
				}
			}
			else {
				if (Adetailfields.length == 0) {
					for (i = 0; i < AMasterFields.length; i++) {
						destino[AMasterFields[i].toUpperCase()] = master.getvalue(AMasterFields[i]);
					};
				} else {
					for (var i = 0; i < AMasterFields.length; i++) {
						destino[Adetailfields[i].toUpperCase()] = master.getvalue(AMasterFields[i]);
					};
				};
			};
		}
		
		return destino;
	}

	getMasterValues() {
		if (!this.masterfields) return {};
		
		const masterForm = FormManager.byname(this.masterclass);
		const values = (fields) => this.masterfields.reduce((values, masterField, index) => ({
			...values,
			[fields[index].toUpperCase()]: masterForm.getvalue(masterField),
		}), {});

		return values(this.detailfields.length ? this.detailfields : this.masterfields);
	}

	MontarFiltroChavePrimaria() {
		var Filtros = {};
		
		if (this.chaveprimaria){
			for (var i = 0; i < this.chaveprimaria.length; i++) {
				var fieldname = this.chaveprimaria[i];
				if ((this.getvalue(fieldname) != '') && (this.getvalue(fieldname) != null)) {
					Filtros[fieldname] = this.getvalue(fieldname);
				}
			};
		};

		if (this.filtrofixo != null) {
			for (var AProp in this.filtrofixo) {
				Filtros[AProp] = this.filtrofixo[AProp];
			}
		}

		if (this.masterclass != null) {
			const masterForm = FormManager.byname(this.masterclass);
			const detailFields = this.detailfields;

			this.masterfields?.forEach((field, index) => {
				let value = masterForm.getvalue(field);

				if ((value != '') && (value != null)) {
					Filtros[detailFields[index]] = value;
				}
			});
		};

		return Filtros;
	}

	setarmascaracampo(nomefield, mascara) {
		var afield = this.fieldbyNamerDic(nomefield);
		if (afield != null) {
			afield.setarmascara(mascara);
		}
	}

	removermascaracampo(nomefield) {
		var afield = this.fieldbyNamerDic(nomefield);
		if (afield != null) {
			afield.removermascara();
		}
	}

	getvalueLookup(campoPesquisa, campoRetorno) {
		KTApp.blockPage();

		let afield = this.fieldbyName(campoPesquisa);
		
		if (!afield && this.masterclass && FormManager) {
			afield = FormManager.byname(this.masterclass).fieldbyName(campoPesquisa);	
		}
		
		var d = new $.Deferred();
		
		let btnLookup = $(afield.componente).siblings().children('#btn' + campoPesquisa);

		let datasetName = 'T' + btnLookup.data('linklookupdatasetnome');
		let keyFields = btnLookup.data('linklookupkeyfields').split(';');
		let filterOptions = [];

		keyFields.forEach((key, index) => {
			if (filterOptions.length > 0) {
				filterOptions.push('and');
			}
			filterOptions.push([key, '=', this.getvalue(key)]);	
		});

		var params = {
			filter: JSON.stringify(filterOptions),
			skip: 0,		
			take: 1,
			_AClass: datasetName	
		};
					
		$.getJSON('nHttpServicos/DadosConsulta2', params).done((data) => {
			if (data.data.items.length > 0) {
				d.resolve(data.data.items[0][campoRetorno]);
			} else {
				d.resolve();
			}			
		}).fail(() => {
			KTApp.unblockPage();	  
		  });
		
		return d.promise();	
	}

	focusFirstField() {
		for (let field of this.fields) {
			if (field.canFocus()) {
				field.focus();

				if (!field.haspicker()) {
					if ((field.autoincrementar) && (!field.getvalue())) {
						AjaxAutoCompleteField(field);
					}
				}
				break;
			}
		}		
	}

	limparAtencaoFields() {
		this.fields.forEach(function (field) {
			field.removeatencao();
		})
	}

	PodeNavegarEntreRegistros() {
		return (JSON.stringify(this.masterfields) != JSON.stringify(this.chaveprimaria))
	}

	habilitarSalvar(enable = true) {
		const saveButton = getBtnSalvar(this);
		
		if (!saveButton.length) return;

		saveButton.attr("disabled", !enable)
		saveButton.siblings().first().attr("disabled", !enable);
	}

	setAction(options) {
		try {
			let action = new nAAction[options.name](options);

			if (this._actions.has(options.name))
				action = this._resolveActionCollision(options, action);

			this._actions.set(options.name, action);
		}
		catch (e) {
			console.error(`Action <${options.name}> must be custom and available on global scope.`, e)
		}
	}

	_resolveActionCollision(options, action) {
		let existentAction = this._actions.get(options.name);

		if (!(existentAction.constructor.name === 'Map'))
			existentAction = new Map([[existentAction.idElement, existentAction]]);

		existentAction.set(options.idElement, action);
		
		return existentAction;
	}

	executeAction(actionName, actionElementId) {
		if (actionElementId)
			this._actions
					.get(actionName)
					.get(actionElementId)
					.execute();
		else
			this._executeNamedOrFirstAction(actionName);		
	}

	_executeNamedOrFirstAction(actionName) {
		let action = this._actions.get(actionName);

		if (action) {
			if (action.constructor.name === "Map") {
				action = action.values().next().value
			} 
			
			action.execute();
		}
	}

	CarregarDefinicaoHistorico(event, options) {
		for (let field of this.fields) {
			if (field._fraControlHistCtb) {
				field._fraControlHistCtb.CarregarDefinicaoHistorico(event, options);
			}
		}
	}

	changeState(state) {
		this.estado = state;

		if (this._captionComponent.exists()) this._changeStateCaption(state);
		
		this._handleState(state);
	}

	_changeStateCaption(state) {
		const getCaptionState = (state) => {
			switch(state) {
				case FormState.cadAlterando: return '(Editando)';
				case FormState.cadInsert: return '(Inserindo)';
				case FormState.cadNavegando: return '(Paginando)';
				default: return '(Paginando)'
			}
		}

		this._captionComponent.text(getCaptionState(state));
	}

	_handleState(state) {
		const clearDetails = (state) => {
			FormManager.itens
				.filter((form) => form.ehdetalhe && form.masterclass === this.classe)
				.forEach((form) => {
					form.changeState(state);
	
					if (form.isFormRepeater && state === FormState.cadInsert) form.formRepeaterInstance.reset();
				});
		};

		const setEnabledPKey = (enabled = false) => {
			const setEnabled = (fieldName) => {
				if (!this.isFormRepeater) this.fieldbyName(fieldName)?.setenabled(enabled)
			};

			this.chaveprimaria?.forEach((key) => {
				if (this.afterCreateStaticFields.isStopped) return setEnabled(key)
				
				this.afterCreateStaticFields
					.pipe(take(1))
					.subscribe(() => setEnabled(key));
			});
		};

		const cadAlterando = () => {
			this.habilitarSalvar(true);

			setEnabledPKey();
		};

		const cadInsert = () => {
			this.habilitarSalvar(true);
			
			setEnabledPKey(true);

			clearDetails(FormState.cadInsert);
		};

		const defaultState = () => {
			this.habilitarSalvar(false);

			setEnabledPKey();
			
			clearDetails(FormState.cadNavegando);
		};

		switch(state) {
			case FormState.cadInsert: return cadInsert();
			case FormState.cadAlterando: return cadAlterando();
			default: return defaultState();
		}
	}

	setatencao(fieldName) {
		this.fieldbyName(fieldName)?.setatencao();
	}

	validateRequiredFields() {
		return nFraControl.ValidarCamposObrigatorios(this);
	}

	triggerExpressions() {
		this.fields[0]?.onChange();
	}
}
