import FormManager from "./FormManager";
import { ControlType, Field } from "./nFormulario";
import { RequestDadosLookup } from "./nRequests";
import { getFormatOptions } from "./setDatepicker";

export const saveLookupFields = function (form) {
	var savedLookups = {
		fields: {},
		form
	};

	for (let field of form.fields) {
		let Field = $(field.componente);
		if (Field.data('multicolumn'))
			savedLookups.fields[Field.attr('id')] = Field.val();
	}
	return savedLookups;
}

export const recoverLookupFields = function (savedLookups) {
	const { form } = savedLookups;	
	for (let field of form.fields) {
		if ($(field.componente).data('multicolumn')) {			
			field.componente.val(savedLookups.fields['ctrl' + field.nome]);
		}
	}
}

export const fixLookupFields = function (savedLookups) {
	const { form } = savedLookups;

	for (let field of form.fields) {
		if (field.multiColumn) {
			if (field.linkbutton && field.linkbutton.length > 0) {
				field.setvalue(field.getvalue());
			}
		}
	}
}

export const itsToUseLookupInside = function (multiColumn) {
	return (window.innerWidth < 991.98 || multiColumn);
}

export const initLookupStyle = function (multiColumn, isModuleSelection) {
    var $window = $(window);

    $window
        .off('resize.lookupstyle')
        .on('resize.lookupstyle', function () { setLookupStyle(multiColumn, isModuleSelection) })
        .trigger('resize.lookupstyle', [multiColumn, isModuleSelection]);

}

function setLookupStyle(multiColumn, isModuleSelection) {
    $('.form-control', isModuleSelection ? 'form.formTnDMSelecao' : document).each(function (I) {
        if (hasLinkLookup(this)) {
            if (!$(this).data('multicolumn')) {
                if (hasToBeInsideInput(this, 991.98))
                    adjustLookupStyle(this, 'inside');
                else if (hasToBeOutsideInput(this, 991.98))
                    adjustLookupStyle(this, 'sideBySide');
            }
            else if (isModuleSelection)
                setInitialValueModuleSelection(this);
            else
                $(this).removeClass('text-right');
        }
    });
}

function hasLinkLookup(input) {
    return ($(input).data('haslinklookup'));// && ($("#" + input.id + "resultText").length));
}

function adjustLookupStyle(input, style) {
    if (style == 'inside') {
        setStyleLookupInside(input);
        let inputVal = $(input).inputVal();
        let valueWithLookup = inputVal ? (inputVal + " - " + $.trim($("#" + input.id + "RESULTTEXT").text())) : '';
        $(input).val(valueWithLookup);
    }
    else if (style == 'sideBySide') {
        setStyleLookupOutside(input);
        let valueWithLookup = $(input).inputVal();
        let endValue = valueWithLookup.indexOf(" -");
        let trueValue = valueWithLookup.slice(0, endValue == -1 ? valueWithLookup.length : endValue);
        $(input).val(trueValue);
    }
}

function setStyleLookupInside(input) {
    let parentEl = $(input).hasClass('datepicker') ? $(input).closest('.input-addon-default') : $(input.parentElement);

    $(input).removeClass('text-right');
    parentEl.removeClass(['col-md-6']).addClass(['col-md-12']);
}

function setStyleLookupOutside(input) {
    let parentEl = $(input).hasClass('datepicker') ? $(input).closest('.input-addon-default') : $(input.parentElement);

    $(input).addClass('text-right');
    parentEl.addClass(['col-md-6']).removeClass(['col-md-12']);
}

export const cxGridIncluirFiltroLookup = function (params, options) {
  params.filter = IncluirFiltrosLookup(params.filter, options);
}

function IncluirFiltrosLookup(_AFiltroAtual, { targetButton, gridLookupHandler } = {}) {
	let isFromGrid = !Boolean(targetButton);
	var datagrid = $("#dataGridModal").length ? dxDataGrid.getInstance($("#dataGridModal")) : null;
	let canBuildFilter = targetButton || gridLookupHandler;
	let filter = (_AFiltroAtual || '');

	if (filter) {
		filter = filter.replace("[[", "[");
		filter = filter.replace("]]", "]");
	}

	if (canBuildFilter) {
		const includeFilterValue = (filter, column, AValor) => {
			if (filter) filter = filter + ',"and",';

			let endTrueValue = AValor.indexOf(" -");

			AValor = AValor.slice(0, endTrueValue < 0 ? AValor.length : endTrueValue);

			return `${filter}["${column.dataField}", "=", "${AValor}"]`;
		};

		let lookupConfig = gridLookupHandler && gridLookupHandler.Lookup;
		let arraylinklookupkeyfields = (targetButton ? targetButton.data('linklookupkeyfields') : lookupConfig.LinkLookupKeyFields).split(";");
		let arraylinkKeyfields = (targetButton ? targetButton.data('linkkeyfields') : lookupConfig.LinkKeyFields).split(";");


		for (var j = 0; j < arraylinklookupkeyfields.length - 1; j++) {
			datagrid.option('columns').forEach((column) => {
				if (column.dataField == arraylinklookupkeyfields[j]) {
					let AValor;

					if (isFromGrid) {
						let rowData = gridLookupHandler.data;

						AValor = rowData[arraylinkKeyfields[j]]
					}
					else {
						let form = targetButton.data('q-field').InstanciaFormulario;
						AValor = form.getvalueByNameOrDic(arraylinkKeyfields[j]);

						if ((AValor == '') && (form.masterclass != null)) {
							const masterForm = FormManager.byname(form.masterclass);

							AValor = masterForm.getvalueByNameOrDic(arraylinkKeyfields[j]);
						}
					}
					if (AValor) {
						filter = includeFilterValue(filter, column, AValor);
					}
				}
			});
		}

		if (filter.includes(',"and",')) {
			filter = "[" + filter + "]";
		}
	}
	return filter;
}

export const fixLookupValueOnProcess = async function (forceLookupLoad) {// Arruma os lookups quando não vem CAMPOresult
    if (forceLookupLoad) {
        let promises = [];
        $(".form-control[data-haslinklookup='true']").each((index, el) => {
            let name = $(el).attr('name').split('.')[1];
            let dmclassname = $(el).data('dmclassname');
            let alias = $(el).data('alias');

            promises.push(FindLookupValue(name, dmclassname, alias))
        });
        await Promise.all(promises);
    }
    else {
        for (var i of FormManager.lookupToFix) {
            if (i.componente.length > 0 && typeof i.componente.prop('onblur') == 'function') {
                i.componente.prop('onblur')();
            }
        }
        FormManager.lookupToFix = [];
    }
}

export const FindLookupValue = async function (_AName = '', _ADMClassName = '', _AAlias = '', forceSearch = false) {
	let form = FormManager.byname(_ADMClassName, _AAlias);
	
	if (DevePesquisarLoookup(form, _AName) || forceSearch || _ADMClassName === 'TnDMSelecao') {
		let afield = form.fieldbyName(_AName);
		let searchButton = afield.linkbutton;

		afield.componente.data('value', afield.componente.val());
		afield.resultcomponentetext.html('<div class="lookupSpinner"></div>');

		let _AFieldsValue = getKeysValuesToSearchForm({
			form,
			Name: _AName,
			LinkLookupKeyFields: searchButton.data('linklookupkeyfields'),
			KeyFieldControls: searchButton.data('keyfieldcontrols'),
			LinkKeyFields: searchButton.data('linkkeyfields')
		})

		return makeRequestLookupValue({
			_ADMClassName,
			_AAlias,
			_AFieldsValue,
			_AField: afield,
			LinkLookupDataSetNome: searchButton.data('linklookupdatasetnome'),
			LinkLookupResultField: (searchButton.data('linklookupresultfield') || ''),
			_AControleResult: _AName + "RESULT",
		});
	}
	else if (itsToUseLookupInside(form.multiColumn)) { // Ao sair do campo lookup sem ser necessário a request, preenche o campo com "codigo - descricao"
		const isEmpty = (val) => [undefined, null, ''].includes(val);
		const setDescription = (element, value) => { if (!element.is(':focus')) element.val(value); };

		const /** @type {Field} */ field = form.fieldbyName(_AName);
		const component = field.componente;

		if (field && component.length && !field.controlType == ControlType.ntcLinkEdit || field?.isRequestingLookup) return;

		const { value, lookup, mask } = component.data();
		const inputValue = mask?.getCleanVal() || component.val();

		if (isEmpty(value) || isEmpty(inputValue)) return;

		setTimeout(() => {
			if (field.haspicker()) {
				const datePickerEl = field.getDatepickerElement();
				const { mask } = datePickerEl.data('mask');
				const { momentFormat } = getFormatOptions(mask);
				const dateVal = moment(value, 'YYYY-MM-DD').format(momentFormat);

				setDescription(datePickerEl, `${dateVal} - ${lookup}`);
			}
			else {
				setDescription(component, `${value} - ${lookup}`);
			}
		});
	}
}

function getKeysValuesToSearchForm({ Name, form, LinkLookupKeyFields, KeyFieldControls, LinkKeyFields }) {
	let _linkLookupKeyFields = LinkLookupKeyFields.split(';');
	let _linkKeyFields = (KeyFieldControls || LinkKeyFields).split(';');
	let fieldsValues = [];

	_linkLookupKeyFields.forEach((FieldName, index) => {
		let isLast = (index == (_linkLookupKeyFields.length - 1));
		let searchField = isLast ? Name : _linkKeyFields[index];

		let Value = form.getvalueByNameOrDic(searchField);

		if (Value == '') {
			if (form.masterclass != null) {
				const masterForm = FormManager.byname(form.masterclass);

				Value = masterForm.getvalueByNameOrDic(searchField);
			}
			if (Value == '') {
				Value = PegarValorSelecao(searchField);
			}
		}
		if (Value) {
			Value = String(Value).split(' -')[0];
		}
		else {
			Value = '';
		}

		fieldsValues[index] = { FieldName, Value };
	});

	return fieldsValues;
}

export function makeRequestLookupValue({ _ADMClassName, _AAlias, _AFieldsValue, _AField, LinkLookupDataSetNome, _AControleResult, LinkLookupResultField }) {
	if (!LinkLookupDataSetNome) return;

	let reqOptions = {
		_AConsulta: LinkLookupDataSetNome,
		_ALookupkeyfieldsValues: JSON.stringify(_AFieldsValue),
		_AResultField: LinkLookupResultField,
		_AControleResult: _AControleResult,
		_ADMClassName: _ADMClassName,
		_AAlias: _AAlias,
		_AField
	}

	return new RequestDadosLookup(reqOptions).executar();
}

function DevePesquisarLoookup(_AForm, _AName) {
    const Afield = _AForm.fieldbyName(_AName);
	
	if (!Afield?.componente || Afield?.isRequestingLookup) return false;

	const AInput = Afield.componente;
	const { value, lookup, mask } = AInput.data();
	const AValor = mask && mask.getCleanVal ? mask.getCleanVal() : AInput.val();;

	if (AValor != value || [null, undefined, '', 'Registro não encontrado'].includes(lookup)) {
        if (AValor != '') return true;

		$(Afield.resultcomponente.find('div.spinner')).remove();
		Afield.resultcomponente.val('');
		Afield.resultcomponentetext.text('Nenhum item Selecionado');
    }

    return false;
}

function setInitialValueModuleSelection(input) {
    let inputVal = $(input).data('q-field').getvalue();
    let valueWithLookup = inputVal ? (inputVal + " - " + $.trim($("#" + input.id + "RESULT").val())) : '';

    setStyleLookupInside(input);
    $(input).val(valueWithLookup);
}

function hasToBeOutsideInput(input, width) {
    let parentEl = $(input).hasClass('datepicker') ? $(input).closest('.input-addon-default') : $(input.parentElement);

    return window.innerWidth > width && parentEl.hasClass('col-md-12');
}

function hasToBeInsideInput(input, width) {
    let parentEl = $(input).hasClass('datepicker') ? $(input).closest('.input-addon-default') : $(input.parentElement);

    return (window.innerWidth < width && parentEl.hasClass('col-md-6'));
}
