import { UrlParamBuilder } from "./UrlParamBuilder";

export class RequestOperacaoDataset {
	constructor({ _ADatamodule, _AAction, _ADataset, _AProcessoParams, _ASelected }) {
		this.datamodule = _ADatamodule;
		this.action = _AAction;
		this.dataset = _ADataset;
		this.dialogs = null;
		this.processoParams = _AProcessoParams;
		this.selecionado = _ASelected;
	}

	stringifyParams(params) {
		let paramStr = JSON.stringify(params)
			.replace(/\\r/g, ' ')
			.replace(/\\n/g, '')
			.replace(/&nbsp/g, ' ')

		return `{p=${paramStr}}`;
	}

  /**
   * 
   * @param {function} _ASucessFunction Success request handler
   * @param  {any} additionalParams Params passed to _ASucessFunction
   */
	executar(_ASucessFunction, additionalParams) {
		var aself = this;
		var d = new $.Deferred();
		var params = {
			ClassName: this.datamodule,
			Action: this.action,
			Dataset: this.dataset,
		};

		if (this.processoParams) {
			params.Params = this.processoParams;
		}

		if (this.dialogs) {
			params.dialogs = this.dialogs;
		}

		if (this.selecionado) {
			params.selecionado = this.selecionado;
		}

		$.ajax({
			type: 'POST',
			url: "nHttpServicos/ExecutarOperacaoDataset",
			cache: true,
			dataType: 'html',
			data: this.stringifyParams(params),
			error: defaulRequestError,
			success: function (data) {
				if (typeof data === 'object') {
					var retorno = data;
				}
				else {
					var retorno = JSON.parse(data);
				}

				KTApp.unblockPage();

				if (retorno.data.dialogs != null) {
					if (DialogsSemResposta(retorno.data.dialogs)) {
						var AOnRequest = function (Scope) {
							aself.dialogs = Scope.dialogs;
							aself.executar(_ASucessFunction, additionalParams);
						};

						ProcessarDialogsRetorno(retorno, AOnRequest, _ASucessFunction, additionalParams);
					}
					else {
						_ASucessFunction(retorno, additionalParams);
					}
				}
				else {
					_ASucessFunction(retorno, additionalParams);
				}
				
				d.resolve();
			}
		});

		return d.promise();
	}
}

export class RequestDadosCadastro{
    constructor(_ACadastro, _AAlias) {
        this.cadastro = _ACadastro;
        this.alias    = _AAlias;
        this.origem   = '';
        this.detalhes = null;
		this.filtros  = {};
		this.idrequest = '0';
        this.chaveprimariamestre = '';
	}

    addFiltro(_ACampo, _AValor){
        this.filtros[_ACampo] = _AValor;
    }

    SetChavePrimariaMestre(_AChavePrimaria){
        if(_AChavePrimaria){
            this.chaveprimariamestre = '&_AChavePrimariaMestre='+_AChavePrimaria;
        }
        else{
            this.chaveprimariamestre = '';
        }          
    }

    addDetalhes(_AFormulario){

    }

    ehDetalhe(){
		return !this.detalhes;
    }

    executar(_ASucessFunction){
		var builder = new UrlParamBuilder("nHttpServicos/DadosCadastro");

		builder.addParam("_AClassName", this.cadastro);
		builder.addParam("_AAlias", this.alias);
		builder.addParam("_AFiltro", JSON.stringify(this.filtros));
		builder.addParam("_AOrigem", this.origem);
		builder.addParam("_ADetalhes", JSON.stringify(this.detalhes || {}));
		builder.addParam("_AEhdetalhe", this.ehDetalhe());
		builder.addParam("_AIDRequest", this.idrequest + this.chaveprimariamestre);
		builder.addParam("_AFiltroFixo", this._AFiltroFixo);

		return $.ajax({
			'type': 'GET',
			'url': builder.buildUrl(),
			'cache': true,
			'error': defaulRequestError,
			'success': function (retorno) {
				_ASucessFunction(retorno);
			}
		});
	}
}

export class RequestRespostaCaptcha{
    constructor(_AID, _AResposta, _Atimer, _AonSucess, _AonError){
        this.id = _AID;
        this.resposta = _AResposta;
        this.timer = _Atimer;
        this.onSucess  = _AonSucess;
        this.onError   = _AonError;
    }

    executar(){
        var origem = this;
				var builder = new UrlParamBuilder("nHttpServicos/RespostaCaptcha");
				builder.addParam("_AID", origem.id);
				builder.addParam("_AResposta", origem.resposta);
        this.timer.IniciarExecucao();       
        if (this.resposta){		
            var req = $.ajax({  type: 'GET',
                                url: builder.buildUrl(),
                                cache: false,
                                success: function (request) {
                                    if (request.data.Erro){
                                        qNotifier.error(request.data.Erro);
                                    }
                                    origem.onSucess(origem.id);                                    
                                },
                                error: function (jqXHR, textStatus, errorThrown) {
                                    qNotifier.error(errorThrown);
                                    origem.onError(origem.id);                                    
                                }
            });
        };

    }
}

export class RequestExecutarExpression{    
    constructor(_ANome, params){
				this.resultado = null;
				this.FUNC = {
					nome: _ANome,
					params: params
				};        
    }
    
    dorequest(){
        var origem = this;
				var builder = new UrlParamBuilder("nHttpServicos/ExecutarExpression");
				builder.addParam("_AFunc", JSON.stringify(this.FUNC));
        return $.ajax({
            type: 'GET',
            url: builder.buildUrl(),
            cache: false,		
            success: function (request) {
                origem.resultado = request.Result;                    
            },
            error: function (jqXHR, textStatus, errorThrown) {
                origem.resultado = null;
                qNotifier.error(errorThrown);                    
            }		
        });        
    }

    async executar(){
        await this.dorequest();
	    return this.resultado;
    }
}

export class RequestConsultaCaptcha{
    constructor(_AID, _AonSucess, _AonError){
        this.id       = _AID;
        this.onSucess = _AonSucess;
        this.onError  = _AonError;
    }

    executar(){
        var origem = this;
				var builder = new UrlParamBuilder("nHttpServicos/ConsultaCaptcha");
				builder.addParam("_AID", origem.id);
        if (this.id){
            $.ajax({
                type: 'GET',
                url: builder.buildUrl(),
                cache: false,		
                success: function (request) {
                    origem.onSucess(request.data);                    
                },
                error: function (jqXHR, textStatus, errorThrown) {
                    qNotifier.error(errorThrown);
                    origem.onError();
                }		
            });
        }
    }
}

export class RequestDadosLookup {
	constructor({ _AConsulta, _ALookupkeyfieldsValues, _AResultField, _AControleResult, _ADMClassName, _AAlias, _AField }) {
		this.Consulta = _AConsulta;
		this.ResultField = _AResultField;
		this.LookupkeyfieldsValues = _ALookupkeyfieldsValues;
		this.ControleResult = _AControleResult;
		this.DMClassName = _ADMClassName;
		this.Alias = _AAlias;
		this.Field = _AField;
	}

	async executar() {
		const builder = new UrlParamBuilder('nHttpServicos/DadosLookup');
		const thatfield = this.Field;

		builder.addParam("_AConsulta", this.Consulta);
		builder.addParam("_ALookupkeyfieldsValues", this.LookupkeyfieldsValues);
		builder.addParam("_AResultField", this.ResultField);
		builder.addParam("_AControleResult", this.ControleResult);
		builder.addParam("_ADMClassName", this.DMClassName);
		builder.addParam("_AAlias", this.Alias);
		builder.addParam("_AOrigem", "lookup");

		if (thatfield) {
			thatfield.isRequestingLookup = true;
		}

		return $.ajax({
			'type': 'GET',
			'url': builder.buildUrl(),
			'cache': true,
			'error': function () {
				if (thatfield) { // lookup on Form
					thatfield.resultcomponentetext.text('Falha ao buscar lookup...');
					thatfield.isRequestingLookup = false;
				}
			},
			'success': async function (res) {
				if (thatfield) { // lookup on Form
					await CarregarDataCadastro(res);
					thatfield.isRequestingLookup = false;
					thatfield.loaded.next((res || {}).data);
				}
			}
		});
	}
}

export class PostWithForm{
    constructor(Action){
        this.localform = document.createElement('form');
        this.localform.action = Action;
        this.localform.method = 'POST';
        document.body.appendChild(this.localform);
    }

    addParam(Name, value){
        let i = document.createElement('input');
        i.type = 'hidden';
        i.name = Name;
        i.value = value;
        this.localform.appendChild(i);
    }

    submit(){
        this.localform.submit();
    }
}