export function setDatePicker() {
	var prom = $.Deferred();

	$.fn.datepicker.dates['pt-BR'] = {
		days: ["Domingo", "Segunda", "Terça", "Quarta", "Quinta", "Sexta", "Sábado"],
		daysShort: ["Dom", "Seg", "Ter", "Qua", "Qui", "Sex", "Sáb"],
		daysMin: ["Do", "Se", "Te", "Qu", "Qu", "Se", "Sa"],
		months: ["Janeiro", "Fevereiro", "Março", "Abril", "Maio", "Junho", "Julho", "Agosto", "Setembro", "Outubro", "Novembro", "Dezembro"],
		monthsShort: ["Jan", "Fev", "Mar", "Abr", "Mai", "Jun", "Jul", "Ago", "Set", "Out", "Nov", "Dez"],
		today: "Hoje",
		monthsTitle: "Meses",
		clear: "Limpar",
		format: "dd/mm/yyyy"
	};

	$('.datepicker').each(function (index, element) {
		configureDatepicker($(element))
	})

	return prom.resolve("success");
}

export function configureDatepicker(datepickerEl) {
	let mask = datepickerEl[0].dataset['mask'];
	let prevDate = datepickerEl.data('original-value');
	var ismultidate = datepickerEl.hasClass('multidate');
	let formatOptions = getFormatOptions(mask);

	datepickerEl.datepicker({
		todayHighlight: true,
		language: "pt-BR",
		zIndexOffset: 100,
		autoclose: !ismultidate,
		showOnFocus: false,
		multidate: ismultidate,
		format: formatOptions.format,//isMonthYear(datepicketEl) ? "mm/yyyy" : "dd/mm/yyyy", 
		minViewMode: formatOptions.view,
		viewMode: formatOptions.view
	});

	datepickerEl.prev().click(function () {
		if (!datepickerEl.is(':disabled')) {
			datepickerEl.datepicker().shown ? datepickerEl.datepicker('hide') : datepickerEl.datepicker('show');
		}
	}).css('cursor', 'pointer');

	if (prevDate != undefined && prevDate != "") {
		datepickerEl.datepicker('setUTCDate', new Date(prevDate));
	}
	datepickerEl.on('input', propagateToInput);
	datepickerEl.on('changeDate', propagateToInput)
}

export function getFormatOptions(mask) {
	const DATEFORMAT = {
		'00/0000': {
			view: 'months',
			word: 'month',
			regex: /\d\d\/\d\d\d\d/,
			format: 'mm/yyyy',
			momentFormat: 'MM/YYYY'
		},
		'0000': {
			view: 'years',
			word: 'year',
			regex: /\d\d\d\d/,
			format: 'yyyy',
			momentFormat: 'YYYY'
		},
		'00/00': {
			view: 'days',
			word: 'day',
			format: 'dd/mm',
			regex: /\d\d\/\d\d/,
			momentFormat: 'DD/MM'
		},
		'default': {
			view: 'days',
			word: 'day',
			format: 'dd/mm/yyyy',
			regex: /\d\d\/\d\d\/\d\d\d\d/,
			momentFormat: 'DD/MM/YYYY'
		}
	};

	return (DATEFORMAT[mask] || DATEFORMAT['default']);
}

const propagateToInput = function ({ currentTarget }) {
	const putCssValidation = (el, date) => date.isValid() ? el.removeClass('is-invalid') : el.addClass('is-invalid');
	const dateStringOrNull = (dateString, regex) => (dateString.match(regex) && dateString);
	const getFormattedDate = (dateString, mask) => {
		let { word, regex, momentFormat } = getFormatOptions(mask);
		let date = moment(dateStringOrNull(dateString, regex), momentFormat)

		return date.isValid() ? date.startOf(word).format('YYYY-MM-DD') : null;
	}

	let { mask } = currentTarget.dataset;
	let { value } = currentTarget;
	let inputHidden = $(currentTarget).parent().next('input[type=hidden]');
	let formattedDate = getFormattedDate(value, mask);

	putCssValidation($(currentTarget), moment(formattedDate));

	inputHidden
		.val(formattedDate)
		.change();

	if (!formattedDate) inputHidden.data({ value: null });
}