export function CarregarTreeList(_ANome, _AElement) {
	import('jstree').then(() => {
		LoadTreeList(_ANome, _AElement);
	})
}
export function LoadTreeList(_ANome, _AElement) {
		var treeView;

	if (_ANome) {
		treeView = $('#' + _ANome);
	}
	else if (_AElement) {
		treeView = _AElement;
	}
	else {
		treeView = $('#tree');
	}

	if (treeView.length > 0) {
		var treeContent = treeView.data('tree-content');

		treeView.removeData('tree-content');
		treeView.removeAttr('data-tree-content');
		
		treeView.jstree({
			core: {
				themes: {
					responsive: false
				},
				check_callback: true,
				data: treeContent.map(formatTreeNodes)
			},
			types: {
				default: {
					icon: "fa fa-folder kt-font-info"
				}
			},
			plugins: ["types"]
		});
	}

	if ($('[table-sort="true"]').length > 0) {
		$('[table-sort="true"]').tablesorter();
	}
}

function formatTreeNodes(item) {
	var children = [],
		icon = "";

	if (item.nodes) {
		children = item.nodes.map(formatTreeNodes);
	}

	if (item.icon) {
		switch (item.icon) {
			case 'glyphicon glyphicon-remove':
				icon = 'la la-remove kt-font-danger';
				break;
			case 'glyphicon glyphicon-warning-sign':
				icon = 'la la-warning kt-font-warning';
				break;
			case 'glyphicon glyphicon-ban-circle':
				icon = 'la la-ban kt-font-danger';
				break;
			case 'glyphicon glyphicon-text-size':
				icon = 'la la-text-height kt-font-primary';
				break;
			case 'glyphicon glyphicon-info-sign':
				icon = 'la la-info kt-font-info';
				break;
			case 'glyphicon glyphicon-ok':
				icon = 'la la-check kt-font-success';
				break;
			case 'glyphicon glyphicon-trash':
				icon = 'la la-trash kt-font-danger';
				break;
			case 'glyphicon glyphicon-cloud':
				icon = 'la la-cloud kt-font-info';
				break;
		}
	}

	if (item.TreeLink) { // Build 'clique aqui' link to "Cadastro" or "Processo"
		const buildFilter = ({ KeyFields, KeyValues }) => {
			const arrayToObject = (keys, values) => keys.reduce((obj, key, index) => ((obj[key] = values[index]), obj), {});
			let filter = null;

			if (KeyFields) {
				filter = arrayToObject(KeyFields.split(';'), KeyValues)
			}

			return [
				filter,
				KeyFields.split(';')
			];
		}

		let { TreeLink } = item;

		let [filter, filterKeyFields] = buildFilter(TreeLink.Filtrar);
		let [suggestion, suggestionKeyFields] = buildFilter(TreeLink.Sugerir);

		let onclickString = '';

		if (TreeLink.Type == "TnTreeLinkDM") {
			let actionParam = {
				_AClasse: TreeLink.ClassLink,
				AFiltros: filter,
				ASugerir: suggestion,
				AGrid: "",
				ANovo: false,
				AChaveMestre: filterKeyFields,
			}

			onclickString = `AbrirCadastroModal(${JSON.stringify(actionParam)})`;
		}
		else {
			let actionParam = {
				_AClasse: TreeLink.ClassLink,
				AFiltros: suggestion,
			}

			onclickString = `AbrirProcessoModal(${JSON.stringify(actionParam)})`;
		}

		let linkHtml = `<span class='kt-link kt-link--orange-q' data-is-redirect-link='true' onclick='${onclickString}'>Clique aqui.</a>`;

		console.log('item', item)
		item.text = item.text.replace(/clique aqui.$/, linkHtml);
	}

	return {
		icon: icon,
		text: item.text,
		children: children
	}
}

export function CarregarCheckTreeView({ TreeviewName, element, _ANome, selectionMode }) {
	import('jstree').then(() => {
		LoadCheckTreeView({ TreeviewName, element, _ANome, selectionMode });
	});
}

export function LoadCheckTreeView({ TreeviewName, element, _ANome, selectionMode }) {
		let checkTreeView;

	if (element && element.length) checkTreeView = element;	
	else checkTreeView = $("#" + TreeviewName);
	 
	if (checkTreeView.length > 0) {
		let treeContent = checkTreeView.data("tree-content");
		checkTreeView.removeAttr('data-tree-content');

		checkTreeView.jstree({
			core: {
				themes: {
					responsive: true,
					icons: false
				},
				check_callback: true,
				data: treeContent.map(formatCheckTreeNode),
				...getSelectionConfig(selectionMode),
			},
			...getCheckboxConfig(selectionMode),
		});
	}

	$.contextMenu({
		selector: '#' + TreeviewName,
		trigger: 'right',

		callback: function (key, options) {
			if (key == 'check_all') checkTreeView.jstree().check_all();
			else checkTreeView.jstree().uncheck_all();
		},
		items: {
			"check_all": { name: "Marcar Todos" },
			"uncheck_all": { name: "Desmarcar Todos" },
		}
	});

	selectCheckTreeListener(TreeviewName, _ANome)
}

function formatCheckTreeNode(item) {
	let children = [];
	let value = item.valor != -1 ? item.valor : null

	if (item.nodes) {
		children = item.nodes.map(formatCheckTreeNode);
	}

	return {
		text: item.nome,
		children: children,
		li_attr: { node_value: value }
	};
}

function getSelectionConfig(selectionMode = 'multiple') {
	return {
		multiple: selectionMode === 'single' ? false : true,
	};
}

function getCheckboxConfig(selectionMode = 'multiple') {
	if (selectionMode == 'single') {
		return {};
	}

	return {
		checkbox: {
			default: {
				icon: "fa fa-folder kt-font-info"
			}
		},
		plugins: ["checkbox"],
	};
}

function selectCheckTreeListener(TreeviewName, _ANome) {
	let tree = $('#' + TreeviewName);

	tree.on('changed.jstree', function (e, data) {
		let checkSelected = [];

		data.selected.forEach(element => {
			let value = data.instance.get_node(element).li_attr.node_value;
			if (value) checkSelected.push(value);
		});

		$('#ctrl' + _ANome).val(checkSelected.join(';'));
	})
}