import { expect } from "chai";
import { JSDOM } from "jsdom";
import { DOMBuilder } from "../../../Utils/DOMBuilder";


describe('Criação ActionAvisos', async () => {
    let DOM;

    before(async () => {
        DOM = await DOMBuilder.build();
        await DOMBuilder.includeGlobalsJq(DOM);

        global.KTUtil = { scrollInit: () => { } };
    });

    after(() => {
        DOMBuilder.clearGlobals();
        
        global.KTUtil = null;
    });

    it('Deve criar as tags span com o texto e a cor corretamente', async () => {
        let notifications = ['teste @rgb(10,10,55) para dia 10 @rgb(232,11,23) fazer @rgb(11,255,10) verificação'];
        let { getSpanText } = (await import('../../../../js/DynamicImports/Actions/ActionAvisos'))
        let [spanText] = getSpanText({ notifications });

        expect(spanText[0]).to.be.equal('teste ')
        
        expect(spanText[1].length).to.be.equal(1)
        expect(spanText[1].css('color')).to.be.equal('rgb(10, 10, 55)');
        expect(spanText[1].html()).to.be.equal('para dia 10 ')

        expect(spanText[2].length).to.be.equal(1)
        expect(spanText[2].css('color')).to.be.equal('rgb(232, 11, 23)');
        expect(spanText[2].html()).to.be.equal('fazer ')
        
        expect(spanText[3].length).to.be.equal(1)
        expect(spanText[3].css('color')).to.be.equal('rgb(11, 255, 10)');
        expect(spanText[3].html()).to.be.equal('verificação')
    });

    it('Deve criar o popup das notificações', async () => {        
        let notifications = ['teste @rgb(10,10,55) para dia 10 @rgb(232,11,23) fazer @rgb(11,255,10) verificação'];
        let { getSpanText, getPopoverBody } = (await import('../../../../js/DynamicImports/Actions/ActionAvisos'))

        let spanText = getSpanText({ notifications });
        let body = getPopoverBody(spanText, $(''));
        let closeBtn = body.find('span.mb-auto');
        let closeBtnIcon = closeBtn.find('i.las.la-times')
        let notificationList = body.find('ul.workspace-notification')
        let items = notificationList.find('li.workspace-notification__item');
        let prependIcon = items.eq(0).find('.workspace-notification__item-prepend .las.la-exclamation-triangle')
        let notificationText = items.eq(0).find('.workspace-notification__item-text');
        let texts = notificationText.find('span');

        expect(body.length).to.be.equal(1);
        expect(closeBtn.length).to.be.equal(1);
        expect(notificationList.length).to.be.equal(1);
        expect(items.length).to.be.equal(1);

        expect(prependIcon.length).to.be.equal(1);

        expect(notificationText.length).to.be.equal(1);

        expect(texts.eq(0).html()).to.be.equal('para dia 10 ')
        expect(texts.eq(0).css('color')).to.be.equal('rgb(10, 10, 55)');

        expect(texts.eq(1).html()).to.be.equal('fazer ')
        expect(texts.eq(0).css('color')).to.be.equal('rgb(10, 10, 55)');

        expect(texts.eq(2).html()).to.be.equal('verificação')
        expect(texts.eq(0).css('color')).to.be.equal('rgb(10, 10, 55)');
    })
});