import { expect, use} from 'chai';
import { DynamicImport } from '../../../js/Utils/DynamicImport';
import { BuildModalGrid } from '../../../js/DynamicImports/Grid/ModalGridConfig';
import * as ModalGridConfig from '../../../js/DynamicImports/Grid/ModalGridConfig';

/**
 * To work with async code rejection
 */
use(require('chai-as-promised'));

describe('Importação dinâmica de módulos', function () {
	it('Deve retornar BuildModalGrid', async function () {
		const entity = await DynamicImport.Import('Grid/ModalGridConfig', 'BuildModalGrid');

		expect(entity).to.be.equal(BuildModalGrid);
	});

	it('Deve retornar undefined para namedImport inexistente', async function () {
		const entity = await DynamicImport.Import('Grid/ModalGridConfig', 'da');

		expect(entity).to.be.undefined;
	});

	it('Deve retornar CodigoEventoConfigFolha para namedImport vazio', async function () {
		const entity = await DynamicImport.Import('Grid/ModalGridConfig', '');

		expect(entity).to.be.equal(ModalGridConfig);
	});

	it('Deve retornar CodigoEventoConfigFolha sem namedImport', async function() {
		const entity = await DynamicImport.Import('Grid/ModalGridConfig');

		expect(entity).to.be.equal(ModalGridConfig);
	});

	it('Deve lançar exceção para módulo inexistente', async function() {
		const callEntity = async () => await DynamicImport.Import('entityDoesntExists');

		return expect(callEntity()).to.be.rejected;
	});
})