import { expect } from "chai";
import { JSDOM } from "jsdom";
import { NtcCheckTreeView } from "../../../../../ts/components/fra-control/ntc-check-tree-view/ntc-check-tree-view";
import { isTreeViewItem } from "../../../../../ts/components/fra-control/types";
import { DOMBuilder } from "../../../../Utils/DOMBuilder";
import { checkTreeViewGrupoOperacao } from "./field-config/check-tree-view-grupo-operacao";
import { checkTreeViewSituacaoFuncionario } from "./field-config/check-tree-view-situacao-funcionario";

/**
 * A exception: UnhandledPromiseRejectionWarning: ReferenceError: MutationObserver is not defined
 * irá acontecer devido ao componente NtcCheckTreeView realizar uma chamada ao construtor da treview $.jstree
 * que atualmente não temos como testar. 
 */


describe('Criação dinâmica do controle ntcCheckTreeView', () => {
    let DOM: JSDOM;

    before(async () => {
        DOM = await DOMBuilder.build();
        await DOMBuilder.includeGlobalsJq(DOM);
    });

    after(DOMBuilder.clearGlobals);

    it('Deve criar os elementos do controle', () => {
        const element = <JQuery<HTMLElement>>NtcCheckTreeView(checkTreeViewSituacaoFuncionario);

        let label = element.find('label#labelctrloptPSITUACAO');
        let divCol = element.children().eq(1);

        let checkTreeViewContainer = divCol.find('div#checktreeviewPSITUACAO');
        let inputHidden = divCol.find('input#ctrlPSITUACAO')

        let treeviewConf = checkTreeViewContainer.data('tree-content');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);

        expect(checkTreeViewContainer.length).to.equal(1);
        expect(inputHidden.length).to.equal(1);

        expect(isTreeViewItem(treeviewConf)).to.be.true;
    });

    it('Deve criar os elementos do controle', () => {
        const element = <JQuery<HTMLElement>>NtcCheckTreeView(checkTreeViewGrupoOperacao);

        let label = element.find('label#labelctrloptPGRUPOOPERACAO');
        let divCol = element.children().eq(1);

        let checkTreeViewContainer = divCol.find('div#checktreeviewPGRUPOOPERACAO');
        let inputHidden = divCol.find('input#ctrlPGRUPOOPERACAO')

        let treeviewConf = checkTreeViewContainer.data('tree-content');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);

        expect(checkTreeViewContainer.length).to.equal(1);
        expect(inputHidden.length).to.equal(1);

        expect(isTreeViewItem(treeviewConf)).to.be.true;
    });
});