import { expect } from "chai";
import { JSDOM } from "jsdom";
import { NtcCheckbox } from "../../../../../ts/components/fra-control/ntc-checkbox/ntc-checkbox";
import { DOMBuilder } from "../../../../Utils/DOMBuilder";
import { checkboxField } from "./field-config/checkbox-field";

describe('Criação dinâmica do controle ntcCheckBox', () => {
    let DOM: JSDOM;

    before(async () => {
        DOM = await DOMBuilder.build();
        await DOMBuilder.includeGlobalsJq(DOM);
    });

    after(DOMBuilder.clearGlobals);

    it('Deve criar os elementos do controle', () => {
        const element = <JQuery<HTMLElement>>NtcCheckbox(checkboxField);

        let label = element.find('label#labelctrloptPINCLUIRPERIODOANTERIOR');
        let divCol = element.children().eq(1);
        let checkboxWrapper = divCol.find('div.checkbox-wrapper');
        let labelCheckbox = checkboxWrapper.find('label.kt-checkbox');
        let inputVisual = labelCheckbox.find('input[type=checkbox]');
        let inputHidden = labelCheckbox.find('input#ctrlPINCLUIRPERIODOANTERIOR[type=hidden]');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);
        expect(checkboxWrapper.length).to.equal(1);
        expect(labelCheckbox.length).to.equal(1);
        expect(inputVisual.length).to.equal(1);
        expect(inputHidden.length).to.equal(1);
    });

    it('Change value inputHidden (simulate user click check)', () => {
        const element = <JQuery<HTMLElement>>NtcCheckbox(checkboxField);
        let inputVisual = element.find('input[type=checkbox]');
        let inputHidden = element.find('input#ctrlPINCLUIRPERIODOANTERIOR[type=hidden]');

       inputVisual.prop('checked', true).trigger('change');
       
       expect(inputHidden.val()).to.be.equal('1');
    });

    it('Change value inputHidden (simulate user click uncheck)', () => {
        const element = <JQuery<HTMLElement>>NtcCheckbox(checkboxField);
        let inputVisual = element.find('input[type=checkbox]');
        let inputHidden = element.find('input#ctrlPINCLUIRPERIODOANTERIOR[type=hidden]');

       inputVisual.prop('checked', false).trigger('change');
       
       expect(inputHidden.val()).to.be.equal('0');
    });

    it('Change value inputVisual (simulate form.setvalue(1))', () => {
        const element = <JQuery<HTMLElement>>NtcCheckbox(checkboxField);
        let inputVisual = element.find('input[type=checkbox]');
        let inputHidden = element.find('input#ctrlPINCLUIRPERIODOANTERIOR[type=hidden]');

       inputHidden.val(1).trigger('change');
       
       expect(inputVisual.prop('checked')).to.be.equal(true);
    });
});