import { expect } from "chai";
import { JSDOM } from "jsdom";
import { NtcComboBox } from "../../../../../ts/components/fra-control/ntc-combo-box/ntc-combo-box";
import { DOMBuilder } from "../../../../Utils/DOMBuilder";
import { consultaBalanceteField } from "./field-config/consulta-balancete-field";
import { relacionarFilialField } from "./field-config/relacionar-filial-field";

describe('Criação dinâmica do controle ntcComboBox', () => {
    let DOM: JSDOM;

    before(async () => {
        DOM = await DOMBuilder.build();
        await DOMBuilder.includeGlobalsJq(DOM);
    });

    after(DOMBuilder.clearGlobals);

    it('Deve criar os elementos do controle', () => {
        const element = <JQuery<HTMLElement>>NtcComboBox(consultaBalanceteField);

        let label = element.find('label#labelctrloptPTIPOCONSULTABALANCETE');
        let divCol = element.children().eq(1);
        let select = divCol.find('select#ctrlPTIPOCONSULTABALANCETE');
        let options = select.find('option');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);
        expect(select.length).to.equal(1);
        expect(options.length).to.equal(3);

        expect(options.eq(0).val()).to.equal('0');
        expect(options.eq(0).text()).to.equal('Débito e Crédito');

        expect(options.eq(1).val()).to.equal('1');
        expect(options.eq(1).text()).to.equal('Débito, Crédito e Movimento');
        expect(options.eq(1).prop('selected')).to.equal(true);

        expect(options.eq(2).val()).to.equal('2');
        expect(options.eq(2).text()).to.equal('Movimento');
    });

    it('Deve criar os elementos do controle', () => {
        const element = <JQuery<HTMLElement>>NtcComboBox(relacionarFilialField);

        let label = element.find('label#labelctrloptPRELACIONARESTAB');
        let divCol = element.children().eq(1);
        let select = divCol.find('select#ctrlPRELACIONARESTAB');
        let options = select.find('option');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);
        expect(select.length).to.equal(1);
        expect(options.length).to.equal(4);

        expect(options.eq(0).val()).to.equal('0');
        expect(options.eq(0).text()).to.equal('Não');

        expect(options.eq(1).val()).to.equal('1');
        expect(options.eq(1).text()).to.equal('Sim, sem sintética');

        expect(options.eq(2).val()).to.equal('2');
        expect(options.eq(2).text()).to.equal('Sim, com sintética');

        expect(options.eq(3).val()).to.equal('3');
        expect(options.eq(3).text()).to.equal('Sim, sintética mais uma filial');
    });
});