import { expect } from 'chai';
import { JSDOM } from 'jsdom';

import { NtcEdit } from '../../../../../ts/components/fra-control/ntc-edit/nct-edit';
import { DOMBuilder } from '../../../../Utils/DOMBuilder';
import { configCurrencyInput } from './field-config/currency-input';
// import { configDateInput } from './field-config/date-input';
import { configDefaultInput } from './field-config/default-input';
import { configMaskedInput } from './field-config/masked-input';
import chai from 'chai';
import chaiArrays from 'chai-arrays';

chai.use(chaiArrays);

describe('Criação dinâmica do controle ntcEdit', () => {
    let DOM: JSDOM;

    before(async () => {
        DOM = await DOMBuilder.build();
        await DOMBuilder.includeGlobalsJq(DOM);
    });

    after(DOMBuilder.clearGlobals);

    it('Deve criar controle com a variação padrão', () => {
        const element = <JQuery<HTMLElement>>NtcEdit(configDefaultInput);

        let label = element.find('label');
        let inputWrapper = element.children().eq(1);
        let input = inputWrapper.find('input');

        expect(label.length).to.equal(1);
        expect(inputWrapper.length).to.equal(1);
        expect(input.length).to.equal(1);
    });

    it('Deve conter todos os atributos necessários para a variação padrão', () => {
        var element = <JQuery<HTMLElement>>NtcEdit(configDefaultInput);

        let inputWrapper = element.children().eq(1);
        let input = inputWrapper.find('input');

        let classList = input.attr('class')?.split(' ');
        let expectClassList = ['form-control', 'kt-input', 'cad-control', 'cad-logchange', 'ctrlPNUMDOCUMENTO'];

        expect(inputWrapper.length).to.be.equal(1);
        expect(input.length).to.be.equal(1);

        expect(classList).to.be.containingAllOf(expectClassList);

        expect(input.attr('autocomplete')).to.be.equal('off'); 
        expect(input.attr('data-alias')).to.not.be.undefined;
        expect(input.attr('data-autoincrementar')).to.be.equal('false'); 
        expect(input.attr('data-classname')).to.be.equal('TnCtbDPRelacionarLctoFiscalSocietario');
        expect(input.attr('data-dicionario')).to.be.undefined; 
        expect(input.attr('disabled')).to.be.undefined;
        expect(input.attr('id')).to.be.equal('ctrlPNUMDOCUMENTO'); 
        expect(input.attr('name')).to.be.equal('p.PNUMDOCUMENTO'); 
        expect(input.attr('readonly')).to.be.undefined; 
        expect(input.attr('required')).to.be.undefined;
        expect(input.attr('step')).to.be.equal('any'); 
        expect(input.attr('style')).to.be.undefined; 
        expect(input.attr('title')).to.be.equal('Infomre o número do documento para filtro dos lançamentos'); 
        expect(input.attr('type')).to.be.equal('text'); 					
        expect(input.attr('value')).to.be.undefined;
    })

    /**
     * não funcina devido a dependencia de plugin jquery
     */

    // it('Deve criar controle com a variação de data', () => {
    //     const element = <JQuery<HTMLElement>>NtcEdit(configDateInput);

    //     let label = element.find('label');
    //     let inputWrapper = element.children().eq(1);
    //     let inputGroup = inputWrapper.find('div.input-group');
    //     let inputGroupPrepend = inputGroup.find('div.input-group-prepend');
    //     let inputGroupText = inputGroupPrepend.find('span.input-group-text');
    //     let icon = inputGroupText.find('i.far.fa-calendar-alt');
    //     let input = inputGroup.find('input');
    //     let inputHidden = inputGroup.siblings('input#ctrlDATALCTOCTB');

    //     expect(label.length).to.be.equal(1);
    //     expect(inputWrapper.length).to.be.equal(1);
    //     expect(input.length).to.be.equal(1);
    //     expect(inputHidden.length).to.be.equal(1);
    //     expect(inputGroup.length).to.be.equal(1);
    //     expect(inputGroupPrepend.length).to.be.equal(1);
    //     expect(inputGroupText.length).to.be.equal(1);
    //     expect(icon.length).to.be.equal(1);
    // });


/**
 * 
 * 
 * Não funciona devido a dependencia de plugin jquery.
 * 
 */

    // it('Deve conter todos os atributos necessários para a variação data', () => { 
    //     var element = NtcEdit(configDateInput);

    //     let inputWrapper = element.children().eq(1);
    //     let inputGroup = inputWrapper.find('div.input-group');
    //     let input = inputGroup.find('input');
    //     let inputHidden = inputGroup.siblings('input#ctrlDATALCTOCTB');

    //     let classList = input.attr('class')?.split(' ');
    //     let expectClassList = ['form-control', 'kt-input', 'cad-control', 'cad-logchange', 'datepicker', 'ctrlDATALCTOCTB'];
        
    //     expect(inputWrapper.length).to.be.equal(1);
    //     expect(inputGroup.length).to.be.equal(1);
    //     expect(input.length).to.be.equal(1);
    //     expect(inputHidden.length).to.be.equal(1);
        
    //     expect(inputHidden.attr('autocomplete')).to.be.equal('off'); 
    //     expect(inputHidden.attr('type')).to.be.equal('hidden'); 
    //     expect(inputHidden.attr('class')).to.be.equal('hasdatepicker'); 
    //     expect(inputHidden.attr('id')).to.be.equal('ctrlDATALCTOCTB'); 
    //     expect(inputHidden.attr('name')).to.be.equal('p.DATALCTOCTB');
    //     expect(inputHidden.attr('value')).to.be.undefined;
    //     expect(inputHidden.attr('data-Dicionario')).to.be.undefined;
    //     expect(inputHidden.attr('data-classname')).to.be.equal('TnCtbDMLctoCtb');
    //     expect(inputHidden.attr('data-alias')).to.not.be.undefined;

    //     expect(classList).to.be.containingAllOf(expectClassList);
    //     expect(input.attr('autocomplete')).to.be.equal('off');
    //     expect(input.attr('data-alias')).to.not.be.undefined;
    //     expect(input.attr('data-classname')).to.be.equal('TnCtbDMLctoCtb');
    //     expect(input.attr('data-dicionario')).to.be.undefined;
    //     expect(input.attr('data-mask')).to.be.equal('00/00/0000');
    //     expect(input.attr('data-original-value')).to.be.undefined;
    //     expect(input.attr('disabled')).to.be.undefined;
    //     expect(input.attr('readonly')).to.be.undefined;
    //     expect(input.attr('required')).to.be.equal('required');
    //     expect(input.attr('style')).to.be.undefined;
    //     expect(input.attr('title')).to.be.equal('Data do lançamento contábil');
    //     expect(input.attr('type')).to.be.equal('text');  
    //     expect(input.attr('value')).to.be.undefined;
    // })

    it('Deve criar controle com a variação de máscara', () => {
        const element = <JQuery<HTMLElement>>NtcEdit(configMaskedInput);

        let label = element.find('label');
        let inputWrapper = element.children().eq(1);
        let input = inputWrapper.find('input');

        expect(label.length).to.equal(1);
        expect(inputWrapper.length).to.equal(1);
        expect(input.length).to.equal(1);
    });

    it('Deve conter todos os atributos necessários para a variação máscara', () => {
        var element = <JQuery<HTMLElement>>NtcEdit(configMaskedInput);

        let inputWrapper = element.children().eq(1);
        let input = inputWrapper.find('input');

        let classList = input.attr('class')?.split(' ');
        let expectClassList = ['form-control', 'kt-input', 'cad-control', 'cad-logchange', 'text-right', 'ctrlCODIGOHISTCTB']
        
        expect(inputWrapper.length).to.be.equal(1);
        expect(input.length).to.be.equal(1);
        
        expect(classList).to.be.containingAllOf(expectClassList);

        expect(input.attr('autocomplete')).to.be.equal('off');
        expect(input.attr('data-autoincrementar')).to.be.equal('true');
        expect(input.attr('data-alias')).to.not.be.undefined;
        expect(input.attr('data-mask')).to.be.equal('00000');
        expect(input.attr('data-mask-reverse')).to.be.equal('true');				
        expect(input.attr('data-dicionario')).to.be.equal('CODIGOHISTCTB');
        expect(input.attr('data-classname')).to.be.equal('TnCtbDMHistoricoCtb');
        expect(input.attr('disabled')).to.be.undefined; 
        expect(input.attr('id')).to.be.equal('ctrlCODIGOHISTCTB');
        expect(input.attr('name')).to.be.equal('p.CODIGOHISTCTB');
        expect(input.attr('required')).to.be.equal('required');
        expect(input.attr('readonly')).to.be.undefined;
        expect(input.attr('step')).to.be.equal('any');
        expect(input.attr('style')).to.be.undefined;
        expect(input.attr('title')).to.be.equal('Informe o código do Histórico Contábil');
        expect(input.attr('type')).to.be.equal('text');
        expect(input.attr('value')).to.be.undefined;
    })

    it('Deve criar controle com a variação de moeda', () => {
        var element = <JQuery<HTMLElement>>NtcEdit(configCurrencyInput);

        let label = element.find('label');
        let inputWrapper = element.children().eq(1);
        let inputGroup = inputWrapper.find('.input-group-prepend');
        let spanGroupText = inputGroup.find('span.input-group-text');
        let currencyIcon = spanGroupText.find('i.fa.fa-fw.fa-dollar-sign');
        let input = inputWrapper.find('input');

        expect(label.length).to.be.equal(1);
        expect(inputWrapper.length).to.be.equal(1);
        expect(inputGroup.length).to.be.equal(1);
        expect(spanGroupText.length).to.be.equal(1);
        expect(currencyIcon.length).to.be.equal(1);
        expect(input.length).to.equal(1);
    });

    it('Deve conter todos os atributos necessários para a variação moeda', () => {
        var element = <JQuery<HTMLElement>>NtcEdit(configCurrencyInput);

        let inputWrapper = element.children().eq(1);
        let input = inputWrapper.find('input');

        let classList = input.attr('class')?.split(' ');
        let expectClassList = ['form-control', 'kt-input', 'cad-control', 'cad-logchange', 'ctrlVALORLCTOCTB', 'text-right']

        
        expect(inputWrapper.length).to.be.equal(1);
        expect(input.length).to.be.equal(1);
        
        expect(classList).to.be.containingAllOf(expectClassList);

        expect(input.attr('autocomplete')).to.be.equal('off');
        expect(input.attr('data-autoincrementar')).to.be.equal('false');
        expect(input.attr('data-alias')).to.not.be.undefined;
        expect(input.attr('data-classname')).to.be.equal('TnCtbDMLctoCtb');
        expect(input.attr('data-dicionario')).to.be.undefined;
        expect(input.attr('data-mask')).to.be.equal('#.##0,00');
        expect(input.attr('data-mask-reverse')).to.be.equal('true');				
        expect(input.attr('disabled')).to.be.undefined;
        expect(input.attr('id')).to.be.equal('ctrlVALORLCTOCTB');
        expect(input.attr('name')).to.be.equal('p.VALORLCTOCTB');
        expect(input.attr('readonly')).to.be.undefined;
        expect(input.attr('required')).to.be.equal('required');
        expect(input.attr('step')).to.be.equal('any') 
        expect(input.attr('style')).to.be.undefined; 
        expect(input.attr('title')).to.be.equal('Valor do Lançamento Contábil');
        expect(input.attr('type')).to.be.equal('text'); 
        expect(input.attr('value')).to.be.undefined;
    })
});