import { expect } from "chai";
import { JSDOM } from "jsdom";
import { NtcLinkEdit } from "../../../../../ts/components/fra-control/ntc-link-edit/ntc-link-edit";
import { DOMBuilder } from "../../../../Utils/DOMBuilder";
import { descriptionSearchField } from "./field-config/description-search-field";
import { valueSearchInlineFieldWithAdd, valueSearchInlineFieldWithoutAdd, valueSearchField } from "./field-config/value-search-field";

describe('Criação dinâmica do controle ntcLinkEdit', () => {
    let DOM: JSDOM;

    before(async () => {
        DOM = await DOMBuilder.build();
        await DOMBuilder.includeGlobalsJq(DOM);
    });

    after(DOMBuilder.clearGlobals);

    it('Deve criar os elementos do controle do tipo busca por descrição', () => {
        const element = <JQuery<HTMLElement>>NtcLinkEdit(descriptionSearchField);

        let label = element.find('label#labelctrloptCONTACTBDEB');
        let divCol = element.children().eq(1);
        let inputGroup = divCol.find('div.input-group');

        let dxEditor = inputGroup.find('div.form-control[data-lookupalphanumeric="true"]')
        let inputResult = inputGroup.find('input#ctrlCONTACTBDEBRESULT');
        let inputControl = inputGroup.find('input#ctrlCONTACTBDEB');

        let buttonWrapper = inputGroup.find('div.input-group-append');
        let buttonSearch = buttonWrapper.find('button[data-id="ctrlCONTACTBDEB"] i.la.la-search');
        let buttonAdd = buttonWrapper.find('button[data-original-title="Incluir um Novo"] i.la.la-plus');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);
        expect(inputGroup.length).to.equal(1);
        
        expect(dxEditor.length).to.equal(1);
        expect(inputResult.length).to.equal(1);
        expect(inputControl.length).to.equal(1);
        
        expect(buttonWrapper.length).to.equal(1);
        expect(buttonSearch.length).to.equal(1);
        expect(buttonAdd.length).to.equal(1);
    });

    it('Deve criar os elementos do controle do tipo busca por código com MultiColuna', () => {
        const element = <JQuery<HTMLElement>>NtcLinkEdit(valueSearchField);

        let label = element.find('label#labelctrloptCODIGOESTAB');
        let divCol = element.children().eq(1);
        let inputGroup = divCol.find('div.input-group');

        let inputResult = divCol.find('input#ctrlCODIGOESTABRESULT');
        let inputControl = inputGroup.find('input#ctrlCODIGOESTAB');

        let buttonWrapper = inputGroup.find('div.input-group-append');
        let buttonSearch = buttonWrapper.find('button[data-id="ctrlCODIGOESTAB"] i.la.la-search');
        let buttonAdd = buttonWrapper.find('button[data-original-title="Incluir um Novo"] i.la.la-plus');
        let controlFeedBack = divCol.find('div.form-control-feedback');
        let controlFeedBackText = controlFeedBack.find('b#ctrlCODIGOESTABRESULTTEXT');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);
        expect(inputGroup.length).to.equal(1);
        
        expect(inputResult.length).to.equal(1);
        expect(inputControl.length).to.equal(1);
        
        expect(buttonWrapper.length).to.equal(1);
        expect(buttonSearch.length).to.equal(1);
        expect(buttonAdd.length).to.equal(0);

        expect(controlFeedBack.length).to.equal(0);
        expect(controlFeedBackText.length).to.equal(0);
    });
    
    it('Deve criar os elementos do controle do tipo busca por código sem multicoluna e sem botão de adicionar', () => {
        const element = <JQuery<HTMLElement>>NtcLinkEdit(valueSearchInlineFieldWithoutAdd);

        let label = element.find('label#labelctrloptPCODIGOEMPRESA');
        let divCol = element.children().eq(1);
        let inputGroup = divCol.find('div.input-group');

        let inputResult = divCol.find('input#ctrlPCODIGOEMPRESARESULT');
        let inputControl = inputGroup.find('input#ctrlPCODIGOEMPRESA');

        let buttonWrapper = inputGroup.find('div.input-group-append');
        let buttonSearch = buttonWrapper.find('button[data-id="ctrlPCODIGOEMPRESA"] i.la.la-search');
        let buttonAdd = buttonWrapper.find('button[data-original-title="Incluir um Novo"] i.la.la-plus');

        let controlFeedBack = divCol.find('div.form-control-feedback');
        let controlFeedBackText = controlFeedBack.find('b#ctrlPCODIGOEMPRESARESULTTEXT');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);
        expect(inputGroup.length).to.equal(1);
        
        expect(inputResult.length).to.equal(1);
        expect(inputControl.length).to.equal(1);
        
        expect(buttonWrapper.length).to.equal(1);
        expect(buttonSearch.length).to.equal(1);
        expect(buttonAdd.length).to.equal(0);

        expect(controlFeedBack.length).to.equal(1);
        expect(controlFeedBackText.length).to.equal(1);
    });

    it('Deve criar os elementos do controle sem multicoluna e com botão de adicionar', () => {
    
        const element = <JQuery<HTMLElement>>NtcLinkEdit(valueSearchInlineFieldWithAdd);

        let label = element.find('label#labelctrloptCHAVELCTOCTBSOC');
        let divCol = element.children().eq(1);
        let inputGroup = divCol.find('div.input-group');

        let inputResult = divCol.find('input#ctrlCHAVELCTOCTBSOCRESULT');
        let inputControl = inputGroup.find('input#ctrlCHAVELCTOCTBSOC');

        let buttonWrapper = inputGroup.find('div.input-group-append');
        let buttonSearch = buttonWrapper.find('button[data-id="ctrlCHAVELCTOCTBSOC"] i.la.la-search');
        let buttonAdd = buttonWrapper.find('button[data-original-title="Incluir um Novo"] i.la.la-plus');

        let controlFeedBack = divCol.find('div.form-control-feedback');
        let controlFeedBackText = controlFeedBack.find('b#ctrlCHAVELCTOCTBSOCRESULTTEXT');


        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);
        expect(inputGroup.length).to.equal(1);
        
        expect(inputResult.length).to.equal(1);
        expect(inputControl.length).to.equal(1);
        
        expect(buttonWrapper.length).to.equal(1);
        expect(buttonSearch.length).to.equal(1);
        expect(buttonAdd.length).to.equal(1);

        expect(controlFeedBack.length).to.equal(1);
        expect(controlFeedBackText.length).to.equal(1);
    });
})