import { expect } from "chai";
import { JSDOM } from "jsdom";
import { NtcLink } from "../../../../../ts/components/fra-control/ntc-link/ntc-link";
import { DOMBuilder } from "../../../../Utils/DOMBuilder";
import { numberField } from "./field-config/number-field";
import { textField } from "./field-config/text-field";
import { unmaskedNumberField } from './field-config/unmasked-number-field';

describe('Criação dinâmica do controle ntcLink', () => {
    let DOM: JSDOM;

    before(async () => {
        DOM = await DOMBuilder.build();
        await DOMBuilder.includeGlobalsJq(DOM);
    });

    after(DOMBuilder.clearGlobals);

    it('Deve criar os elementos do controle do tipo numérico com máscara', () => {
        const element = <JQuery<HTMLElement>>NtcLink(numberField);

        let label = element.find('label#labelctrloptCODIGOHISTCTB');
        let divCol = element.children().eq(1);
        let inputGroup = divCol.find('div.input-group');
        let input = inputGroup.find('input#ctrlCODIGOHISTCTB');
        let buttonWrapper = inputGroup.find('div.input-group-append');
        let buttonSearch = buttonWrapper.find('button[data-id="ctrlCODIGOHISTCTB"] i.la.la-search');
        let buttonAdd = buttonWrapper.find('button[data-id="ctrlnewCODIGOHISTCTB"] i.la.la-plus');

        expect(label.length).to.equal(1);
        expect(divCol.length).to.equal(1);
        expect(inputGroup.length).to.equal(1);
        expect(input.length).to.equal(1);
        expect(buttonWrapper.length).to.equal(1);
        expect(buttonSearch.length).to.equal(1);
        expect(buttonAdd.length).to.equal(1);
    });

    it('Deve criar os elementos do controle do tipo texto', () => {
        const element = <JQuery<HTMLElement>>NtcLink(textField);

        let label = element.find('label#labelctrloptPCLASSIFCONTA');
        let inputGroup = element.children().eq(1);
        let input = inputGroup.find('input#ctrlPCLASSIFCONTA');
        let buttonWrapper = inputGroup.find('div.input-group-append');
        let buttonSearch = buttonWrapper.find('button[data-id="ctrlPCLASSIFCONTA"] i.la.la-search');
        let buttonAdd = buttonWrapper.find('button[data-id="ctrlnewPCLASSIFCONTA"] i.la.la-plus');

        expect(label.length).to.equal(1);
        expect(inputGroup.length).to.equal(1);
        expect(input.length).to.equal(1);
        expect(buttonWrapper.length).to.equal(1);
        expect(buttonSearch.length).to.equal(1);
        expect(buttonAdd.length).to.equal(0);
    });

    it('Deve criar os elementos do tipo numérico sem máscara', () => {
        const element = <JQuery<HTMLElement>>NtcLink(unmaskedNumberField);

        let label = element.find('label#labelctrloptNUMEROCONTA1');
        let inputGroup = element.children().eq(1);
        let input = inputGroup.find('input#ctrlNUMEROCONTA1');
        let buttonWrapper = inputGroup.find('div.input-group-append');
        let buttonSearch = buttonWrapper.find('button[data-id="ctrlNUMEROCONTA1"] i.la.la-search');
        let buttonAdd = buttonWrapper.find('button[data-id="ctrlnewNUMEROCONTA1"] i.la.la-plus');

        expect(label.length).to.equal(1);
        expect(inputGroup.length).to.equal(1);
        expect(input.length).to.equal(1);
        expect(buttonWrapper.length).to.equal(1);
        expect(buttonSearch.length).to.equal(1);
        expect(buttonAdd.length).to.equal(0);
    });
});