import path from 'path';
import { expect } from 'chai';
import { DOMBuilder } from '../../../Utils/DOMBuilder';
import { FraControlPropertiesExtractor } from '../../../../js/form/PropertiesExtractor/FraControlPropertiesExtractor';
import { PropertiesExtractor } from '../../../../js/form/PropertiesExtractor/PropertiesExtractor';


describe('Tratamento de propriedades baseado no tipo de campo. Classe FraControlPropertiesExtractor', () => {
	let DOM;

	before(async () => {
		DOM = await DOMBuilder.build(path.resolve(__dirname, 'FraControlPropertiesExtractor.html'));
		await DOMBuilder.includeGlobalsJq(DOM);
	});

	after(DOMBuilder.clearGlobals);

	it('Deve transformar a string "asd.dsa" em "asd\\.dsa"', () => {
		const str = 'asd.dsa';
		const expectedStr = 'asd\\.dsa';

		expect(FraControlPropertiesExtractor._fixDotInName(str)).to.be.equal(expectedStr);
	});

	it('Deve retornar as propriedades necessárias para a criação de um campo comum. Método: _ntcDefaultProperties', () => {
		const form = {
			element: $('form.formTnCtbDMHistoricoCtb')
		}
		const expectedProps = {
			alias: '',
			classname: 'TnCtbDMHistoricoCtb',
			expressoes: null,
			index: 1,
			isGroupColumn: null,
			isImage: false,
			isMultiColumn: false,
			name: 'CODIGOHISTCTB',
			ntcSubtype: '',
			ntcType: 'ntcEdit',
			nvValor: '',
			visible: true,
		}

		const expectedDefaultProps = [
			{
				componente: $('#ctrlCODIGOHISTCTB', form.element),
				componenteimage: null,
				expressoes: null,
				index: 1,
				isGroupColumn: null,
				isMultiColumn: false,
				label: $('#labelctrloptCODIGOHISTCTB', form.element),
				linkbutton: $('#btnCODIGOHISTCTB', form.element),
				nome: 'CODIGOHISTCTB',
				resultcomponente: $('#ctrlCODIGOHISTCTBRESULT', form.element),
				resultcomponentetext: $('#ctrlCODIGOHISTCTBRESULTTEXT', form.element),
				visible: true,
				controlType: 'ntcEdit',
			}
		];

		const props = PropertiesExtractor.extractOptionsField($('#FraControlCreateCODIGOHISTCTB'));
		const defaultProps = FraControlPropertiesExtractor._ntcDefaultProperties({ ...props, form });

		expect(props).to.be.eql(expectedProps);
		expect(defaultProps).to.be.eql(expectedDefaultProps);
	});

	it('Deve retornar as propriedades necessárias para a criação de um campo do tipo checkListBox. Método: _ntcCheckListBoxProperties', async () => {
		const form = {
			element: $('form.formnFisRRProduto')
		};

		const expectedOptionsOpt1 = {
			name: 'PENTRADACON',
			classname: 'nFisRRProduto',
			alias: '',
			isImage: false,
			visible: true,
			index: 1,
			isMultiColumn: true,
			isGroupColumn: null,
			expressoes: null,
			ntcType: 'ntcCheckListBox',
			ntcSubtype: '',
			nvValor: 1,
		};

		const expectedOptionsOpt2 = {
			name: 'PENTRADACON',
			classname: 'nFisRRProduto',
			alias: '',
			isImage: false,
			visible: true,
			index: 2,
			isMultiColumn: true,
			isGroupColumn: null,
			expressoes: null,
			ntcType: 'ntcCheckListBox',
			ntcSubtype: '',
			nvValor: 0,
		};

		const expectedPropsOption1 = [{
			componente: $('#ctrlPENTRADACON_checkbox_1_ctrlPENTRADACON', form.element),
			componenteimage: null,
			expressoes: null,
			index: 1,
			isGroupColumn: null,
			isMultiColumn: true,
			label: $('#labelctrloptPENTRADACON_checkbox_1', form.element),
			linkbutton: $('#btnPENTRADACON_checkbox_1', form.element),
			nome: "PENTRADACON_checkbox_1",
			resultcomponente: $('#ctrlPENTRADACON_checkbox_1RESULT', form.element),
			resultcomponentetext: $('#ctrlPENTRADACON_checkbox_1RESULTTEXT', form.element),
			visible: true,
			controlType: 'ntcCheckListBox',
		}];

		const expectedPropsOption2 = [{
			componente: $('#ctrlPENTRADACON_checkbox_0_ctrlPENTRADACON', form.element),
			componenteimage: null,
			expressoes: null,
			index: 2,
			isGroupColumn: null,
			isMultiColumn: true,
			label: $('#labelctrloptPENTRADACON_checkbox_0', form.element),
			linkbutton: $('#btnPENTRADACON_checkbox_0', form.element),
			nome: "PENTRADACON_checkbox_0",
			resultcomponente: $('#ctrlPENTRADACON_checkbox_0RESULT', form.element),
			resultcomponentetext: $('#ctrlPENTRADACON_checkbox_0RESULTTEXT', form.element),
			visible: true,
			controlType: 'ntcCheckListBox',
		}];

		const optionsOption1 = PropertiesExtractor.extractOptionsField($('[id=FraControlCreatePENTRADACON]').eq(0));
		const propsOption1 = FraControlPropertiesExtractor._ntcCheckListBoxProperties({ ...optionsOption1, form });

		const optionsOption2 = PropertiesExtractor.extractOptionsField($('[id=FraControlCreatePENTRADACON]').eq(1));
		const propsOption2 = FraControlPropertiesExtractor._ntcCheckListBoxProperties({ ...optionsOption2, form });
		
		expect(optionsOption1).to.be.eql(expectedOptionsOpt1);
		expect(propsOption1).to.be.eql(expectedPropsOption1);

		expect(optionsOption2).to.be.eql(expectedOptionsOpt2);
		expect(propsOption2).to.be.eql(expectedPropsOption2);
	});

	it('Deve retornar as propriedades necessárias para a criação de um campo com lookup padrão. Método: _ntcLinkEditProperties', () => {
		const form = {
			element: $('form.formTnFisDPConsultaSaldoFornecedor')
		}
		const expectedOption = {
			name: 'PCODIGOESTAB',
			classname: 'TnFisDPConsultaSaldoFornecedor',
			alias: '',
			isImage: false,
			visible: true,
			index: 5,
			isMultiColumn: true,
			isGroupColumn: null,
			expressoes: null,
			ntcType: 'ntcLinkEdit',
			ntcSubtype: '',
			nvValor: '',
		};

		const expectedProps = [
			{
				componente: $('#ctrlPCODIGOESTAB', form.element),
				componenteimage: null,
				expressoes: null,
				index: 5,
				isGroupColumn: null,
				isMultiColumn: true,
				label: $('#labelctrloptPCODIGOESTAB', form.element),
				linkbutton: $('#btnPCODIGOESTAB', form.element),
				nome: 'PCODIGOESTAB',
				resultcomponente: $('#ctrlPCODIGOESTABRESULT', form.element),
				resultcomponentetext: $('#ctrlPCODIGOESTABRESULTTEXT', form.element),
				visible: true,
				controlType: "ntcLinkEdit",
			},
			{
				componente: $('#ctrlPCODIGOESTABRESULT', form.element),
				isMultiColumn: true,
				nome: 'PCODIGOESTABRESULT'
			},
			{
				componente: $('#ctrlPCODIGOESTABRESULTTEXT', form.element),
				isMultiColumn: true,
				nome: 'PCODIGOESTABRESULTTEXT'
			}
		];

		const option = PropertiesExtractor.extractOptionsField($('#FraControlCreatePCODIGOESTAB'));
		const props = FraControlPropertiesExtractor._ntcLinkEditProperties({ ...option, form });

		expect(option).to.be.eql(expectedOption);
		expect(props).to.be.eql(expectedProps);
	});
});