import { expect, use } from 'chai';
import { DOMBuilder } from '../../Utils/DOMBuilder';
import { PropertiesExtractor } from '../../../js/form/PropertiesExtractor/PropertiesExtractor';

use(require('chai-as-promised'));

describe('PropertiesExtractor form', () => {
	beforeEach(async () => await DOMBuilder.includeGlobalsJq(await DOMBuilder.build()) );
	afterEach(DOMBuilder.clearGlobals);

	it(`Deve retornar as propriedades do form`, () => {
		const div = $(`
			<div 
				id="TnCtbDMLctoCtb"
				data-script-form-create="true"
				data-classe="TnCtbDMLctoCtb"
				data-alias=""
				data-formdetalhe="null"
				data-chaveprimaria="[&quot;CODIGOEMPRESA&quot;,&quot;CHAVELCTOCTB&quot;]"
				data-masterclass="null"
				data-masterfields="null"
				data-detailfields="null"
				data-filtrofixo="null"
				data-multicolumn="true"
				data-caption="Lançamentos Contábeis"
				data-configextra="{&quot;DireitoAlteraDadoFixoLancamentoProg&quot;:false,&quot;UsaLote&quot;:false,&quot;UsaNumeroDcto&quot;:false,&quot;UsaOrigemLctoCtb&quot;:true,&quot;FilialSelecionada&quot;:true,&quot;UsaOrigemLctoCtb&quot;:true,&quot;UsaNumeroDcto&quot;:false,&quot;UsaLote&quot;:false,&quot;DataInicialSelecao&quot;:42736,&quot;DataFinalSelecao&quot;:42745,&quot;PertenceLote&quot;:false,&quot;DataInicialLote&quot;:0,&quot;DataFinalLote&quot;:0,&quot;EhOptanteIFRS&quot;:false,&quot;UsaAtividades&quot;:false,&quot;UsaCentroCusto&quot;:true}"
				data-idrequest="0"
				data-tipoobjeto="ntoCadastro">
			</div>
	 `);

		const expectedProp = {
			classe: 'TnCtbDMLctoCtb',
			alias: '',
			formdetalhe: null,
			chavePrimaria: ['CODIGOEMPRESA', 'CHAVELCTOCTB'],
			masterclass: null,
			masterfields: null,
			detailfields: null,
			filtrofixo: null,
			multiColumn: true,
			caption: 'Lançamentos Contábeis',
			configextra: {
				DireitoAlteraDadoFixoLancamentoProg: false,
				UsaLote: false,
				UsaNumeroDcto: false,
				UsaOrigemLctoCtb: true,
				FilialSelecionada: true,
				DataInicialSelecao: 42736,
				DataFinalSelecao: 42745,
				PertenceLote: false,
				DataInicialLote: 0,
				DataFinalLote: 0,
				EhOptanteIFRS: false,
				UsaAtividades: false,
				UsaCentroCusto: true
			},
			idrequest: 0,
			tipoObjeto: 'ntoCadastro'
		};

		$('body').append(div);

		const props = PropertiesExtractor.extractOptionsForm(div);

		expect(props).to.be.eql(expectedProp);
	});
});

describe('PropertiesExtractor fields', () => {
	beforeEach(async () => {
		await DOMBuilder.includeGlobalsJq(await DOMBuilder.build());
	})


	it(`Deve retornar as propriedades de um field com expressões`, () => {
		const div = $(`
			<div 
				id="FraControlCreatePCODIGOCFOP" 
				data-fracontrol-script-create="true"
				data-name="PCODIGOCFOP"
				data-classname="TNARQDPIMPORTARARQNFE"
				data-alias=""
				data-is-image="false"
				data-visible="true"
				data-index="29"
				data-is-multicolumn="true"
				data-is-groupcolumn="null"
				data-expressoes="{&quot;ExpressaoEnabled&quot;:&quot;IGUAL(PVALIDAESTAB, 1)&quot;}"
				data-ntc-type="ntcLink"
				data-ntc-subtype=""
				data-nv-valor="">
			</div>
		`);

		const expectedProp = {
			name: 'PCODIGOCFOP',
			classname: 'TNARQDPIMPORTARARQNFE',
			alias: '',
			isImage: false,
			visible: true,
			index: 29,
			isMultiColumn: true,
			isGroupColumn: null,
			expressoes: {
				ExpressaoEnabled: 'IGUAL(PVALIDAESTAB, 1)'
			},
			ntcType: 'ntcLink',
			ntcSubtype: '',
			nvValor: ''
		}

		$('body').append(div);

		const props = PropertiesExtractor.extractOptionsField(div);

		expect(props).to.be.eql(expectedProp);
	});

	it(`Deve retornar as propriedades de um field do tipo checklistbox, com expressões, e é grupo coluna`, () => {
		const div = $(`
			<div 
				id="FraControlCreatePOBJETOJOBESTADUAL" 
				data-fracontrol-script-create="true"
				data-name="POBJETOJOBESTADUAL"
				data-classname="TNDPCADASTROFLUXOJOB"
				data-alias=""
				data-is-image="false"
				data-visible="true"
				data-index="1"
				data-is-multicolumn="true"
				data-is-groupcolumn="true"
				data-expressoes="{&quot;ExpressaoVisible&quot;:&quot;IGUAL (PCODIGOJOB, 1)&quot;,&quot;ExpressaoRequired&quot;:&quot;IGUAL (PCODIGOJOB, 1)&quot;}"
				data-ntc-type="ntcCheckListBox"
				data-ntc-subtype=""
				data-nv-valor="0">
			</div>
		`);

		const expectedProp = {
			name: 'POBJETOJOBESTADUAL',
			classname: 'TNDPCADASTROFLUXOJOB',
			alias: '',
			isImage: false,
			visible: true,
			index: 1,
			isMultiColumn: true,
			isGroupColumn: true,
			expressoes: {
				ExpressaoVisible: 'IGUAL (PCODIGOJOB, 1)',
				ExpressaoRequired: 'IGUAL (PCODIGOJOB, 1)'
			},
			ntcType: 'ntcCheckListBox',
			ntcSubtype: '',
			nvValor: 0
		}

		$('body').append(div);

		const props = PropertiesExtractor.extractOptionsField(div);

		expect(props).to.be.eql(expectedProp);
	});

	it(`Deve retornar as propriedades de um field do tipo data`, () => {
		const div = $(`
			<div 
				id="FraControlCreateDATAFINAL" 
				data-fracontrol-script-create="true"
				data-name="DATAFINAL"
				data-classname="TNDMJOBAGENDA"
				data-alias=""
				data-is-image="false"
				data-visible="true"
				data-index="2"
				data-is-multicolumn="true"
				data-is-groupcolumn="true"
				data-expressoes="null"
				data-ntc-type="ntcActData"
				data-ntc-subtype=""
				data-nv-valor="">
			</div>
		`);

		const expectedProp = {
			name: 'DATAFINAL',
			classname: 'TNDMJOBAGENDA',
			alias: '',
			isImage: false,
			visible: true,
			index: 2,
			isMultiColumn: true,
			isGroupColumn: true,
			expressoes: null,
			ntcType: 'ntcActData',
			ntcSubtype: '',
			nvValor: ''
		}

		$('body').append(div);

		const props = PropertiesExtractor.extractOptionsField(div);

		expect(props).to.be.eql(expectedProp);
	});
});

describe(`PropertiesExtractor action`, () => {
	beforeEach(async () => await DOMBuilder.includeGlobalsJq(await DOMBuilder.build()));
	afterEach(DOMBuilder.clearGlobals);

	it('Deve retornar as propriedades para a action actDatasetPostExecute do botão salvar e voltar', () => {
		const div = $(`
			<a 
				id="btnpost3" 
				class="dropdown-item" 
				data-posttype="3" 
				href="javascript:;" 
				data-n-onclick="actDataSetPostExecute"  
				data-create-naction="true" 
				data-class="TnCtbDMHistoricoCtb" 
				data-alias="" 
				data-cadcons="TnCtbDCHistoricoCtb" 
				data-topicoajuda="nCtbDMHistoricoCtb">
				<i class="la la-undo"> </i>
				<span class=""> Salvar e Voltar </span>
			</a>
		`);

		const expectedProp = {
			class: 'TnCtbDMHistoricoCtb',
			alias: '',
			actions: [
				{
					cadCons: 'TnCtbDCHistoricoCtb',
					topicoAjuda: 'nCtbDMHistoricoCtb',
					idElement: 'btnpost3',
					name: 'actDataSetPostExecute',
					type: 'click',
					config: {
						nOnclick: 'actDataSetPostExecute',  
						createNaction: true,
						class: 'TnCtbDMHistoricoCtb', 
						alias: '', 
						cadcons: 'TnCtbDCHistoricoCtb', 
						topicoajuda: 'nCtbDMHistoricoCtb',
						posttype: 3
					}
				}
			]
		}

		$('body').append(div);

		const props = PropertiesExtractor.extractOptionsAction(div);

		expect(props).to.be.eql(expectedProp);
	})

	it('Deve retornar as propriedades para a action actDatasetPostExecute do botão ações', () => {
		const div = $(`
			<a 
				class="kt-nav__link disabled toolbutton" 
				title="Salvar" 
				id="tbtnactDataSetPostExecuteTnCtbDMHistoricoCtb"
				data-disabled="true" 
				data-n-onclick="actDataSetPostExecute"  
				data-create-naction="true" 
				data-class="TnCtbDMHistoricoCtb" 
				data-alias="" 
				data-cadcons="TnCtbDCHistoricoCtb" 
				data-topicoajuda="nCtbDMHistoricoCtb">
				<i class="kt-nav__link-icon la la-actDataSetPost"></i>
				<span class="kt-nav__link-text">Salvar</span>
			</a>
		`);

		const expectedProp = {
			class: 'TnCtbDMHistoricoCtb',
			alias: '',
			actions: [
				{
					cadCons: 'TnCtbDCHistoricoCtb',
					topicoAjuda: 'nCtbDMHistoricoCtb',
					idElement: 'tbtnactDataSetPostExecuteTnCtbDMHistoricoCtb',
					name: 'actDataSetPostExecute',
					type: 'click',
					config: {
						disabled: true, 
						nOnclick: 'actDataSetPostExecute',  
						createNaction: true,
						class: 'TnCtbDMHistoricoCtb', 
						alias: '', 
						cadcons: 'TnCtbDCHistoricoCtb', 
						topicoajuda: 'nCtbDMHistoricoCtb'
					}
				}
			]
		}

		$('body').append(div);

		const props = PropertiesExtractor.extractOptionsAction(div);

		expect(props).to.be.eql(expectedProp);
	});


	it('Deve retornar as propriedades para a action actOpenWorkSpace do twMenu', () => {
		const div = $(`
			<button 
				type="button"
				class="twMenu nav-link btn btn-sm btn-secondary-q kt-btn kt-btn--icon border-radius"
				data-create-naction="true"
				data-n-onclick="actOpenWorkSpace"
				data-class="TnFisDPAreaTrabalhoSimplesNacional"
				data-redirect-class="TNFISDPALIQSSIMPLESFEDERAL">
        <span class="d-none d-xs-inline"> Ações </span>
        <span> Simulador </span>
    	</button>
		`);

		const expectedProp = {
			class: 'TnFisDPAreaTrabalhoSimplesNacional',
			alias: undefined,
			actions: [
				{
					cadCons: undefined,
					topicoAjuda: undefined,
					idElement: undefined,
					name: 'actOpenWorkSpace',
					type: 'click',
					config: {
						nOnclick: 'actOpenWorkSpace',  
						createNaction: true,
						class: 'TnFisDPAreaTrabalhoSimplesNacional', 
						redirectClass: 'TNFISDPALIQSSIMPLESFEDERAL',
					}
				}
			]
		}

		$('body').append(div);

		const props = PropertiesExtractor.extractOptionsAction(div);

		expect(props).to.be.eql(expectedProp);
	});
});