import path from 'path';
import FormManager from '../../../../js/FormManager';
import dxAutocomplete from 'devextreme/ui/autocomplete';
import { expect } from 'chai';
import { DOMBuilder } from '../../../Utils/DOMBuilder';
import { lookupAlphaNumeric } from '../../../../js/lookup';
import Initializer from '../../../../js/initializer';
import { registerPrototypes } from '../../../../js/registerPrototypes';

describe('Lookup com autocomplete', function() {
	before(async () => {
		let DOM = await DOMBuilder.build(path.resolve(__dirname, 'lookupAlphaNumeric.html'));
		await DOMBuilder.includeGlobalsJq(DOM);
		registerPrototypes();
	});

	after(DOMBuilder.clearGlobals);

	it ('Deve extrair os atributos como experado pelo método _extractOptions', () => {
		const expectedOptions = {
			alias: '',
			displayExpr: 'CODIGOPESSOA;NOMEPESSOA;INSCRFEDERAL',			
			searchExpr: 'INSCRFEDERAL;NOMEPESSOA',
			searchClassName: 'nGemDCPessoa',
			dmClassName: 'TnFisDPConsultaSaldoFornecedor',
			linkkeyfields: 'CODIGOPESSOA',
			fieldName: 'PCODIGOPESSOA',
			isMultiColumn: true,
			linklookupresultfield: 'NOMEPESSOA',
			linklookupkeyfields: 'CODIGOPESSOA',
			linklookupFieldOrdination: undefined,
			initialValue: ''
		}

		let options = lookupAlphaNumeric.prototype._extractOptions($('[data-lookupid="ctrlPCODIGOPESSOA"]'));

		expect(options).to.be.eql(expectedOptions);
	});

	it ('Deve o método _buildDataSource retornar, um objeto com uma função de load', () => {
		const options = lookupAlphaNumeric.prototype._extractOptions($('[data-lookupid="ctrlPCODIGOPESSOA"]'));

		let dataSource = lookupAlphaNumeric.prototype._buildDataSource(options);

		expect(dataSource).to.have.ownProperty('load');
		expect(typeof dataSource.load).to.be.equal('function');
	});

	it ('Deve retornar os atributos a serem inclusos no input do componente. Método _generateInputAttr', () => {
		const options = lookupAlphaNumeric.prototype._extractOptions($('[data-lookupid="ctrlPCODIGOPESSOA"]'));
		const expectedInputAttr = {
			style: 'padding: 0; font-family: poppins; font-size: 1rem;',
			class: 'cad-control cad-logchange',
			'data-classname': 'TnFisDPConsultaSaldoFornecedor',
			'data-alias': ''
		};

		let inputAttr = lookupAlphaNumeric.prototype._generateInputAttr(options);

		expect(inputAttr).to.be.eql(expectedInputAttr);
	});
	
	it ('Deve retornar uma série de handlers para o componente. Método _handlersConfig', () => {
		const options = lookupAlphaNumeric.prototype._extractOptions($('[data-lookupid="ctrlPCODIGOPESSOA"]'));

		let handlersConfig = lookupAlphaNumeric.prototype._handlersConfig(options);
		let { 
			onInput, 
			itemTemplate, 
			valueExpr, 
			onItemClick,
			onFocusOut,
			onFocusIn
		} = handlersConfig;

		expect(handlersConfig).to.have.ownProperty('onInput');
		expect(handlersConfig).to.have.ownProperty('itemTemplate');
		expect(handlersConfig).to.have.ownProperty('valueExpr');
		expect(handlersConfig).to.have.ownProperty('onItemClick');
		expect(handlersConfig).to.have.ownProperty('onFocusOut');
		expect(handlersConfig).to.have.ownProperty('onFocusIn');


		expect(typeof onInput).to.be.equal('function');
		expect(typeof itemTemplate).to.be.equal('function');
		expect(typeof valueExpr).to.be.equal('function');
		expect(typeof onItemClick).to.be.equal('function');
		expect(typeof onFocusOut).to.be.equal('function');
		expect(typeof onFocusIn).to.be.equal('function');
	});

	it ('Deve retornar uma instância da classe dxAutocomplete. Método _buildAutocomplete', () => {
		const options = lookupAlphaNumeric.prototype._extractOptions($('[data-lookupid="ctrlPCODIGOPESSOA"]'));
		const config = {
			dataSource: lookupAlphaNumeric.prototype._buildDataSource(options),
			inputAttr: lookupAlphaNumeric.prototype._generateInputAttr(options),
			searchExpr: options.searchExpr.split(';'),
			...lookupAlphaNumeric.prototype._handlersConfig(options)
		};

		let instance = lookupAlphaNumeric.prototype._buildAutocomplete($('[data-lookupid="ctrlPCODIGOPESSOA"]'), config);

		expect(instance).to.be.an.instanceOf(dxAutocomplete);
	});

	it('Deve criar uma instância da classe lookupAlphaNumeric.', () => {
		let form = FormManager.createForm($('[data-script-form-create=true]'));

		let param = { 
			element: $(`[data-lookupid=ctrlPCODIGOPESSOA]`),
			form: form
		};

		let lookup = new lookupAlphaNumeric(param);

		
		expect(lookup).to.be.an.instanceOf(lookupAlphaNumeric);
	});
});