import { expect, assert } from 'chai';
import { nExpressionExecutor } from "../../../js/nExpressionExecutor";
import moment from 'moment';

describe("nExpressionExecutor Tests:", function () {
	it('Teste COPY -> Deve retornar 5', async function () {
		var teste = new nExpressionExecutor();
		teste.registrarValor('CODIGOCFOP', '5101001');			
		var result = await teste.execute('COPY(CODIGOCFOP, 1, 1)');
    expect(result).to.equal('5');
	})
	it('Teste COPY -> Deve retornar funcao', async function () {
		var teste = new nExpressionExecutor();
		teste.registrarValor('DESCRICAO', 'Teste funcao copy');			
		var result = await teste.execute('COPY(DESCRICAO, 7, 6)');
    expect(result).to.equal('funcao');
	})
	it('Teste CONTAINS -> Deve retornar true', async function () {
		var teste = new nExpressionExecutor();		
		var result = await teste.execute('CONTAINS("1;2;3;4", "2")');
    expect(result).to.equal(true);
  })
	it('Teste CONTAINS -> Deve retornar true', async function () {
		var teste = new nExpressionExecutor();		
		var result = await teste.execute('CONTAINS("TESTE DA QUESTOR", "DA")');
    expect(result).to.equal(true);
  })
	it('Teste CONTAINS -> Deve retornar true', async function () {
		var teste = new nExpressionExecutor();
		teste.registrarValor('PARAM', 1);	
		teste.registrarValor('PARAM1', 'TESTE');		
		var result = await teste.execute('CONTAINS(PARAM1, IF(IGUAL(PARAM,1), "TEST", "QUESTOR"))');
    expect(result).to.equal(true);
  })
	it('Teste IF -> Deve retornar 50', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', 50);
    var result = await teste.execute('IF(true, TESTE, 25)');
    expect(result).to.equal(50);
  })
  it('Teste IF -> Deve retornar 25', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', 50);
    var result = await teste.execute('IF(false, TESTE, 25)');
    expect(result).to.equal(25);
  })
  it('Teste SOMAR -> Deve retornar 25.23', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', 25);
    teste.registrarValor('TESTE2', 0.23);
    var result = await teste.execute('SOMAR(TESTE, TESTE2)');
    expect(result).to.equal(25.23);
  })
  it('Teste SOMAR -> Array', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', 25);
    teste.registrarValor('TESTE2', 0.23);
    teste.registrarValor('TESTE3', 1);
    var result = await teste.execute('SOMAR(TESTE, [TESTE2, TESTE3])');
    expect(result).to.equal(26.23);
  })
  it('Teste SUBTRAIR', async function () {
    var result = await new nExpressionExecutor().execute('SUBTRAIR(50, 25.12)');
    expect(result).to.equal(24.88);
  })  
  it('Teste retorno zero', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', 1);
    var result = await teste.execute('IF(IGUAL(TESTE, 1), 0, 1)');
    expect(result).to.equal(0);
  })  
  it('Teste SUBTRAIR -> Array', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', 25);
    teste.registrarValor('TESTE2', 1);
    teste.registrarValor('TESTE3', 1);
    var result = await teste.execute('SUBTRAIR(TESTE, [TESTE2, TESTE3])');
    expect(result).to.equal(23);
  })
  it('Teste MULTIPLICAR', async function () {
    var result = await new nExpressionExecutor().execute('MULTIPLICAR(50, 2)');
    expect(result).to.equal(100);
  })
  it('Teste DIVIDIR', async function () {
    var result = await new nExpressionExecutor().execute('DIVIDIR(50, 2)');
    expect(result).to.equal(25);
  })
  it('Teste ENTRE numero True', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', 50);
    var result = await teste.execute('ENTRE(TESTE, 50, 75)');
    expect(result).to.equal(true);    
  })
  it('Teste ENTRE numero false', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', 20);
    var result = await teste.execute('ENTRE(TESTE, 5, 10)');
    expect(result).to.equal(false);    
  })
  it('Teste ENTRE data True', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', '2010-01-01');
    var result = await teste.execute('ENTRE(TESTE, "2010-01-01", "2010-01-31")');
    expect(result).to.equal(true);    
  })
  it('Teste ENTRE data False', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', '2010-01-01');
    var result = await teste.execute('ENTRE(TESTE, "2010-01-02", "2010-01-31")');
    expect(result).to.equal(false);    
  })
  it('Teste ENTRE data invertida true', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', '2010-01-01');
    var result = await teste.execute('ENTRE(TESTE, "01/01/2010", "31/01/2010")');
    expect(result).to.equal(true);
  })
  it('Teste ENTRE data invertida false', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('TESTE', '2010-01-01');
    var result = await teste.execute('ENTRE(TESTE, "02/01/2010", "31/01/2011")');
    expect(result).to.equal(false);
  })
  it('Teste MAIORQUE NUMERO false', async function () {
    var result = await new nExpressionExecutor().execute('MAIORQUE(0, 1)');
    expect(result).to.equal(false);
  })
  it('Teste MAIORQUE NUMERO True', async function () {
    var result = await new nExpressionExecutor().execute('MAIORQUE(1, 0)');
    expect(result).to.equal(true);
  })
  it('Teste MAIORQUE Data True', async function () {
    var result = await new nExpressionExecutor().execute('MAIORQUE("2010-01-01", "2009-12-31")');
    expect(result).to.equal(true);
  })
  it('Teste MAIORQUE Data Invertida True', async function () {
    var result = await new nExpressionExecutor().execute('MAIORQUE("01/01/2010", "31/12/2009")');
    expect(result).to.equal(true);
  })
  it('Teste MAIORQUE Data false', async function () {
    var result = await new nExpressionExecutor().execute('MAIORQUE("2010-01-01", "2012-12-31")');
    expect(result).to.equal(false);
  })
  it('Teste MAIORQUE Data Invertida false', async function () {
    var result = await new nExpressionExecutor().execute('MAIORQUE("01/01/2010", "31/12/2012")');
    expect(result).to.equal(false);
  })
  it('Teste MENORQUE Data', async function () {
    var result = await new nExpressionExecutor().execute('MENORQUE("2010-01-01", "2012-12-31")');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('MENORQUE("2012-01-01", "2010-01-01")');
    expect(result).to.equal(false);
  })
  it('Teste MENORQUE Valor', async function () {
    var result = await new nExpressionExecutor().execute('MENORQUE(25, 50)');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('MENORQUE(50, 50)');
    expect(result).to.equal(false);
  })
  it('Teste MENORQUE Data Invertida', async function () {
    var result = await new nExpressionExecutor().execute('MENORQUE("01/01/2010", "31/12/2012")');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('MENORQUE("01/01/2012", "31/12/2010")');
    expect(result).to.equal(false);
  })
  it('Teste MENORIGUAL Data', async function () {
    var result = await new nExpressionExecutor().execute('MENORIGUAL("01/01/2010", "01/01/2010")');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('MENORIGUAL("01/01/2012", "31/12/2010")');
    expect(result).to.equal(false);
  })
  it('Teste MENORIGUAL valor', async function () {
    var result = await new nExpressionExecutor().execute('MENORIGUAL(50.23, 50.23)');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('MENORIGUAL(50, 25)');
    expect(result).to.equal(false);
  })
  it('Teste AND', async function () {
    var result = await new nExpressionExecutor().execute('AND([true, true])');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('AND([true, false])');
    expect(result).to.equal(false);
  })
  it('Teste OR', async function () {
    var result = await new nExpressionExecutor().execute('OR([true, false])');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('OR([false, false])');
    expect(result).to.equal(false);
  })
  it('Teste DIFERENTE', async function () {
    var result = await new nExpressionExecutor().execute('DIFERENTE(50, 49.99)');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('DIFERENTE(50, 50)');
    expect(result).to.equal(false);
  })
  it('Teste IGUAL', async function () {
    var result = await new nExpressionExecutor().execute('IGUAL(50, 50.00)');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('IGUAL(49.99, 50)');
    expect(result).to.equal(false);
  })
  it('Teste IN', async function () {
    var result = await new nExpressionExecutor().execute('IN(50, [50.00, 49, 48])');
    expect(result).to.equal(true);
    var result = await new nExpressionExecutor().execute('IN(49.99, [50,61])');
    expect(result).to.equal(false);
  })
  it('Teste IN tipos diferentes', async function () {
    var teste = new nExpressionExecutor();
	teste.registrarValor('PARENTESCO', "50");	
	var result = await teste.execute('IN(PARENTESCO, ["50", "49", "48"])');
    expect(result).to.equal(true);		
  })  
  it('Teste IN tipos diferentes 2', async function () {
    var teste = new nExpressionExecutor();
	teste.registrarValor('PARENTESCO', 50);	
	var result = await teste.execute('IN(PARENTESCO, ["50", "49", "48"])');
    expect(result).to.equal(true);		
  })
  it('Teste IN tipos diferentes 3', async function () {
    var teste = new nExpressionExecutor();
	teste.registrarValor('PARENTESCO', "11");	
	var result = await teste.execute('IN( PARENTESCO, ["01", "11", "13", "15", "17", "18"])');
    expect(result).to.equal(true);		
  })  
   it('Teste IN tipos diferentes 4', async function () {
    var teste = new nExpressionExecutor();
	teste.registrarValor('PARENTESCO', "11");	
	var result = await teste.execute('IN( PARENTESCO, [11, 13, 15, 17, 18])');
    expect(result).to.equal(true);		
  })
  it('Teste IN lista complexa', async function () {
    var teste = new nExpressionExecutor();
	teste.registrarValor('PARENTESCO', "true");	
	teste.registrarValor('VALORTRUE', "true");
	var result = await teste.execute('IN(PARENTESCO, [IF(true, VALORTRUE, false), false])');
    expect(result).to.equal(true);		
  })  
  it('Teste CASE', async function () {
    var result = await new nExpressionExecutor().execute('CASE("A", ["B", "A", "C"], [1,3,2])');
    expect(result).to.equal(3);	
    var result = await new nExpressionExecutor().execute('CASE(2, [1,3,2], ["B", "A", "C"])');	
	expect(result).to.equal('C');	
  })  
  it('Teste CASE VAL', async function () {
	var teste = new nExpressionExecutor();
	teste.registrarValor('TESTE', "A");
    var result = await teste.execute('CASE(TESTE, ["B", "A", "C"], [1,3,2])');
    expect(result).to.equal(3);	    
  })  
  it('Teste CASE VAL_OPCOES', async function () {
	var teste = new nExpressionExecutor();
	teste.registrarValor('TESTE', "A");
	teste.registrarValor('VALA', 'A');
    teste.registrarValor('VALB', 'B');
    teste.registrarValor('VALC', 'C');
    var result = await teste.execute('CASE(TESTE, [VALB, VALA, VALC], [1,3,2])');
    expect(result).to.equal(3);	    
  }) 
  it('Teste CASE VAL_VALORES', async function () {
	var teste = new nExpressionExecutor();
	teste.registrarValor('TESTE', "A");
	teste.registrarValor('VALA', "A");
    teste.registrarValor('VALB', "B");
    teste.registrarValor('VALC', "C");
	teste.registrarValor('VAL1', 1);
    teste.registrarValor('VAL2', 2);
    teste.registrarValor('VAL3', 3);
    var result = await teste.execute('CASE(TESTE, [VALB, VALA, VALC], [VAL1, VAL3, VAL2])');
    expect(result).to.equal(3);	    
  }) 
  it('Teste CASE_ELSE', async function () {
    var result = await new nExpressionExecutor().execute('CASE("D", ["B", "A", "C"], [1,3,2], 0)');
    expect(result).to.equal(0);	    
  })  
  it('Teste INCDAY', async function () {
    var result = await new nExpressionExecutor().execute('INCDAY("2010-01-01", 1)');
    expect(result).to.equal(moment('2010-01-02').format('YYYY-MM-DD'));    
  })
  it('Teste INCMONTH', async function () {
    var result = await new nExpressionExecutor().execute('INCMONTH("2010-01-01", 1)');
    expect(result).to.equal(moment('2010-02-01').format('YYYY-MM-DD'));    
  })
  it('Teste INCYEAR', async function () {
    var result = await new nExpressionExecutor().execute('INCYEAR("2010-01-01", 1)');
    expect(result).to.equal(moment('2011-01-01').format('YYYY-MM-DD'));    
  })
  it('Teste NOT', async function () {
    var result = await new nExpressionExecutor().execute('NOT(true)');
    expect(result).to.equal(false);    
  })
  it('Teste MAIORQUE ValorFormatado', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('PRECEITABRUTAANUAL', '70.000,00');
    var result = await teste.execute('MAIORQUE(PRECEITABRUTAANUAL, 60000)');
    expect(result).to.equal(true);    
  })
  it('Teste MENORQUE ValorFormatado', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('PRECEITABRUTAANUAL', '70.000,00');
    var result = await teste.execute('MENORQUE(PRECEITABRUTAANUAL, 60000)');
    expect(result).to.equal(false);    
  })
  it('Teste MENORQUE ValorFormatado', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('VALORREAL', "500,00");
    teste.registrarValor('VALORINDEVIDO', "150,00");
    var result = await teste.execute('SUBTRAIR(VALORREAL, VALORINDEVIDO)');
    expect(result).to.equal(350.00);    
  })  
  it('Teste MENORQUE ValorFormatado e nao formatado', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('VALORREAL', "5.000,00");
    teste.registrarValor('VALORINDEVIDO', "1000,00");
    var result = await teste.execute('SUBTRAIR(VALORREAL, VALORINDEVIDO)');
    expect(result).to.equal(4000.00);    
  })  
  it('Teste GrupoImposto ValorFormatado e nao formatado', async function () {
    var teste = new nExpressionExecutor();
    teste.registrarValor('GRUPOIMPOSTO', "1.01");
    var result = await teste.execute('IGUAL(GRUPOIMPOSTO, "1.01")');
    expect(result).to.equal(true);    
  })  

	it('Teste valor: "" deve ser vazio', async function () {
		let test = new nExpressionExecutor();
		let value = '';

		test.registrarValor('field', value);

		let result = await test.execute('VAZIO(field)');

		expect(result).to.equal(true);
	})

	it('Teste valor: " " deve ser vazio', async function () {
		let test = new nExpressionExecutor();
		let value = ' ';

		test.registrarValor('field', value);

		let result = await test.execute('VAZIO(field)');

		expect(result).to.equal(true);
	})

	it('Teste valor: "a " não deve ser vazio', async function () {
		let test = new nExpressionExecutor();
		let value = 'a ';

		test.registrarValor('field', value);

		let result = await test.execute('VAZIO(field)');

		expect(result).to.equal(false);
	})

	it('Teste valor: " a" não deve ser vazio', async function () {
		let test = new nExpressionExecutor();
		let value = 'a ';

		test.registrarValor('field', value);

		let result = await test.execute('VAZIO(field)');

		expect(result).to.equal(false);
	})

	it('Teste valor: 1 não deve ser vazio', async function () {
		let test = new nExpressionExecutor();
		let value = 1;

		test.registrarValor('field', value);

		let result = await test.execute('VAZIO(field)');

		expect(result).to.equal(false);
	})

	it('Teste valor: "0" não deve ser vazio', async function () {
		let test = new nExpressionExecutor();
		let value = "0";

		test.registrarValor('field', value);

		let result = await test.execute('VAZIO(field)');

		expect(result).to.equal(false);
	})

	it('Teste valor: " 1" não deve ser vazio', async function () {
		let test = new nExpressionExecutor();
		let value = " 1";

		test.registrarValor('field', value);

		let result = await test.execute('VAZIO(field)');

		expect(result).to.equal(false);
	})

	it('Teste valor: "1 " não deve ser vazio', async function () {
		let test = new nExpressionExecutor();
		let value = " 1";

		test.registrarValor('field', value);

		let result = await test.execute('VAZIO(field)');

		expect(result).to.equal(false);
	})
});