import path from 'path';
import Mocha from 'mocha';
import dotenv from 'dotenv';
import glob from 'glob';
import { DOMBuilder } from './Utils/DOMBuilder';

/* get config from .env */
dotenv.config();

/** Instantiate a Mocha with options */
var mocha = new Mocha({
		"globals": ['window', 'document', '$', 'jQuery', 'KTUtil'],
		"ignoreLeaks": false,
    "reporter": "mocha-multi-reporters",
    "reporterOptions": {
        "reporterEnabled": "mochawesome, mocha-jenkins-reporter",    
        "mochaJenkinsReporterReporterOptions": {
          "junit_report_name": "Cloud unit test for javascript",
          "junit_report_path": `${process.env.MOCHA_RESULT_PATH}${process.env.RESULT_FILE_NAME}.xml`,
          "junit_report_stack": 1
        },
        "mochawesomeReporterOptions": {
          "reportDir": process.env.MOCHA_RESULT_PATH,
          "reportFilename": process.env.RESULT_FILE_NAME,
					"fullTrace": true,
					"showPassed": false,
        },
		},
});

function getGlob() {
	const testDir = process.env.CLI_TEST_DIR;
	const expression = (name) => (name ? name.replace(/\.spec\.[j|t]s/, '') : '*'); 
	let fileName = process.argv[2];

	return `${testDir}/**/${expression(fileName)}.spec.[j|t]s`;
}

async function runMochaOnFiles(er, files) {
	// Create an initial dom with jQuery globals
	// to properly load dependencies that use dom  
	await DOMBuilder.includeGlobalsJq(await DOMBuilder.build());
  // files is an array of filenames.
  // If the `nonull` option is set, and nothing
  // was found, then files is ["**/*.js"]
  // er is an error object or null.
	console.log("files: ", files);
	
  for (let file of files) 
    mocha.addFile(path.resolve(file));
  
  mocha.run(function(failures) {
    process.exitCode = failures ? 1 : 0;
	});
}

glob(getGlob(), {}, runMochaOnFiles);