import FormManager from "../../../js/FormManager";
import Initializer from "../../../js/initializer";
import { FormState } from "../../../js/nCadastro";
import { FormatErrorMessage } from "../../../js/nDialog";
import { Formulario } from "../../../js/nFormulario";

declare var KTApp: any;
declare var axios: any;
declare var CarregarDataCadastro: any;

interface LoadCadPageDynamicTabProps {
    Class: string;
    Alias: string;
    MasterClass: string;
    MasterFields: Array<string>;
    DetailFields: Array<string>;
}

export const LoadCadPageDynamicTab = async ({ Alias, Class, DetailFields, MasterClass, MasterFields }: LoadCadPageDynamicTabProps) => {
    const tabEl = $(`#${Class}${Alias}`);
    const { loaded } = tabEl.data();
    
    if (loaded) return;
    
    KTApp.blockPage();
    
    tabEl.data('loaded', true);
    
    try {
        const form = FormManager.byname(MasterClass) as Formulario;
        const params = getParams({ Alias, Class, form });
        const { location: { origin, pathname } } = window;
        const { data, request, config } = await axios.get(`Detalhe?Scope=${JSON.stringify(params)}`);

        const requestURL = `${origin}${pathname.replace(/(\S+\/)\w+/, '$1')}${config.url}`;
        const responseURL = decodeURIComponent(request.responseURL);
        const redirectDetected = responseURL !== requestURL;

        if (redirectDetected) {
            window.location.reload();
        }
        else {
            const detailEl = $(data as string);

            tabEl.append(detailEl);

            FormManager.createDynamicDetailForm({
                Alias,
                Class,
                contextElement: tabEl,
                elementMissingProps: {
                    formdetalhe: true,
                    masterclass: MasterClass,
                    masterfields: MasterFields,
                    detailfields: DetailFields,
                },
            });

            loadDataFromDynamicTab({ Alias, Class, element: tabEl });

            renderDynamicContent(tabEl);

            scrollToElement(tabEl);
        }
    }
    catch(e) {
        tabEl.data('loaded', false);

        console.error(e);

        FormatErrorMessage('Foram encontrados problemas ao carregar a página');
    }
    finally {
        KTApp.unblockPage();
    }
};
  
interface GetParamsProps extends Pick<LoadCadPageDynamicTabProps, 'Alias' | 'Class'> {
    form: Formulario;
}

const getParams = ({ Alias, Class, form }: GetParamsProps) => ({
    Class,
    Alias,
    ...getFilter(form),
    ...getInsertParam(form),
});

const getFilter = (form: Formulario) => {
    const Filtros = form.MontarFiltroChavePrimaria();

    return Filtros ? { Filtros } : {};
};

const getInsertParam = (form: Formulario) => {
    return form?.estado == FormState.cadInsert
        ? { new: true }
        : {};
};

export const scrollToElement = (element: JQuery<HTMLElement>) => {
    const header = $('#kt_header').outerHeight() || 0;
    const subHeader = $("#kt_subheader").outerHeight() || 0;
    const elementYPosition = element.offset()?.top || 0;

    scrollTo(0, elementYPosition - header - subHeader);
};

const renderDynamicContent = (contextElement: JQuery<HTMLElement>) => {
    const dependencies = Initializer._searchDependencies(contextElement);

    Initializer._requestDynamicDependency(dependencies);
};

const loadDataFromDynamicTab = ({ Alias = '', Class, element }: { Alias: string, Class: string, element: JQuery<HTMLElement> }) => {
    const dataEl = $(`#loadCadPage${Class}${Alias}`, element);

    if (!dataEl.length) return;

    const { data } = dataEl.data();

    dataEl.remove();

    CarregarDataCadastro({
        ...data,
        _AAlias: Alias,
    });
};