import { Field, GroupColumn, Prop, ToolButton } from "../../fra-control/types";
import { FormRepeaterForm } from "../form-repeater";
import { FormRepeaterFields } from "../form-repeater-field";
import { AddButton as getAddButton } from "./add-button";
import { DeleteButton as getDeleteButton } from "./delete-button";
import { SaveButton as getSaveButton } from "./save-button";

declare var KTApp: any;

export interface ButtonProps extends FormOptions {
    /** ntcGrid Field for FormRepeaterFields */
    field?: Field;
    button: ToolButton;
}

interface ButtonsProp extends FormOptions {
    /** ntcGrid Field for FormRepeaterFields */
    field?: Field;
    toolButtons: ToolButton[];
}

interface FormOptions extends Prop<(Field | GroupColumn)[]> {
    uuid: string;

    /** Available only on execution time */
    formRepeater?: FormRepeaterFields | FormRepeaterForm;
}

interface RenderButtonProps {
    className: string;
    hint: string;
    icon: string;
    onClick?: any;
    type: string;
    visible: boolean;
}

export const Buttons = (props: ButtonsProp): JQuery<HTMLElement> => {
    const { toolButtons } = props;

    let wrapper = $('<div class="form-repeater__button-group col mt-auto">');

    reorderButtons(toolButtons).forEach((button) => {
        wrapper.append(getVariant({
            button,
            formInstance: props.formInstance,
            formProp: props.formProp,
            formRepeater: props.formRepeater,
            uuid: props.uuid,
            item: props.item,
            field: props.field
        }))
    })

    return wrapper;
}

function reorderButtons(buttons: ToolButton[]): ToolButton[] {
    const save = 'actDataSetPost';
    const add = 'actDataSetInsert';
    const del = 'actDataSetDelete';

    return buttons.sort((a, b) => {
        if (a.Name == save) return -1;
        if (b.Name == save) return 1;

        if (a.Name == del) return -1;
        if (b.Name == del) return 1;

        if (a.Name == add) return -1;
        if (b.Name == add) return 1;

        return 0;
    });
}

const getVariant = (props: ButtonProps): JQuery<HTMLElement> => {
    switch (props.button.Name) {
        case 'actDataSetInsert': return getAddButton(props);
        case 'actDataSetDelete': return getDeleteButton(props);
        case 'actDataSetPost': return getSaveButton(props);
        // default: return defaultButton(props);
    }

    return $()
}

export const renderButton = (props: RenderButtonProps): JQuery<HTMLButtonElement> => {
    const {
        className = 'btn-text-dark  btn-hover-transparent-dark',
        hint,
        icon,
        onClick = (() => { }),
        type = 'button',
        visible = true,
    } = props;

    let defaultClass = 'btn';
    let iconEl = icon ? $(`<i class="${icon}">`) : $('');
    let button = ($('<button>') as JQuery<HTMLButtonElement>)
        .attr({
            class: `${defaultClass} ${className}`,
            title: hint,
            style: `border-color: #ababab;${visible ? '' : ' display: none'}`,
            type,
        })
        .append(iconEl)

    if (onClick) {
        button.on('click', onClick);
    }

    if (hint) {
        $(() => KTApp.initTooltip(button)); //await until metronic is loaded
    }

    return button;
};