import { actDataSetDeleteExecute } from "../../../../js/form/actions/ActDataSetDeleteExecute";
import { FormState } from "../../../../js/nCadastro";
import { FormRepeaterForm } from "../form-repeater";
import { FormRepeaterFields } from "../form-repeater-field";
import { ButtonProps, renderButton } from "./buttons";
import { defaultActionOptions } from "./default-button";

export const DeleteButton = (props: ButtonProps): JQuery<HTMLElement> => {
    let buttonEl = renderButton({
        className: 'btn-icon btn-text-dark btn-hover-transparent-dark btn-sm',
        hint: 'Remover',
        icon: `la la-${props.button.Name}`,//props.button.icon,
        type: 'button',
        visible: true,
        onClick: getOnClick(props),
    });

    if (props.formRepeater instanceof FormRepeaterForm) {
        setActionForm(props, buttonEl);
    }

    return buttonEl;
}

const getOnClick = (props: ButtonProps): Function | void => {
    if (props.formRepeater instanceof FormRepeaterFields) {
        return (event: Event) => removeFormRepeater(event, props);
    }
}

const removeFormRepeater = (event: Event, props: ButtonProps) => {
    const { currentTarget } = event;

    let tooltip = $(currentTarget!).data('bs.tooltip');

    if (tooltip) {
        tooltip.dispose();
    }

    props.formRepeater?.removeInstance(props.uuid);
}

const setActionForm = (props: ButtonProps, buttonEl: JQuery<HTMLElement>) => {
    props.formInstance!.setAction({
        elementTrigger: buttonEl.get(0),
        form: props.formInstance,
        handler: {
            type: 'click',
            action: (actionContext: actDataSetDeleteExecute, event: Event) => {
                let { formInstance } = props;
                let hasDataToDelete = formInstance?.hasRecord() && (formInstance.estado != FormState.cadInsert) && formInstance.tipoObjeto != 'ntoProcesso';
                
                if (hasDataToDelete) {
                    actionContext.execute(event);
                }
                else {
                    removeFormRepeater(event, props);
                }
            }
        },
        onExecuted: async ({ originalEvent, response }: any) => {
            let resp = await response;
            
            console.log('response', resp)
            
            removeFormRepeater(originalEvent, props);
        },
        ...defaultActionOptions({ ...props, idElement: buttonEl.attr('id')! }),
    });
}