import { Formulario } from "../../../js/nFormulario";
import { Field, GroupColumn, Prop, ToolButton } from "../fra-control/types";
import { setAttrIfHasValue } from "../fra-control/util/util";

export interface FormRepeaterItem {
    /** Wrapper element of FormRepeater instance  */
    element: JQuery<HTMLElement>;

    /** When FormRepeaterFields */
    fieldName?: string;
    
    formComponent: Formulario;
}

export interface RenderProps extends Prop<(Field | GroupColumn)[]> {
    append?: boolean;
    buttons: ToolButton[];
    element: JQuery<HTMLElement>;
    formRepeaterElementId?: string;
    uuid: string;
}

export interface FormRepeaterProps extends Prop<(Field | GroupColumn)[]> {
    append?: boolean;
    element: JQuery<HTMLElement>;
    isField?: boolean;
}

export abstract class FormRepeaterBase {
    public abstract addInstance(props: FormRepeaterProps): FormRepeaterItem;
    
    public abstract removeInstance(uuid: string): void;
    
    public abstract loadData(data: unknown): void;
    
    protected abstract createFormRepeaterItem(props: RenderProps): FormRepeaterItem;

    protected _items = new Map<string, FormRepeaterItem>();
    
    protected _elementRoot: JQuery<HTMLElement> | null = null;

    get element() {
        return this._elementRoot;
    }
    
    protected render(props: RenderProps): FormRepeaterItem {
        return props.append ? this.append(props) : this.createNew(props);
    }

    protected append(props: RenderProps): FormRepeaterItem {
        let item = this.createFormRepeaterItem(props);
        
        item.element.insertAfter(props.element);
        
        return item;
    }
    
    protected createNew(props: RenderProps): FormRepeaterItem {
        let wrapper = $(`<div class="form-repeater w-100">`);
        let item = this.createFormRepeaterItem(props);

        setAttrIfHasValue(wrapper, {
            id: props.formRepeaterElementId
        });

        props.element?.replaceWith(wrapper);

        wrapper.append(item.element);

        this._elementRoot = wrapper;

        return item;
    }

    public getItem(uuid: string): FormRepeaterItem | null {
        return this._items.get(uuid) ?? null;
    }
}