import moment from 'moment';
import { Field } from '../../../fra-control/types';
import { ItemData } from '../../types';

export function getDataToLoad(configFields: Field[], data: ItemData): ItemData {
    const mappedData = mapLookupGridToLookupForm(configFields, data);

    return fixValues(mappedData);
}

export function mapLookupGridToLookupForm(configFields: Field[], data: ItemData): ItemData {
    const lookupMap = configFields
        .filter((field) => field.ntcLinkEdit || field.Lookup)
        .reduce((mapped, field, index, fields) => {
            const nextField = fields[index + 1];

            if (nextField?.Lookup) {
                return {
                    ...mapped,
                    [nextField.Name]: field.Name,
                };
            }

            return mapped;
        }, {});

    return Object
        .entries(lookupMap)
        .reduce((newData, [descriptionFieldName, codeFieldName]) => ({
            ...newData,
            [`${codeFieldName}RESULT`]: newData[descriptionFieldName],
        }), { ...data });
}

export function fixValues(data: ItemData): ItemData {
    const fix = (value: string) => {
        if (value.match(/\d{2}\/\d{2}\/\d{4}/)) return moment(value, 'MM/DD/YYYY').format('YYYY-MM-DD');
        if (value.match(/\d{2}\/\d{2}\/\d{4} \d{2}:\d{2}:\d{2}/)) return moment(value, 'MM/DD/YYYY h:m:s').format('YYYY-MM-DD HH:mm:ss');
        return value.replace(/&nbsp/g, ' ');
    };

    return Object
        .entries(data)
        .reduce((newData, [key, value]) => ({
            ...newData,
            [key]: value ? fix(value) : value,
        }), {});
}

export function splitData<T>(data: T[], size: number): T[][] {
    return [data.slice(0, size), data.slice(size)];
} 
