import classNames from 'classnames';
import { ControlElementCreator } from "../types";
import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { setAttrIfHasValue } from "../util/util";
import { FraControlElement } from "../fra-control-element/fra-control-element";
import { loadBase64File } from "../../../../js/nArquivos";

declare var focusImputFunc: any;

export const DicDiretorioAbrir: ControlElementCreator = (props) => {
    const { item, formProp } = props;
    const inputBlock = InputBlock(props);
    const label = Label(props);
    const wrapper = getWrapper(props);
    const controlElement = FraControlElement(props);

    inputBlock.attr('id', `formField${item.Name}_DIRABRIR${formProp.Class}`);

    return inputBlock.append(label, wrapper, controlElement);
};

const getWrapper: ControlElementCreator = (props) => {
    const wrapper = $('<div>').addClass(classNames({
        'select-file': true,
        'col-12 col-sm-9 col-lg-7': !props.formProp.MultiColuna,
    }));
    const customFileWrapper = $('<div class="custom-file mr-2">');
    const input = getInput(props);
    const label = getLabel(props);

    customFileWrapper.append(input, label);
    
    return wrapper.append(customFileWrapper)
};

const getInput: ControlElementCreator = (props) => {
    const { formProp, item } = props;
    const input = $('<input />')
        .attr({
            'data-alias': formProp.Alias || '',
            'data-classname': item.DMClassName,
        })
        .on('focus', (event) => { focusImputFunc({ element: event.target }); })
        .on('change', (event) => {
            loadBase64File((event.target as HTMLInputElement).files, `${item.Name}_DIRABRIR`, item.DMClassName, formProp.Alias || '');
        });

    setAttrIfHasValue(input, {
        accept: item.Filter,
        autocomplete: 'off',
        class: classNames({
            'form-control kt-input cad-control custom-file-input cad-logchange': true,
            [`${item.ControlClass}`]:  item.ControlClass,
            [`ctrl${item.Name}`]: true,
        }),
        disabled: item.disabled,
        id: `ctrl${item.Name}_DIRABRIR`,
        name: `p.${item.Name}_DIRABRIR`,
        required: item.Required,
        step: 'any',
        title: item.Hint,
        type: item.InputType, 
        value: item.Valor,
        'data-AutoIncrementar': item.AutoIncrementar,
        'data-Dicionario': item.Dicionario,
    });

    return input;
};

const getLabel: ControlElementCreator = ({ item }) => {
    return $('<label>')
        .attr({
            class: 'custom-file-label text-left',
            for: `p.${item.Name}`
        })
        .append('Nenhum Arquivo Selecionado');
};