import { ControlElementCreator, Field, FormProp } from "../types";
import classNames from 'classnames';

export const Label: ControlElementCreator = (props) => {
    if (!props.item.DisplayLabel) return $();

    return buildLabel(props);
}

const buildLabel: ControlElementCreator = ({ item, formProp }) => {
    let classList = getClassListForLabel(item, formProp);
    let spanText = getSpanText(item); 
    let spanRequired = getSpanRequired(item);

    return $('<label>')
        .attr({
            id: `labelctrlopt${item.Name}`,
            for: `ctrl${item.Name}`,
            class: classList
        })
        .append(spanText)
        .append(spanRequired);
}

function getClassListForLabel(field: Field, formOpt: FormProp) {
    return classNames({
        'col-form-label': true,
        'col-12 col-sm-3': !formOpt.MultiColuna,
        'kt-font-warning': field.LabelClass == "label-warning",
        'search-label': field.SearchStyle,
        [`${field.LabelClass}`]: field.LabelClass,
    });
}

function getSpanText(field: Field): JQuery<HTMLElement> {
    return $('<span>')
        .attr('data-text', '')
        .html(field.DisplayLabel);
}

function getSpanRequired(field: Field): JQuery<HTMLElement> {
    return $('<span>')
        .attr({
            'data-span-required': field.Name,
            'style': field.Required ? '' : 'display:none;'
        })
        .html('*');
}