import { ControlElementCreator, Field } from "../types";
import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';
import { FraControlElement } from "../fra-control-element/fra-control-element";

declare var focusImputFunc: any;
declare var carregarDadosDoCEP: any;

export const NtcActBuscarCEP: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = Label(props);
    let wrapper = getWrapper(props);
    let insertFormEl = FraControlElement(props);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(insertFormEl);
}

const getWrapper: ControlElementCreator = (props) => {
    const { formProp } = props;

    let wrapper = $('<div>').addClass(formProp.MultiColuna ? 'input-group' : 'col-12 col-sm-9 col-lg-7');
    let inputWrapper = $('<div>').addClass('pl-0 pr-0 input-group input-addon-default col-md-12');
    let input = getInput(props);
    let button = getButton(props);

    inputWrapper
        .append(input)
        .append(button);

    return wrapper.append(inputWrapper);
}

const getInput: ControlElementCreator = ({ item, formProp }) => {
    let classList = getInputClassList(item);
    let input = $('<input>').on('focus', function ({ target }) { focusImputFunc({ element: target }) });

    setAttrIfHasValue(input, {
        autocomplete: 'off',
        class: classList,
        disabled: item.disabled,
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        readonly: item.ReadOnly,
        required: item.Required,
        step: 'any',
        style: item.TextCase,
        title: item.Hint,
        type: item.InputType,
        value: item.Valor,
        'data-alias': formProp.Alias,
        'data-classname': item.DMClassName,
        'data-dicionario': item.Dicionario,
        'data-mask': item.mask,
    });

    return input;
}

const getInputClassList = ({ ControlClass, Name }: Field): string => {
    return classNames({
        'form-control kt-input cad-control cad-logchange': true,
        [`ctrl${Name}`]: true,
        [`${ControlClass}`]: ControlClass
    })
}

const getButton: ControlElementCreator = ({ item, formProp }) => {
    let buttonWrapper = $('<div class="input-group-append">');
    let icon = $('<i class="la la-search">');
    let button = $('<button>')
        .attr({
            class: 'input-group-text btn btn-text-dark btn-hover-transparent-dark',
            id: `btn${item.Name}`,
            style: 'border-color: #ababab;',
            tabindex: '-1',
            type: 'button',
            'data-id': `ctrl${item.Name}`,
            'data-original-title': 'Buscar CEP',
        })
        .on('click', function () {
            carregarDadosDoCEP(item.DMClassName || '', formProp.Alias || '', item.LinkLookupResultField, item.Name);
        });

    button.append(icon);
    
    return buttonWrapper.append(button);
}