import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { ControlElementCreator, Field, FormProp, NameValue } from "../types";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';
import { FraControlElement } from "../fra-control-element/fra-control-element";

declare var focusImputFunc: any;

type CheckboxElementCreator = (props: {
    item: Field,
    formProp: FormProp,
    checkbox: NameValue
}) => JQuery<HTMLElement>;

export const NtcCheckListBox: ControlElementCreator = (props) => {
    let wrapper = getWrapper(props);
    let label = Label(props);
    let inputBlock = InputBlock(props);
    let checkboxWrapper = getCheckboxWrapper(props);

    wrapper.append(checkboxWrapper);

    return inputBlock
        .append(label)
        .append(wrapper);
}

const getWrapper: ControlElementCreator = ({ item, formProp }) => {
    let wrapper = $('<div>')
        .addClass(classNames({
            'col-12 ntcCheckListBox': true,
            'col-sm-9 col-lg-7': !formProp.MultiColuna
        }));

    setAttrIfHasValue(wrapper, {
        'data-alias': formProp.Alias,
        'data-classname': item.DMClassName,
    });

    return wrapper;
}

const getCheckboxWrapper: ControlElementCreator = ({ item, formProp }) => {
    let checkboxWrapper = $('<div>')
        .addClass(classNames({
            'checkbox-wrapper': true,
            [`${item.ControlClass}`]: item.ControlClass
        }))
        .attr({
            id: `ctrl${item.Name}check`,
            'data-scroll': 'true'
        })

    item.NomesValoresList?.forEach(checkbox => {
        let checkboxEl = getCheckbox({ item, formProp, checkbox });
        let controlElement = FraControlElement({
            formProp,
            item: {
                ...item,
                nvValor: checkbox.nvValor
            }
        })
        checkboxWrapper.append(controlElement);
        checkboxWrapper.append(checkboxEl)
    });

    return checkboxWrapper;
}

const getCheckbox: CheckboxElementCreator = (props) => {
    const { checkbox } = props;
    const exists = (val: any) => ![null, undefined].includes(val);

    let label = $('<label class="kt-checkbox">');
    let text = $(`<div>`).append(checkbox.nvNome);

    if (exists(checkbox.nvValor)) {
        let input = getInputCheckbox(props);

        label
            .append(input)
            .append($('<span>'))
    }
    else {
        text.attr('style', 'font-size:15px!important;');
    }

    label.append(text);

    return label;
}

const getInputCheckbox: CheckboxElementCreator = ({ checkbox, item, formProp }) => {
    let input = $('<input>')
        .on('focus', function ({ target }) { focusImputFunc({ element: target }) })

    setAttrIfHasValue(input, {
        autocomplete: 'off',
        class: 'cad-control cad-logchange',
        checked: item.checked,
        disabled: item.disabled,
        form: getFormId(formProp),
        id: `ctrl${item.Name}_checkbox_${checkbox.nvValor}_ctrl${item.Name}`,
        name: `p.${item.Name}_checkbox_${checkbox.nvValor}`,
        style: item.TextCase,
        title: item.Hint,
        type: 'checkbox',
        value: 'true',
        'data-alias': formProp.Alias,
        'data-autoincrementar': item.AutoIncrementar,
        'data-classname': item.DMClassName,
        'data-dicionario': item.Dicionario
    });

    return input;
}