import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { ControlElementCreator, Prop } from "../types";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';
import { FraControlElement } from "../fra-control-element/fra-control-element";

declare var focusImputFunc: any;

export const NtcComboBox: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = Label(props);
    let wrapper = getWrapper(props);
    let controlElement = FraControlElement(props);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(controlElement);
};

const getWrapper: ControlElementCreator = (props) => {
    let wrapper = $('<div class="select-input">');
    let select = getSelect(props);

    wrapper.append(select);

    if (!props.formProp.MultiColuna) {
        let divCol = $('<div class="col-12 col-sm-9 col-lg-7">');

        return divCol.append(wrapper);
    }

    return wrapper;
};

const getSelect: ControlElementCreator = (props) => {
    const { item, formProp } = props;
    let classList = getSelectClassList(props);
    let select = $('<select>')
        .on('focus', function({ target }) { focusImputFunc({ element: target }) })

    setAttrIfHasValue(select, {
        class: classList, 
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        disabled: item.disabled, 
        name: `p.${item.Name}`, 
        readonly: item.ReadOnly,
        required: item.Required, 
        title: item.Hint, 
        value: item.Valor, 
        'data-alias': formProp.Alias,
        'data-autoincrementar': item.AutoIncrementar, 
        'data-classname': item.DMClassName,
        'data-dicionario': item.Dicionario, 
    });

    item.NomesValoresList?.forEach(opt => {
        let option = $('<option>').append(opt.nvNome);
        
        setAttrIfHasValue(option, {
            value: opt.nvValor,
            selected: opt.nvSelected
        });

        select.append(option);
    });

    return select;
};

const getSelectClassList = ({ item: { Name, SearchStyle } }: Prop): string => {
    return classNames({
        'form-control kt-input cad-control cad-logchange': true,
        'border-radius-left border-radius-right form-control-search': SearchStyle,
        [`ctrl${Name}`]: true,
    });
};