import { ControlElementCreator, Field } from "../types";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';

declare var focusImputFunc: any;

export const CurrencyInput: ControlElementCreator = (props) => {
    const { item, formProp } = props;

    let classList = getInputClassList(item);
    let element = getInputGroup();
    let prependIcon = getPrependIcon(item);

    let input = $('<input />')
        .attr({
            'data-alias': formProp.Alias,
            'data-classname': item.DMClassName,
        })
        .on('focus', function(event) { focusImputFunc({ element: event.target }) });

    setAttrIfHasValue(input, {
        autocomplete: 'off',
        class: classList,
        disabled: item.disabled,
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        readonly: item.ReadOnly,
        required: item.Required,
        step: 'any',
        style: item.TextCase,
        title: item.Hint,
        type: item.InputType,
        value: item.Valor,
        'data-autoincrementar': item.AutoIncrementar,
        'data-dicionario': item.Dicionario,
        'data-mask': item.mask,
        'data-mask-reverse': item.maskreverse,
    })

    return element
        .append(prependIcon)
        .append(input)
}

function getInputClassList({ ControlClass, Name, SearchStyle }: Field): string {
    return classNames({
        'form-control kt-input cad-control cad-logchange': true,
        'border-radius-right form-control-search': SearchStyle,
        [`ctrl${Name}`]: true,
        [`${ControlClass}`]: ControlClass,
    })
}

function getInputGroup(): JQuery<HTMLElement> {
    return $('<div class="input-group">');
}

function getPrependIcon(field: Field): JQuery<HTMLElement> {
    let wrapper = $('<div class="input-group-prepend">');
    let span = $('<span>')
        .addClass(classNames({
            'input-group-text': true,
            'border-radius-left input-append-blue': field.SearchStyle
        }))

    let icon = $('<i>')
        .addClass(classNames({
            'fa fa-fw fa-dollar-sign': true,
            'icon-blue': field.SearchStyle
        }));

    span.append(icon);

    return wrapper.append(span);
}