import { ControlElementCreator, Field } from "../types";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';
import { nFraControl } from "../../../../js/nFraControl";
import { configureDatepicker } from '../../../../js/setDatepicker';

export const DateInput: ControlElementCreator<Field, JQuery<HTMLElement>[]> = (props) => {
    const { item, formProp } = props;

    let classList = getInputClassList(item);
    let element = getInputGroup();
    let prependIcon = getPrependIcon(item);

    let inputHidden = $('<input />')
        .attr({
            'data-alias': formProp.Alias,
            'data-classname': item.DMClassName,
        });

    let input = $('<input />')
        .attr({
            'data-alias': formProp.Alias,
            'data-classname': item.DMClassName,
        })
        .on('focus', function () { nFraControl.focusInputFunc({ element: inputHidden } as any); })

    setAttrIfHasValue(input, {
        autocomplete: 'off',
        class: classList,
        disabled: item.disabled,
        required: item.Required,
        readonly: item.ReadOnly,
        style: item.TextCase,
        title: item.Hint,
        type: 'text',
        value: '',
        'data-dicionario': item.Dicionario,
        'data-mask': item.mask,
        'data-original-value': item.Valor,
    });

    setAttrIfHasValue(inputHidden, {
        autocomplete: 'off',
        class: 'hasdatepicker',
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        type: 'hidden',
        value: item.Valor,
        'data-dicionario': item.Dicionario
    });

    element.append(prependIcon).append(input);

    configureDatepicker(input);

    return [
        element,
        inputHidden
    ];
}

function getInputClassList({ ControlClass, Name, SearchStyle }: Field) {
    return classNames({
        'form-control kt-input cad-control cad-logchange': true,
        'form-control-search border-radius-right': SearchStyle,
        [`ctrl${Name}`]: true,
        [`${ControlClass}`]: ControlClass,
    })
}

function getInputGroup(): JQuery<HTMLElement> {
    return $('<div class="input-group">');
}

function getPrependIcon(field: Field): JQuery<HTMLElement> {
    let wrapper = $('<div class="input-group-prepend">');
    let span = $('<span>')
        .addClass(classNames({
            'input-group-text': true,
            'border-radius-left': field.SearchStyle,
            'btn btn-icon': !field.SearchStyle
        }));

    let icon = $('<i>')
        .addClass(classNames({
            'far fa-calendar-alt': true,
            'icon-blue': field.SearchStyle
        }));

    span.append(icon);

    return wrapper.append(span);
}