import { ControlElementCreator, Field } from '../types';
import { InputBlock } from '../input-block/input-block';
import { DateInput } from './date-input';
import { MaskedInput } from './masked-input';
import { CurrencyInput } from './currency-input';
import { DefaultInput } from './default-input';
import { Label as getLabel } from '../label/label';
import { FraControlElement } from '../fra-control-element/fra-control-element';

type FieldVariation = 'date' | 'currency' | 'masked' | 'default';

export const NtcEdit: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = getLabel(props);
    let wrapper = getWrapper(props);
    let input = getInput(props);
    let insertFormEl = FraControlElement(props);
    
    wrapper.append(input);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(insertFormEl)
}

const getWrapper: ControlElementCreator = ({ formProp: { MultiColuna } }) => {
    let multiColumnClass = MultiColuna ? 'input-group' :  'col-12 col-sm-9 col-lg-7';

    return $('<div>').addClass(multiColumnClass);
}

const getInput: ControlElementCreator<Field, JQuery<HTMLElement> | JQuery<HTMLElement>[]> = (props) => {
    const { InputType, mask } = props.item;
    const variation = getVariation(InputType, mask!);

    switch(variation) {
        case "currency": return CurrencyInput(props);
        case "masked": return MaskedInput(props);
        case "date": return DateInput(props);
        default: return DefaultInput(props);
    }
}

function getVariation(InputType: string, mask: string): FieldVariation {
    if (InputType == 'date') return 'date';
    if (!mask) return 'default';
    if (mask == '#.##0,00') return 'currency';
    return 'masked';
}