import { ControlElementCreator, Field } from "../types";
import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';
import { FraControlElement } from "../fra-control-element/fra-control-element";

declare var focusImputFunc: any;

export const NtcHistorico: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = Label(props);
    let wrapper = getWrapper(props);
    let controlElement = FraControlElement(props);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(controlElement);
}

const getWrapper: ControlElementCreator = (props) => {
    let wrapper = $('<div>')
        .addClass(classNames({
            'col-12': true,
            'col-sm-9 col-lg-7': !props.formProp.MultiColuna
        }));

    let input = getInput(props);
    let complementDiv = getComplementDiv(); 
    
    return wrapper
        .append(input)
        .append(complementDiv)
}

const getInput: ControlElementCreator = ({ item, formProp }) => {
    let classList = getClassList(item);
    let input = $('<input>')
        .append(item.Valor || '')
        .on('focus', ({ target }) => focusImputFunc({ element: target })) 
    
    setAttrIfHasValue(input, {
        autocomplete: 'off', 
        class: classList, 
        disabled: item.disabled, 
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        step: 'any',
        title: item.Hint,
        type: 'hidden', 
        value: item.Valor,
		'data-autoincrementar': item.AutoIncrementar, 
        'data-dicionario': item.Dicionario
    });

    return input;
}

const getClassList = ({ ControlClass, Name }: Field) => {
    return classNames({
        'form-control kt-input cad-control cad-logchange complhistoricoctb': true,
        [`ctrl${Name}`]: true,
        [`${ControlClass}`]: ControlClass,
    })
}

const getComplementDiv = (): JQuery<HTMLElement> => {
    return $('<div id="divComplHist" class="divComplHist">')
}