import { ControlElementCreator, Field } from "../types";
import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { getFormId, setAttrIfHasValue } from "../util/util";
import { showThumbnail } from "../../../../js/nArquivos";
import classNames from 'classnames';
import { FraControlElement } from "../fra-control-element/fra-control-element";

declare var focusImputFunc: any;

export const NtcImage: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = Label(props);
    let wrapper = getWrapper(props);
    let controlElement = FraControlElement(props);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(controlElement);
}

const getWrapper: ControlElementCreator = (props) => {
    let wrapper = $('<div>').addClass('col-12 col-sm-9 col-lg-7');
    let inputWrapper = $('<div>').addClass('d-flex align-items-top flex-wrap flex-sm-nowrap');
    let input = getInput(props);
    let inputHidden = getInputHidden(props);

    inputWrapper
        .append(input)
        .append(inputHidden);

    return wrapper.append(inputWrapper);
}

const getInput: ControlElementCreator = (props) => {
    const { item, formProp } = props;

    let classList = getInputClassList(item);
    let wrapper = $('<div class="custom-file mr-2">');
    let label = getInputLabel(props);
    let input = $('<input>')
        .on('focus', function ({ target }) { 
            focusImputFunc({ element: target });
        })
        .on('change', function({ target }) {
            showThumbnail((<HTMLInputElement>target).files, item.Name, (item.DMClassName || ''), (formProp.Alias || ''));
        });

    setAttrIfHasValue(input, {
        accept: (item.Filter || '.jpeg,.jpg,.bmp'), 
        autocomplete: 'off',
        class: classList,
        disabled: item.disabled, 
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        required: item.Required,
        step: 'any', 
        title: item.Hint,
        type: 'file',
        value: item.Valor,
        'data-alias': formProp.Alias,
        'data-autoincrementar': item.AutoIncrementar,
        'data-classname': item.DMClassName,
        'data-dicionario': item.Dicionario,
    });

    return wrapper
        .append(input)
        .append(label);
}

const getInputLabel: ControlElementCreator = ({ item }) => {
    return $('<label>')
        .attr({
            class: 'custom-file-label text-left',
            for: `p.${item.Name}`
        })
        .append('Nenhum Arquivo Selecionado');
}

const getInputClassList = ({ControlClass, Name}: Field): string => {
    return classNames({
        'form-control kt-input custom-file-input cad-control cad-logchange': true,
        [`ctrl${Name}`]: true,
        [`${ControlClass}`]: ControlClass
    })
}

const getInputHidden: ControlElementCreator = ({ item, formProp }) => {
    let input = $('<input>')
        .attr({
            autocomplete: 'off',
            class: `img-thumbnail align-self-start ctrlImg${item.Name}`,
            height: '105',
            id: `ctrlImg${item.Name}`,
            name: `p.Img${item.Name}`,
            src: item.Valor || 'http://placehold.it/136x105',
            type: 'image',
            width: '136',
            'data-alias': formProp.Alias,
            'data-classname': item.DMClassName,
        })
        .on('click', () => false)

    return input;
} 