import classNames from "classnames";
import { ControlElementCreator, Field } from "../types";
import { getFormId, setAttrIfHasValue } from "../util/util";
import { getButtons } from "./buttons";
import { InputResult as getInputResultHidden } from "./input-result";
import { LookupFeedBack as getLookupFeedBack } from "./lookup-feedback";

export const LookupDescriptionSearch: ControlElementCreator<Field, JQuery<HTMLElement> | JQuery<HTMLElement>[]> = (props) => {
    const { formProp } = props;
    let wrapper = getWrapper(props);

    if (!formProp.MultiColuna) {
        let lookupFeedback = getLookupFeedBack(props);
        
        wrapper.append(lookupFeedback);

        return [wrapper, lookupFeedback];
    }

    return wrapper;
}

const getWrapper: ControlElementCreator = (props) => {
    const { formProp } = props;
    let divLookup = getDivContainerLookup(props);
    let buttons = getButtons(props);
    let inputResultHidden = getInputResultHidden(props);
    let inputHidden = getInputHidden(props);

    let wrapper = $('<div>')
        .attr({
            class: `input-group input-addon-default col-md-${formProp.MultiColuna ? 12 : 6}`,
            style: 'padding-left: 0px; padding-right: 0px'
        })
        .append(divLookup)
        .append(buttons)
        .append(inputResultHidden)
        .append(inputHidden)

    return wrapper;
}

const getDivContainerLookup: ControlElementCreator = (props) => {
    const { item, formProp } = props;
    let divInput = $('<div>')
        .attr({
            'data-classname': item.DMClassName,
            'data-alias': formProp.Alias,
            'data-initialvalue': ''
        });

    setAttrIfHasValue(divInput, {
        class: "form-control kt-input",
        'data-dmclassName': item.DMClassName,
        'data-fieldname': item.Name,
        'data-initialValue': item.Valor,
        'data-ismulticolumn': formProp.MultiColuna,
        'data-linkkeyfields': item.LinkKeyFields,
        'data-linklookupdatasetnome': item.LinkLookupDataSetNome,
        'data-linklookupdescfindfield': item.LinkLookupDescFindField,
        'data-linklookupdescshowfields': item.LinkLookupDescShowFields,
        'data-linklookupkeyfields': item.LinkLookupKeyFields,
        'data-linklookupresultfield': item.LinkLookupResultField,
        'data-lookupalphaNumeric': "true",
        'data-lookupid': `ctrl${item.Name}`,
    });

    return divInput;
}

const getInputHidden: ControlElementCreator = ({ item, formProp }) => {
    let classList = getInputHiddenClassList(item);
    let input = $('<input>');

    setAttrIfHasValue(input, {
        class: classList,
        disabled: item.disabled,
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        readonly: item.ReadOnly,
        requried: item.Required,
        style: `text-align: left !important; ${item.TextCase}`,
        title: item.Hint,
        type: 'hidden',
        value: item.Valor,
        'data-alias': formProp.Alias, 
        'data-autoincrementar': item.AutoIncrementar,
        'data-button': `btn${item.Name}`,
        'data-classname': item.DMClassName,
        'data-dicionario': item.Dicionario,
        'data-dmclassname': item.DMClassName,
        'data-haslinklookup': 'true',
        'data-mask': item.mask,
        'data-mask-reverse': item.maskreverse,
        'data-multicolumn': formProp.MultiColuna,
        'data-value': item.Valor,
        'data-lookup': item.ValorLookup, 
    });

    return input;
}

const getInputHiddenClassList = (item: Field): string => {
    return classNames({
        'form-control kt-input cad-control cad-logchange': true,
        'border-radius-left form-control-search': item.SearchStyle,
        [`ctrl${item.Name}`]: true,
        [`${item.ControlClass}`]: item.ControlClass,
    });
}