import classNames from "classnames";
import { DateInput } from "../ntc-edit/date-input";
import { ControlElementCreator, Field } from "../types";
import { getFormId, setAttrIfHasValue } from "../util/util";
import { getButtons } from "./buttons";
import { InputResult as getInputResult } from "./input-result";
import { LookupFeedBack as getLookupFeedBack } from "./lookup-feedback";

declare var FindLookupValue: any;
declare var focusImputFunc: any;

export const LookupValueSearch: ControlElementCreator<Field, JQuery<HTMLElement> | JQuery<HTMLElement>[]> = (props) => {
    const { formProp } = props;
    let wrapper = getWrapper(props);

    if (!formProp.MultiColuna) {
        var lookupFeedback = getLookupFeedBack(props);
        
        return [wrapper, lookupFeedback];
    }

    return wrapper;
}

const getWrapper: ControlElementCreator = (props) => {
    const { formProp } = props;
    let buttons = getButtons(props);
    let input = getInput(props);
    let inputResultHidden = getInputResult(props);

    let wrapper = $('<div>')
        .attr({
            class: `input-group input-addon-default col-md-${formProp.MultiColuna ? 12 : 6}`,
            style: 'padding-left: 0px; padding-right: 0px'
        })
        .append(input)
        .append(inputResultHidden)
        .append(buttons);

    return wrapper;
}

const getInput: ControlElementCreator<Field, JQuery<HTMLElement> | JQuery<HTMLElement>[]> = (props) => {
    const { item } = props;

    if (item.InputType == 'date') return getInputDate(props);
    
    return getInputText(props);
};

const getInputDate: ControlElementCreator<Field, JQuery<HTMLElement>[]> = (props) => {
    const { item, formProp } = props;
    
    const [element, inputHidden] = DateInput(props);
    const input = element.find('input');

    input
        .addClass('rounded-0')
        .data('haslinklookup', true)
        .on('blur', () => FindLookupValue(item.Name, item.DMClassName, formProp.Alias))
    
    inputHidden.on('change', () => FindLookupValue(item.Name, item.DMClassName, formProp.Alias));

    setAttrIfHasValue(inputHidden, {
        'data-AutoIncrementar': item.AutoIncrementar,
        'data-button': `btn${item.Name}`,
        'data-dmclassName': item.DMClassName,
        'data-haslinklookup': "true" ,
        'data-lookup': item.ValorLookup,
        'data-multicolumn': formProp.MultiColuna,
        'data-value': item.Valor,
    });;

    return [element, inputHidden];
};

const getInputText: ControlElementCreator = ({ item, formProp }) => {
    let classList = getInputClassList(item);
    let input = $('<input>')
        .on('blur', function () { FindLookupValue(item.Name, item.DMClassName, formProp.Alias) })
        .on('focus', function ({ target }) { focusImputFunc({ element: target, multiColumn: formProp.MultiColuna }) })

    setAttrIfHasValue(input, {
        autocomplete: 'off', 
        class: classList,
        disabled: item.disabled,
        form: getFormId(formProp),
        id: `ctrl${item.Name}`, 
        name: `p.${item.Name}`,
        readonly: item.ReadOnly,
        required: item.Required,
        style: `text-align: left !important; ${item.TextCase}`,
        title: item.Hint, 
        type: item.InputType,  
        value: item.Valor,
        'data-autoincrementar': item.AutoIncrementar, 
        'data-alias': formProp.Alias,
        'data-button': `btn${item.Name}`,
        'data-classname': item.DMClassName,
        'data-dicionario': item.Dicionario,
        'data-dmclassName': item.DMClassName,
        'data-haslinklookup': "true", 
        'data-lookup': item.ValorLookup,
        'data-mask': item.mask,
        'data-mask-reverse': item.maskreverse, 
        'data-multicolumn': formProp.MultiColuna,
        'data-value': item.Valor,
    });

    return input;
}

const getInputClassList = (item: Field): string => {
    return classNames({
        'form-control kt-input cad-control cad-logchange': true,
        'border-radius-left form-control-search': item.SearchStyle,
        [`ctrl${item.Name}`]: true,
        [`${item.ControlClass}`]: item.ControlClass,
    });
}