import classNames from "classnames";
import { FraControlElement } from "../fra-control-element/fra-control-element";
import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { getButtonWrapper } from "../ntc-link/ntc-link";
import { NtcMemo } from "../ntc-memo/ntc-memo";
import { ControlElementCreator, Field } from "../types";
import { getFormId, setAttrIfHasValue } from "../util/util";

declare var focusImputFunc: any;

/**
 * - criar controle
 * - criar controle no FraControl.partial
 * - criar o criador do controle
 * - alterar a atribuição de valor nos métodos de lookup
 */

export const NtcMemoAnnotation: ControlElementCreator = (props) => {
    /**
     * The return of modal search (appending values to the existing) is managed on the PreencherKeyFields method through control type validation
     */
    
    const inputBlock = InputBlock(props);
    const label = Label(props);
    const wrapper = getWrapper(props);
    const controlElement = FraControlElement(props);

    inputBlock.append([label, wrapper, controlElement]);
    
[NtcMemo];

    return inputBlock
    //  return NtcMemo(props);
}

const getWrapper: ControlElementCreator = (props) => {
    const { formProp } = props;
    
    let wrapper = $('<div>').addClass(classNames({
        'input-group': formProp.MultiColuna,
        'col-12 col-sm-9 col-lg-7': !formProp.MultiColuna
    }));

    let textArea = getTextArea(props);
    let buttonGroup = getButtonWrapper(props);

    buttonGroup.css('max-height', '36px');

    if (formProp.MultiColuna) {
        wrapper.append(textArea);
        wrapper.append(buttonGroup);

    }
    else {
        let divGroup = $('<div class="input-group">');

        wrapper.append(divGroup);
        divGroup.append(textArea);
        divGroup.append(buttonGroup);
    }

    return wrapper;
}

const getTextArea: ControlElementCreator = ({ formProp, item }) => {
    const classList = getClassList(item);
    const textArea = $('<textarea>')
        .val(item.Valor || '')
        .on('focus', ({ target }) => focusImputFunc({ element: target }))

        ;[formProp];

    setAttrIfHasValue(textArea, {
        class: classList,
        disabled: item.disabled,
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        maxlength: item.DisplayWidth || null,
        name: `p.${item.Name}`,
        required: item.Required,
        rows: 10,
        style: item.TextCase,
        'data-autoincrementar': item.AutoIncrementar,
        'data-dicionario': item.Dicionario,
    });

    return textArea;
}

const getClassList = ({ ControlClass, Name}: Field) => classNames({
    'form-control kt-input cad-control cad-logchange': true,
    [`ctrl${Name}`]: true,
    [`${ControlClass || ''}`]: ControlClass,
});

