import { ControlElementCreator, Field } from "../types";
import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';
import { FraControlElement } from "../fra-control-element/fra-control-element";

declare var focusImputFunc: any;

export const NtcMemo: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = Label(props);
    let wrapper = getWrapper(props);
    let controlElement = FraControlElement(props);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(controlElement);
}

const getWrapper: ControlElementCreator = (props) => {
    let wrapper = $('<div>').addClass('col-12 col-sm-9 col-lg-7');
    let textArea = getTextArea(props);
    
    return wrapper.append(textArea)
}

const getTextArea: ControlElementCreator = ({ item, formProp }) => {
    let classList = getClassList(item);
    let textArea = $('<textarea>')
        .val(item.Valor || '')
        .on('focus', ({ target }) => focusImputFunc({ element: target })) 
    
    setAttrIfHasValue(textArea, {
        class: classList, 
        disabled: item.disabled, 
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        maxlength: (item.DisplayWidth || null),
        name: `p.${item.Name}`, 
        required: item.Required,
        rows: 10, 
        style: item.TextCase, 
        title: item.Hint, 
        'data-autoincrementar': item.AutoIncrementar,
        'data-dicionario': item.Dicionario, 
    });

    return textArea;
}

const getClassList = ({ ControlClass, Name }: Field) => {
    return classNames({
        'form-control kt-input cad-control cad-logchange': true,
        [`ctrl${Name}`]: true,
        [`${ControlClass}`]: ControlClass,
    })
}