import { GroupColumn } from '../types';

interface Attribute {
    [key: string]: any;
}

const isEmpty = (value: any) => ['', null, undefined].includes(value);

export function setAttrIfHasValue(el: JQuery<HTMLElement>, attr: Attribute) {
    const attribute = Object
        .entries(attr)
        .reduce((acc, [key, value]) => {
            if (!isEmpty(value)) {
                acc[key] = value;
            }

            return acc;
        }, {} as Attribute);

    el.attr(attribute);
}

export function setDataIfHasValue(el: JQuery<HTMLElement>, data: Attribute) {
    const dataAttribute = Object
        .entries(data)
        .reduce((acc, [key, value]) => {
            if (!isEmpty(value)) {
                acc[key] = value;
            }

            return acc;
        }, {} as Attribute);

    el.data(dataAttribute);
}

export function isGroupColumn(item: any): item is GroupColumn {
    return item.GrupoColunas ?? false;
}

export async function whenElementOnPage(element: HTMLElement): Promise<HTMLElement> {
    return new Promise((resolve) => {
        new MutationObserver((_, observer) => {
            if (document.contains(element)) {
                resolve(element);

                observer.disconnect();
            }
        }).observe(document.documentElement, {
            childList: true,
        });
    });
}

export function getFormId({ Class = '', Alias = '' }: { Class: string, Alias?: string }): string {
    return `form_${Class}_${Alias}`;
}