import dxDataGrid from 'devextreme/ui/data_grid';
import dxButton from 'devextreme/ui/button';
import { BehaviorSubject } from 'rxjs';
import { getToolbarItemConfig, CustomButtonConfig } from './toolbar-item-custom-config';
import { ToolButton } from '../fra-control/types';

interface AfterSetGridItem {
    instance: dxDataGrid;
    name: string;
}

interface SetActionProps {
    gridName: string,
    actionName: string,
    dxButtonInstance: dxButton
}

export interface CustomGridConfig {
    toolbarItems?: CustomButtonConfig;
}

interface SetCustomGridConfigProps {
    gridName: string;
    config: CustomGridConfig;
}

class GridManager {
    private grids = new Map<string, dxDataGrid | Array<dxDataGrid> >();
    private _actions = new Map<string, Map<string, dxButton> >();
    private _customGridConfig = new Map<string, CustomGridConfig>();

    public afterSetGrid = new BehaviorSubject<AfterSetGridItem | null>(null);

    get(name: string) {
        const grid = this.grids.get(name);
        
        return grid instanceof Array ? grid[0] : grid;
    }

    set(name: string, instance: dxDataGrid) {    
        this.grids.set(name, instance);

        this.afterSetGrid.next({
            instance,
            name,
        });
    }

    setCustomConfig({ gridName, config }: SetCustomGridConfigProps) {
        const previousConfig = this._customGridConfig.get(gridName)

        this._customGridConfig.set(gridName, {
            ...previousConfig,
            ...config,
        });
    }

    getCustomConfig(gridName: string) {
        return this._customGridConfig.get(gridName);
    }

    getCustomToolbarItemConfig(gridName: string, toolbutton: ToolButton) {
        const config = this._customGridConfig.get(gridName);

        if (!config) return {};

        return getToolbarItemConfig({
            toolbutton,
            ...config,
        });
    }
    
    setAction({ gridName, actionName, dxButtonInstance }: SetActionProps) {
        if (this._actions.has(gridName)) {
            this._actions.get(gridName)!.set(actionName, dxButtonInstance);
        }
        else {
            this._actions.set(gridName, new Map([[actionName, dxButtonInstance]]));
        }
    }

    getAction(gridName: string, actionName: string) {
        const actionList = this._actions.get(gridName);
        const action = actionList?.get(actionName);

        if (action) return action;

        const grid = this.get(gridName);

        if (!grid) return;

        this._forceDropdownButtonRender(grid);

        return this._actions.get(gridName)?.get(actionName);
    }

    
    _forceDropdownButtonRender(grid: dxDataGrid) {
        const dropdown = grid
            .element()
            .find('.dx-dropdownmenu')
            .get(0)

        dropdown?.click();
        dropdown?.click();
    }

    resetActions(gridName: string) {
        this._actions.get(gridName)?.clear();
    }
}

export default (new GridManager());