import { ToolButton } from "../fra-control/types";
import { CustomGridConfig } from "./grid-manager";

export interface ButtonProps {
    disabled?: boolean;
    text?: string;
}

interface GetToolbarItemConfigProps extends CustomGridConfig {
    toolbutton: ToolButton;
}

export interface CustomButtonConfig {
    disabled?: Function | boolean;
    text?: Function | string;
} 

export const getToolbarItemConfig = (props: GetToolbarItemConfigProps): ButtonProps => {
    if (!props.toolbarItems) return {};

    return {
        ...getDisabled(props),
        ...getText(props),
    };
}

type PropertyValueGetter = (props: GetToolbarItemConfigProps) => ButtonProps

const getDisabled: PropertyValueGetter = ({ toolbutton, toolbarItems }) => getProp({
    propName: 'disabled',
    toolbutton,
    toolbarItems,
});

const getText: PropertyValueGetter = ({ toolbutton, toolbarItems }) => getProp({
    propName: 'text',
    toolbutton,
    toolbarItems,
});

interface GetPropertyProps extends GetToolbarItemConfigProps {
    propName: 'text' | 'disabled';
}

const getProp = ({ toolbarItems, toolbutton, propName }: GetPropertyProps): ButtonProps => {
    const property = toolbarItems![propName];
    
    if (typeof property == undefined) return {};

    return {
        [propName]: typeof property == 'function' ? property(toolbutton) : property,
    };
};