import { ToolButton } from "../../fra-control/types";
import { DefinicaoObjetosConfigForToolButtons, ScopeConfigForToolButtons } from "../toobuttons";
import { getToolButtonAction } from "../toolbutton-action/toolbutton-action";

declare var actCarregarXMLfield: any;

interface CustomButtonProps {
    DefinicaoObjetos: DefinicaoObjetosConfigForToolButtons;
    toolButton: ToolButton;
    Params?: {
        [key: string]: string;
    };
};

export const renderCustomButton = ({ DefinicaoObjetos, Params }: ScopeConfigForToolButtons/*ScopeConfig*/) => {
    const { ToolButtons } = DefinicaoObjetos;

    return ToolButtons
        .map((toolButton, index) => {
            if (!toolButton.tbsButton || !toolButton.ExibirToolBar || !toolButton.Custom) return;

            if (toolButton.NameEvent === 'actCarregar_XML') {
                return loadXmlButton(index)
            }

            if (!['nActMoverRegAcimaExecute', 'nActMoverRegAbaixoExecute'].includes(toolButton.NameEvent)) {
                return CustomButton({ DefinicaoObjetos, toolButton, Params });
            }

        })
        .filter(button => button);
}


const loadXmlButton = (index: number) => {
    return $('<input>')
        .attr({
            accept: ".xml",
            id: `actCarregarXML${index}`,
            type: "file",
            style: "display:none",
        })
        .css({ display: 'none' })
        .on('change', function () { actCarregarXMLfield(this) })
}

export function actCarregar_XML() {
    $("#actCarregarXML{{-index}}").click();
}

const CustomButton = ({ DefinicaoObjetos, toolButton, Params }: CustomButtonProps) => {
    const { Alias = '', Class = '', CadCons = '', TopicoAjuda = '' } = DefinicaoObjetos;
    const { icon, Caption, Name } = toolButton;

    const text = $('<span>')
        .attr('class', 'h4 font-weight-normal text-uppercase')
        .append(Caption);

    const button = $('<button>')
        .attr({
            class: `btn btn-outline-dark btn-hover-transparent-dark py-2 ${Class}`,
            type: 'button',
            'data-alias': Alias,
            'data-cadcons': CadCons,
            'data-topicoajuda': TopicoAjuda,
            'data-action-name': Name,
        })
        .on('click', getToolButtonAction({ DefinicaoObjetos, toolButton, Params }));

    if (icon) {
        const img = $(`<img src="${icon}" class="icon-md">`);

        button.append(img);
    }

    return button.append(text);
}