import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";
import { getCreateActionData } from "../util/action-data";

interface NavigationButtonProps {
    DefinicaoObjetos: Pick<DefinicaoObjetosConfig, 'Class' | 'Alias' | 'CadCons' | 'TopicoAjuda'>;
}

export const NavigationButton = ({ DefinicaoObjetos: { Class = '', Alias = '', CadCons = '', TopicoAjuda = '' } }: NavigationButtonProps) => {
    const config = {
        next: {
            iconClass: 'la-actDataSetNext',
            text: 'Próximo',
            action: 'actDataSetNextExecute'
        },
        previous: {
            iconClass: 'la-actDataSetPrior',
            text: 'Anterior',
            action: 'actDataSetPriorExecute'
        }
    };

    const btn = (type: 'next' | 'previous') => {
        const btnConfig = config[type];
        const span = $('<span class="d-none d-sm-inline">').append(btnConfig.text);
        const icon = $(`<i class="la kt-nav__link-icon la ${btnConfig.iconClass}"></i>`);
        const li = $(`<li>`)
            .attr({
                class: 'kt-portlet__nav-item',
                'data-class': Class,
                'data-alias': Alias,
                'data-cadcons': CadCons,
                'data-topicoajuda': TopicoAjuda,
            })
            .css('display', 'inline');

        const a = $('<a>')
            .attr({
                id: `${btnConfig.action}${Class}`,
                class: 'btn btn-text-dark btn-hover-transparent-dark py-2 ml-2',
                'data-n-onclick': btnConfig.action,
                ...getCreateActionData(Class, Alias, CadCons, TopicoAjuda)
            });

        a.append(type == 'next' ? [span, icon] : [icon, span]);

        return li.append(a);
    }

    return [
        btn('previous'),
        btn('next'),
    ];
}