import FormManager from "../../../../js/FormManager";
import SessionManager from "../../../../js/SessionManager";
import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";

declare var KTUtil: any;

interface SplitReportParamFields {
    [key: string]: {
        element: JQuery<HTMLElement>;
        value: boolean;
    };
}

class SplitReportParam {
    private fields: SplitReportParamFields;

    constructor({ Fields, buttonEl }: Pick<DefinicaoObjetosConfig, 'Fields'> & { buttonEl: JQuery<HTMLElement> }) {
        this.fields = Fields.reduce((fields, field) => {
            return {
                ...fields,
                [field.Name]: {
                    element: $(`input[data-fieldname=${field.Name}`, buttonEl.find('ul')),
                    value: false,
                }
            };
        }, {} as SplitReportParamFields);
    }

    setValue(name: string | { [key: string]: boolean }, value?: boolean) {
        if (typeof name === 'string') {
            this.fields[name].value = Boolean(value);
        }
        else {
            Object
                .entries(name)
                .forEach(([fieldName, value]) => {
                    const field = (this.fields[fieldName] || this.fields[`P${fieldName}`]);

                    if (!field) return;

                    field.value = value;
                    field.element.prop('checked', value);
                });
        }
    }

    getValues() {
        return Object
            .entries(this.fields)
            .filter(([_, { value }]) => value)
            .map(([fieldName]) => fieldName)
            .join(';');
    }
}

export const createOnFooterToolbutton = ({ el, DefinicaoObjetos }: { el: JQuery<HTMLScriptElement> | HTMLScriptElement, DefinicaoObjetos: DefinicaoObjetosConfig }) => {
    const checkbox = createCheckbox(DefinicaoObjetos);

    $(el).replaceWith(checkbox);
};

export const createOnToolBar = ({ el, DefinicaoObjetos }: { el: JQuery<HTMLScriptElement> | HTMLScriptElement, DefinicaoObjetos: DefinicaoObjetosConfig }) => {
    const checkbox = createDropdown(DefinicaoObjetos);

    $(el).replaceWith(checkbox);
};

const createDropdown = ({ Fields, Class, Alias = '' }: DefinicaoObjetosConfig) => {
    const splitFields = Fields.filter(({ ntcLink, ntcLinkAnnotation, ntcLinkEdit }) => ntcLink || ntcLinkAnnotation || ntcLinkEdit);
    const buttonEl = $(`
        <li class="kt-portlet__nav-item"  style="display: inline">        
            <div class="dropdown dropdown-inline">        
                <button type="button" class="btn btn-sm btn-text-dark btn-hover-transparent-dark  kt-btn py-2 ml-2" data-toggle="dropdown">
                    <i class="la la-object-ungroup"></i>
                </button>
                <div class="dropdown-menu dropdown-menu-right" style="width: 300px;">
                    <div class="p-4" style="font-size: 1.15rem;">Separar por: </div>
                    <ul class="kt-nav pb-8 pt-2 px-4 mb-0 keep-open list-unstyled">
                        ${splitFields
                            .filter(({ ntcLink, ntcLinkAnnotation, ntcLinkEdit }) => ntcLink || ntcLinkAnnotation || ntcLinkEdit)
                            .map(({ DisplayLabel, Name }) => `
                                <li class="kt-nav__item">
                                    <a class="kt-nav__link">
                                        <div class="checkbox-wrapper">
                                            <label class="checkbox checkbox-outline checkbox-dark mb-2">
                                                <input type="checkbox" data-fieldname="${Name}">
                                                <span></span>
                                                <div class="ml-2 mr-5">${DisplayLabel}</div>
                                            </label>
                                        </div>
                                    </a>
                                </li>
                            `)
                            .join('')
                        }
                    </ul>				
                </div>
            </div>
        </li>
    `);

    const splitReportParams = new SplitReportParam({ Fields: splitFields, buttonEl: buttonEl })

    $(() => {
        KTUtil.scrollInit(buttonEl.find('ul').get(0), { height: 300 })
    })

    buttonEl
        .find('ul')
        .on('click.bs.dropdown.data-api', (e) => e.stopPropagation())
        .on('click', 'input', (event) => {
            const { currentTarget } = event;
            const { fieldname } = $(currentTarget).data();

            splitReportParams.setValue(fieldname, $(currentTarget).prop('checked'));
        });

    FormManager.afterCreateAllForms.subscribe(() => {
        const form = FormManager.byname(Class, Alias);

        form.splitReportParam = splitReportParams;
    });

    return buttonEl;
};

const createCheckbox = ({ Class, Alias }: DefinicaoObjetosConfig) => {
    return $(`
            <label class="checkbox checkbox-outline checkbox-dark">
                <input autocomplete="off" type="checkbox" title="Separar Arquivos para Publicação"> 
                <span></span>
                <div class="ml-5">Separar Arquivos para Publicação</div>
            </label>
        `)
        .on('click', 'input', ({ currentTarget }) => {
            const value = $(currentTarget).prop('checked');
            const form = FormManager.byname(Class, Alias);
            const fields = [
                'CodigoEmpresa',
                'CodigoEstab',
                ...(SessionManager.moduloAtual === 'npmnFpa'
                    ? ['CodigoFuncContr']
                    : []
                ),
            ];

            form.splitReportParam.setValue(fields.reduce((fields, field) => ({
                ...fields,
                [field.toUpperCase()]: value,
            }), {}));
        });
};
