import { DefinicaoObjetosConfig, ScopeConfig } from "../../types/page-config/page-config";
import { renderCustomButton } from "./custom-button/custom-button";
import { DropdownButton } from "./dropdown-button/dropdown-button";
import { ExecuteButton } from "./execute-button/execute-button";
import { FieldChooserButton } from "./field-chooser-button/field-chooser-button";
import { FullscreenButton } from "./fullscreen-button/fullscreen-button";
import { NavigationButton } from "./navigation-button/navigation-button";
import { SaveButton } from "./save-button/save-button";

type ToolButtonType =
    | 'execute'
    | 'fieldchooser'
    | 'fullscreen'
    | 'navigation'
    | 'save'
    | 'dropdown';

const canRender = (type: ToolButtonType, { DefinicaoObjetos, EhModal }: ScopeConfigForToolButtons/*ScopeConfig*/) => {
    const { TipoObjeto, ProcessoPadrao, ToolButtons, ProcessoMultiplo, WorkSpace } = DefinicaoObjetos;
    const isReportView = location.pathname.includes('RelatorioExibir');

    switch (type) {
        case "fullscreen": return !isReportView && !EhModal && (ProcessoPadrao || (TipoObjeto === 'ntoRelatorio' || TipoObjeto == 'ntoCadastro'));
        case "navigation": return TipoObjeto === 'ntoCadastro';
        case 'save': return TipoObjeto === 'ntoCadastro' && ToolButtons.find(({ Name }) => Name === 'actDataSetPost');
        case 'execute': return !isReportView && !ProcessoMultiplo && !(TipoObjeto === 'ntoCadastro') ;
        case 'fieldchooser': return TipoObjeto === 'ntoCadastro';
        case 'dropdown': return !WorkSpace;
    }
}
export interface ScopeConfigForToolButtons extends Pick<ScopeConfig, 'Class' | 'Params' | 'EhModal'>{
    DefinicaoObjetos: DefinicaoObjetosConfigForToolButtons;
}

export interface DefinicaoObjetosConfigForToolButtons extends Pick<DefinicaoObjetosConfig, 
    | 'TipoObjeto'
    | 'ProcessoPadrao'
    | 'ProcessoMultiplo'
    | 'WorkSpace' 
    | 'ToolButtons'
    | 'Alias'
    | 'Class'
    | 'CadCons'
    | 'TopicoAjuda'
    | 'Pages'
    | 'FormDetalhe'
    | 'Fields'> {}

export const createToolButtons = (scope: ScopeConfigForToolButtons): JQuery<HTMLElement>[] => {
    const elements = [];

    if (canRender('fullscreen', scope)) elements.push(FullscreenButton());

    elements.push(...renderCustomButton(scope));

    if (canRender('navigation', scope)) elements.push(...NavigationButton(scope));

    if (canRender('save', scope)) elements.push(SaveButton(scope));

    if (canRender('execute', scope)) elements.push(ExecuteButton(scope));

    if (canRender('fieldchooser', scope)) elements.push(FieldChooserButton(scope));

    if (canRender('dropdown', scope)) elements.push(DropdownButton(scope));

    return elements as JQuery<HTMLElement>[];
}

export const createFooterToolbuttons = (scope: ScopeConfigForToolButtons) => {
    const toolbar = $('<div class="footer bg-white py-1 d-flex flex-lg-column">');
    const container = $('<div class="container-fluid d-flex flex-column flex-md-row align-items-center justify-content-between">');
    const toolsRight = $('<div class="text-dark ml-auto">');
    const toolButtons = createToolButtons(scope);

    toolsRight.append(toolButtons);
    container.append(toolsRight);

    return toolbar.append(container);
}

interface PageToolButtonsProp {
    uuid?: string;
    scope: ScopeConfigForToolButtons;
}

export const createPageToolbuttons = ({ uuid = '', scope }: PageToolButtonsProp) => {
    const wrapp = $(`<div id="toolbuttons_tab_${uuid}"  style="display: none">`);
    const toolButtons = createToolButtons(scope);

    return wrapp.append(toolButtons);
}