import { TwCxGridConfig, WidgetPropsBase } from "../types";
import dxDataGrid from 'devextreme/ui/data_grid';
import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";
import { ProcessGrid } from "../../../../js/DynamicImports/Grid/ProcessGrid";
import { buildForm } from "../tw-field-list/tw-field-list";
import FormManager from "../../../../js/FormManager";
import gridManager from "../../grid-manager/grid-manager";
import { prepareMasterDetailFilter } from '../../../../js/gridConfigurator';

interface TwCxGridProps extends Omit<WidgetPropsBase, 'card' | 'config'> {
    widget: TwCxGridConfig;
    DefinicaoObjetos: DefinicaoObjetosConfig,
    Params?: {
        [key: string]: any;
    };
}

interface DetailConfigProcess {
    detailKeyFields: string;
    masterKeyFields: string;
    nameGridDetail: string;
    outsideGrid: boolean;
}

export const TwCxGrid = (props: TwCxGridProps) => {
    const { widget: { Grid } } = props;

    // props.DefinicaoObjetos.ntfCadGrid = true;

    return Grid ? buildSingleGrid(props) : buildTabbedGrids(props);
};

const buildSingleGrid = ({ widget, DefinicaoObjetos, Params }: TwCxGridProps) => {
    const parseGrid = (grid: string) => eval(`(${grid})`);
    const { Name } = widget;
    
    const { Grid, Text } = widget;

    if (!Grid) throw new Error('Grid non existent');
    
    const card = $(`<div class="card card-custom mb-0 mt-4" id="widget_${Name}">`);
    const gridConfig = typeof Grid === 'string' ? parseGrid(Grid) : Grid;

    if (!gridConfig) {
        console.error('Invalid grid configuration');
        return $();
    }
    
    if (gridConfig.nomegrid === 'nClientDataSet' && DefinicaoObjetos.ntfCadGrid) gridConfig.nomegrid = `${DefinicaoObjetos.Class}${DefinicaoObjetos.Alias}`;

    if (Text) card.append(getCardHeader(Text));

    card.append(`
        <div class="card-body">
            <div id="ConteudoGrid_wrapper">						
                <div id="gridTools"></div>	
                <div id="dataGridConteudo${gridConfig.nomegrid || ''}" data-grid="true" ${DefinicaoObjetos.ntfCadGrid ? '' : 'class="gridConteudoAdicional"'}></div>
            </div>
        </div>
    `);

    if (DefinicaoObjetos.ntfCadGrid) {
        const formEl  = buildForm({
            config: {
                DefinicaoObjetos,
            },
            fields: [],
        });

        card.find('.card-body').append(formEl);
    }

    const gridInstance: dxDataGrid = ProcessGrid.buildGrid({
        element: card.find(`#dataGridConteudo${gridConfig.nomegrid || ''}`).get(0),
        definicaoObjetos: DefinicaoObjetos,
        grid: gridConfig,
        params: Params,
    });

    if (DefinicaoObjetos.ntfCadGrid) handleSelectionMasterDetailRegister(gridInstance, DefinicaoObjetos);

    if (gridConfig?.Detail?.outsideGrid) handleSelectionMasterDetailProcess(gridInstance, gridConfig);

    return card;
};

const getCardHeader = (Text: string) => $(
    `<div class="card-header">
        <h3 class="card-title">
            ${Text}	
        </h3>
    </div>`
);

const buildTabbedGrids = ({ widget, DefinicaoObjetos, Params }: TwCxGridProps) => {
    const setOnTabChangeRepaintGrid = (tabLinkWrapper: JQuery<HTMLElement>) => {
        $('a', tabLinkWrapper).on('shown.bs.tab', ({ currentTarget }) => { // repaint grid on tab change
            const gridName = (currentTarget as HTMLLinkElement).href.match(/ConteudoGrid(\S+)_wrapper/)?.[1];
            const gridInstance = (dxDataGrid.getInstance($(`#dataGridConteudo${gridName || ''}`)) as dxDataGrid);

            gridInstance.repaint();
            gridInstance.refresh();
        });
    }

    const { grids, Name } = widget; 

    if (!grids) throw new Error('grids non existent');

    const card = $(`<div id="widget_${Name}" class="card card-custom"><div class="card-body"></div></div>`);
    const tabContent = $(`<div class="tab-content mt-8">`);
    const tabLinks = $(`
        <ul class="nav-tabs-line nav-tabs-line-3x nav-tabs-line-active-border-primary mt-1 nav nav-bold nav-tabs q-nav-tabs" role="tablist">
            ${
                grids!.map(({ nomegrid, Caption }, index) => (
                        `<li class="nav-item ${!index ? 'active' : ''}" data-toggle="tooltip" title="" data-original-title="">
                            <a 
                                class="nav-link px-8 m-0 bg-transparent ${!index ? 'active show' : ''}"
                                href="#ConteudoGrid${nomegrid}_wrapper"  
                                aria-controls="ConteudoGrid${nomegrid}_wrapper" 
                                role="tab" 
                                data-toggle="tab"
                                aria-expanded="false"
                            >${Caption}</a>
                        </li>`
                    ))
                    .join('')
            }
        </ul>
    `);

    setOnTabChangeRepaintGrid(tabLinks);

    grids!.map((gridConfig, index) => {
        const { nomegrid } = gridConfig;
        const { Class } = DefinicaoObjetos;
        const tab = $(`
            <div class="tab-pane ${!index ? 'active' : ''}" id="ConteudoGrid${nomegrid}_wrapper" role="tabpanel">
                <div 
                    data-class-nomegrid="${Class}${nomegrid}"
                    id="dataGridConteudo${nomegrid}"
                    class="gridConteudoAdicional"
                ></div>
            </div>	
        `);

        tabContent.append(tab);

        ProcessGrid.buildGrid({
            element: tab.find('[id*="dataGridConteudo"]'),
            definicaoObjetos: DefinicaoObjetos,
            grid: gridConfig,
            params: Params,
        })
    })
    
    card.find('.card-body').append([tabLinks, tabContent]);
    
    return card;
};

const handleSelectionMasterDetailRegister = (gridInstance: dxDataGrid, DefinicaoObjetos: DefinicaoObjetosConfig) => {
    const form = FormManager.byname(DefinicaoObjetos.Class, DefinicaoObjetos.Alias);

    if (form?.masterclass) {
        const masterForm = FormManager.byname(form.masterclass, '');

        (masterForm?.grid as dxDataGrid)?.on('selectionChanged', () => {
            gridInstance.refresh();
        });
    }
};

const handleSelectionMasterDetailProcess = (gridInstance: dxDataGrid, gridConfig: any) => {
    const Detail: DetailConfigProcess = gridConfig.Detail;

    gridManager.afterSetGrid.subscribe((event) => {
        if (!event) return;

        const { instance, name } = event;

        if(name === Detail.nameGridDetail) {
            const resetFilter = () => {
                instance.filter(['resetFilter', '=', '-1']);
            
                instance.off('contentReady', resetFilter);
            };
            
            instance.on('contentReady', resetFilter)
        }
    });

    gridInstance.on('selectionChanged', ({ selectedRowsData }: {
        component?: dxDataGrid;
        element?: Pick<dxDataGrid, 'element'>;
        model?: any;
        currentSelectedRowKeys?: Array<any>;
        currentDeselectedRowKeys?: Array<any>;
        selectedRowKeys?: Array<any>;
        selectedRowsData?: Array<any>;
    }) => {
        if (!selectedRowsData?.length) return;

        const gridDetail = gridManager.get(Detail.nameGridDetail);

        if (gridDetail instanceof Array) throw new Error('Instance is an array');

        gridDetail?.filter(prepareMasterDetailFilter(Detail.detailKeyFields.split(';'), Detail.masterKeyFields.split(';'), selectedRowsData[0], gridInstance));
    });
};
