import classNames from "classnames";
import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";
import { getToolButtonAction } from "../../toolbuttons/toolbutton-action/toolbutton-action";
import { TwLabelValueConfig, WidgetPropsBase } from "../types";

interface TwLabelValueProps extends Omit<WidgetPropsBase, 'card' | 'config'> {
    widget: TwLabelValueConfig;
    DefinicaoObjetos: DefinicaoObjetosConfig,
}

export const TwLabelValue = ({ widget, DefinicaoObjetos }: TwLabelValueProps) => {
    const { colunas, Title: { Text, Align, Alignment, Font }, Value, Name, Icon, ShowShadow, Symbol } = widget;
    const getIcon = () => {
        if (Icon) {
            return $(`<img>`)
                .attr({
                    class: 'kt-nav__link-icon twLabelValue-img',
                    id: `widget_link_${Name}`,
                    src: Icon.Picture,
                })
            .on('click', getToolButtonAction({ DefinicaoObjetos, toolButton: Icon.Action }));
        }

        return $();
    };

    const labelValueContent = $(`<div style="width: inherit">`);
    const fakeControl = $(`<span class="fake-control hide-scrollbar text-right ${ShowShadow ? 'input-box-shadow' : ''}">`);
    const spanSymbol = $(`<span class="twLabelValue-symbol" style="${Symbol.Font.Style.fsBold ? 'font-weight: bold;' : ''}">${Symbol.Text}</span>`)
    const spanText = $(`<span id="widget_text_${Name}" class="twLabelValue-symbol" style="float: right;">${Value.Text}</span>`);
    const icon = getIcon();

    labelValueContent.append(fakeControl);
    fakeControl.append([spanSymbol, icon, spanText]);

    if (Align === 'alTop') {
        const wrapper = $(`<div id="widget_${Name}" class="input-block form-group twLabelValue ${colunas ? `col-md-${colunas}` : ''}">`);
        const label = $(`<label>${Text}</label>`).attr({
            class: classNames({
                'text-left': Alignment === 'taLeft',
                'text-center': Alignment === 'taCenter',
                'text-right': Alignment === 'taRight',
            }),
            style: classNames({
                'width: inherit;': true,
                'font-weight: bold;': Font.Style.fsBold,
                'text-align: justify;': Alignment === 'taLeftJustify',
            })
        });

        return wrapper.append([label, labelValueContent]);
    }
    else if (Align === 'alLeft') {
        const wrapper = $(`<div id="widget_${Name}" class="twLabelValue ${colunas ? `col-md-${colunas}` : ''}">`);
        const inputBLock = $(`<div class="input-block form-group" style="display: flex;padding: 5px 0;float: right;width: 65%;align-items: center;">`)
        const label = $(`<label>${Text}</label>`).attr({
            class: classNames({
                'col-form-label': true,
                'text-left': Alignment === 'taLeft',
                'text-center': Alignment === 'taCenter',
                'text-right': Alignment === 'taRight',
            }),
            style: classNames({
                'margin-right: 1rem;': true,
                'font-weight: bold;': Font.Style.fsBold,
            }),
        });

        inputBLock.append([label, labelValueContent]);

        return wrapper.append(inputBLock);
    }
    else {
        throw new Error('Align value isn\'t supported');
    }
}