import { Field, GroupColumn } from "../../fra-control/types";
import { ExecuteButton } from "../../toolbuttons/execute-button/execute-button";
import { createFooterToolbuttons, createToolButtons } from "../../toolbuttons/toobuttons";
import { Card, CardBuilder } from "../layout/card";
import { FloatingMenuItems } from "../layout/floating-menu";
import { LayoutScopeConfig, LayoutOptions } from "../layout/layout";
import { buildForm } from "../tw-field-list/tw-field-list";
import { TwParamlistConfig } from "../types";


interface TwParamListProps {
    card: Card;
    config: LayoutScopeConfig;
    widget: TwParamlistConfig;
    floatingMenuItems?: FloatingMenuItems;
    options?: LayoutOptions;
}

export const TwParamList = ({ card, config, widget, floatingMenuItems, options = {} }: TwParamListProps) => {
    if (!config.fields) throw new Error('No existing fields');

    const { fields, DefinicaoObjetos} = config;
    const { Caption, TipoObjeto } = DefinicaoObjetos;
    const { isFrom = {}, isWorkSpace, isWizard } = options;
    const { isMaster, FieldList } = widget;
    const fieldsConfig = (FieldList || [])
        .map((fieldName) => fieldName.toUpperCase())
        .filter((fieldName) => fields.has(fieldName))
        .map((fieldName) => fields.get(fieldName)) as (Field | GroupColumn)[];

    const content = buildForm({
        config,
        fields: fieldsConfig,
        formAttr: {
            action: TipoObjeto === 'ntoRelatorio' ? 'RelatorioImprimir' : 'ProcessoExecutar',
        }
    });

    if (isMaster) {
        const executeButton = getExecuteButton({ DefinicaoObjetos });

        if (isFrom.twMasterDetail || isWizard && false) CardBuilder.setToolbar(card, createToolButtons(config));
        else if (isWorkSpace || isWizard) {
            card.cardHeader.addClass('hidden');
            card.cardElement.css('box-shadow', 'none');
        }
        else card.cardElement.append(createFooterToolbuttons(config));
        
        CardBuilder.setTitle(card, Caption);

        floatingMenuItems?.push({
            el: card.cardElement,
            caption: Caption,
        });

        if (content instanceof Array) {
            return [...content, executeButton, buildResponseContent()];
        }

        return [content,  executeButton, buildResponseContent()];
    }

    return content;
};

const buildResponseContent = () => $(`
    <div id="resultados">
        <div id="Conteudo"></div>
    </div>
`);

const getExecuteButton = ({ DefinicaoObjetos }: Pick<LayoutScopeConfig, 'DefinicaoObjetos'> ) => {
    const  { ExibirExecutar } = DefinicaoObjetos;

    return ExibirExecutar
        ? $('<div class="text-center mt-4">').append(ExecuteButton({ DefinicaoObjetos }))
        : $();
};