import { FieldListBase, LayoutConfig, Widget, WidgetType } from "../types";

const findFirstFieldList = (layout: LayoutConfig, widgets: Map<string, Widget>) => {
    if (layout.WidgetName) {
        const widget = widgets.get(layout.WidgetName);

        if (WidgetType.isTwFieldList(widget) || WidgetType.isTwParamList(widget)) return layout.WidgetName;
    }

    if (layout.Linhas) {
        for (let row of layout.Linhas) {
            const widgetName: string = findFirstFieldList(row, widgets);

            if (widgetName) return widgetName;
        }
    }

    if (layout.Colunas) {
        for (let column of layout.Colunas) {
            const widgetName: string = findFirstFieldList(column, widgets);

            if (widgetName) return widgetName;
        }
    }

    if (layout.Pages) {
        for (let page of layout.Pages) {
            const widgetName: string = findFirstFieldList(page.Coluna, widgets);

            if (widgetName) return widgetName;
        }
    }

    if (layout.SubLayout) {
        const widgetName: string = findFirstFieldList(layout.SubLayout, widgets);

        if (widgetName) return widgetName;
    }

    return '';
}

export const markFirstFieldList = (widgets: Map<string, Widget>, layout: LayoutConfig) => {
    const widgetName = findFirstFieldList(layout, widgets);

    if (widgetName) (widgets.get(widgetName) as FieldListBase).isMaster = true;
}
