import { BehaviorSubject } from "rxjs";
import { TwCardDisplayer } from "./tw-card-displayer/tw-card-displayer";
import { TwMiniCardValueDisplayer } from "./tw-mini-card-value-displayer/tw-mini-card-value-displayer";
import { TwValueDisplayer } from "./tw-value-displayer/tw-value-displayer";
import { TwImageValueDisplayer } from "./tw-image-value-displayer/tw-image-value-displayer";
import { WidgetExpressionExecutor } from "./widget-expression-executor";

export type WidgetInstance = 
    | TwCardDisplayer
    | TwMiniCardValueDisplayer
    | TwImageValueDisplayer
    | TwValueDisplayer;

interface AfterSetWidget {
    name: string;
    instance: WidgetInstance;
}

class WidgetManager {
    private _widgets = new Map<string, WidgetInstance>();

    public afterSetWidget = new BehaviorSubject<AfterSetWidget | null>(null);

    public expressionExecutor = new WidgetExpressionExecutor();

    get<E extends WidgetInstance>(name: string) {
        return this._widgets.get(name) as E | undefined;
    }

    set(name: string, instance: WidgetInstance) {
        this._widgets.set(name, instance);

        this.afterSetWidget.next({
            instance,
            name,
        });
    }
}

export default (new WidgetManager());