import gridManager from "../grid-manager/grid-manager";
import { TwCardDisplayerConfig, TwImageValueDisplayerConfig, TwMiniCardValueDisplayerConfig, TwValueDisplayerConfig, WidgetExpression } from "./types";
import { WidgetBase } from "./widget-base";
import widgetManager from "./widget-manager";

type WidgetConfigWithExpression =
    | TwImageValueDisplayerConfig
    | TwCardDisplayerConfig
    | TwValueDisplayerConfig
    | TwMiniCardValueDisplayerConfig;

export abstract class WidgetWithExpression<WidgetConfig extends WidgetConfigWithExpression, State extends { widget: WidgetConfig, Class: string }> extends WidgetBase<WidgetConfig, State>{
    constructor(config: State) {
        super(config);

        this._listenForExternalExpressionResponse();
        this._setExpressionEvent();
    }

    protected abstract _expressionValueSetValue(value: string): void;

    protected _setExpressionEvent() {
        const { widget: { Expressoes } } = this.getState();

        if (!Expressoes?.DataSet) return;

        const grid = gridManager.get(Expressoes.DataSet);
        let isFirstRun = false;

        if (grid) {
            isFirstRun = true;

            grid.on('contentReady', () => this._runExpression());
        }

        // set event on next construction of grid (ex: proccess execution)
        gridManager.afterSetGrid.subscribe((gridItem) => {
            if (isFirstRun || !gridItem || gridItem.name !== Expressoes.DataSet) return;

            gridItem.instance.on('contentReady', () => this._runExpression());
        });

        isFirstRun = false;
    }

    protected _runExpression() {
        const { widget: { Expressoes } } = this.getState();

        if (!Expressoes?.DataSet) return;

        const grid = gridManager.get(Expressoes.DataSet);

        if (!grid) return;

        if (Expressoes.ExpressaoValue) {
            this._runExpressionValue(Expressoes);
        }
    };

    private _runExpressionValue(Expressoes: WidgetExpression) {
        const state = this.getState();

        widgetManager.expressionExecutor.runExternal({
            className: state.Class,
            datasetName: Expressoes.DataSet!,
            expression: Expressoes.ExpressaoValue!,
            widgetName: state.widget.Name,
            type: 'value',
        });
    }

    private _listenForExternalExpressionResponse() {
        const { widget: { Name } } = this.getState();

        widgetManager.expressionExecutor
            .listenForExternalResponse(Name)
            .subscribe((response) => {
                this._expressionValueSetValue(response.value);
            });
    }

    public runExpression() {
        this._runExpression();
    }
}