import { criarDialogOk, FormatErrorMessage } from "../../../../js/nDialog";
import { Timer } from "../../../../js/Timer";
import { ToolButton } from "../../../components/fra-control/types";
import { OperacaoDataSetProps, ReportViewScopeConfig } from "../../relatorio-exibir/relatorio-exibir";
import { ReportTypeZen, SendHandlerProps } from "../envio-zen";
import { ReportProps } from "../envio-zen-report-view";
import { handleFeedbackModal } from "../feedback-modal";

declare var axios: any;

interface SendEdocButtonProps {
    modal: JQuery<HTMLElement>;
    scope: ReportViewScopeConfig;
}

export const getEnvioEdocButton = ({ modal, scope }: SendEdocButtonProps) => {
    //@ts-ignore
    window['actionOpenSendReportZen'] = actionOpenSendReportZen.bind(null, modal, scope);
    
    return getToolButonConfig(scope);
};

const getToolButonConfig = (scope: ReportViewScopeConfig): ToolButton => ({
    Caption: "Enviar E-Doc",
    Classe: scope.DefinicaoObjetos.Class,
    CleanCaption: "Enviar E-Doc",
    Custom: true,
    ExibirToolBar: true,
    Hint: "Enviar E-Doc",
    Name: "actionOpenSendReportZen",
    NameEvent: "actionOpenSendReportZen",
    Opcoes: {
        ReportTypeZen: 'edoc' as ReportTypeZen,
    },
    TipoAcao: "TnActOperacaoDataset",
    icon: '',
    tbsButton: true,
});

const actionOpenSendReportZen = (modal: JQuery<HTMLElement>, scope: ReportViewScopeConfig, { _AOpcoes }: OperacaoDataSetProps) => {
    const { DefinicaoObjetos, fileNameNhz, Params } = scope;
    const reportName = fileNameNhz;
    const iframeEl = modal.find('iframe');

    if (!reportName) throw new Error('File name is null');

    const params: ReportProps & { ReportType: ReportTypeZen } = {
        Params: Params!,
        ClassName: DefinicaoObjetos.Class,
        ReportName: reportName,
        ReportType: _AOpcoes.ReportTypeZen,
    };

    const modalParams = `BuscarDadosEnvioZen?Scope=${encodeURIComponent(JSON.stringify(params))}`

    iframeEl.contents().find('body').html('');
    iframeEl.attr('src', modalParams);

    // @ts-ignore;
    modal.modal();
};


interface ResponseSuccess {
    result: 'Sucesso';
}

interface ResponseError {
    result: 'Falha';
    grids: any[];
    fields: {
        [key: string]: any;
    };
}

type ResponseData = ResponseSuccess | ResponseError;

export const sendEdocHandler = async ({ reportProps, formValues, gridValues, modal, url }: SendHandlerProps) => {
    const { ClassName, Params, ReportName } = reportProps;
    const timer = new Timer();

    try {
        timer.IniciarExecucao();
       
        const response = await axios.post(url || 'nHttpServicos/ExecutarEnvioZen', {
            p: {
                ClassName,
                ReportName,
                Data: {
                    ...formValues,
                    DataSet: {
                        NomeDataset: 'nClientDataSet',
                        Registros: gridValues,
                    },
                },
                Params,
            }
        });

        timer.FinalizarExecucao();

        const { data }: { data: ResponseData} = response.data;

        console.log('resposeData', data);

        switch(data.result) {
            case 'Sucesso':
                // @ts-ignore
                modal.modal('hide');

                criarDialogOk('Documento(s) enviado(s) com sucesso', 'SUCESSO');

                break;
            case 'Falha':
                // @ts-ignore
                modal.modal('hide');

                handleFeedbackModal({
                    reportProps,
                    formValues,
                    gridsDocument: data.fields.ListaFeedBackEnvio,
                    dataDestinatarios: gridValues,
                });

                break;
            default: 
                FormatErrorMessage('Não foi possível enviar o documento');
        }
    }
    catch (e) {
        timer.FinalizarExecucao();

        console.error(e);
        
        FormatErrorMessage('Não foi possível enviar o documento');
    }
};