import { Field } from "../../../components/fra-control/types";
import { GetFieldsProps } from "../get-fields";

export const getQColaboreFields = (props: GetFieldsProps): Field[] => {
    const Fields: Field[] = [
        getFieldTipoEnvio(props),
        getFieldFormato(props),
        getFieldLinhasIgnorarCabecalho(props),
        getFieldLinhasIgnorarRodape(props),
        getFieldCategoriaQColabore(props),
        getFieldAssunto(props),
        getFieldDescricao(props),
    ];

    return Fields;
};

const getFieldTipoEnvio = ({ Accordion, ClassName }: GetFieldsProps): Field => ({
    Accordion,
    AlignTH: "left",
    Alignment: "taLeftJustify",
    AutoIncrementar: false,
    BoldStyle: "normal",
    DMClassName: ClassName,
    DataType: "ftString",
    Dicionario: "DICTEXTO",
    DisplayLabel: "Tipo Envio",
    DisplayWidth: 100,
    DisplayWidthColHtml: 300,
    Expressoes: {
        ForcarValor: false,
        PermiteAlterarValorCloud: false
    },
    Hint: "Tipo Envio",
    InputType: "text",
    LinkDirecionaCadastro: false,
    LinkLookupMultiSelecao: false,
    Lookup: false,
    MaxLength: 300,
    Name: "TIPOENVIO",
    NumeroColunas: 4,
    NumeroColunasLabel: 3,
    ReadOnly: false,
    Required: true,
    TabStop: true,
    TextCase: "",
    TipoControle: "ntcEdit",
    Valor: "",
    Visible: false,
    disabled: "",
    ntcEdit: true,
    oculto: true,
});

const getFieldFormato = ({ Accordion, ClassName, Definicao }: GetFieldsProps): Field => ({
    Accordion,
    AlignTH: "left",
    Alignment: "taLeftJustify",
    AutoIncrementar: false,
    BoldStyle: "bold",
    DMClassName: ClassName,
    DataType: "ftString",
    Dicionario: "",
    DisplayLabel: "Formato",
    DisplayWidth: 50,
    DisplayWidthColHtml: 150,
    Expressoes: { ForcarValor: false, PermiteAlterarValorCloud: false },
    InputType: "text",
    LinkDirecionaCadastro: false,
    LinkLookupMultiSelecao: false,
    Lookup: false,
    Name: "FORMATO",
    NumeroColunas: 4,
    NumeroColunasLabel: 3,
    ReadOnly: false,
    Required: true,
    TabStop: true,
    TextCase: "",
    TipoControle: "ntcComboBox",
    Valor: "",
    Visible: true,
    disabled: "",
    ntcComboBox: true,
    oculto: false,
    ...getComboBoxNameValue(Definicao.Formato),
});

const getFieldLinhasIgnorarCabecalho = ({ Accordion, ClassName, Definicao }: GetFieldsProps): Field => {
    const expressionValues = getCsvValueForExpression(Definicao.Formato);

    return {
        Accordion,
        AlignTH: "left",
        Alignment: "taLeftJustify",
        AutoIncrementar: false,
        BoldStyle: "normal",
        DMClassName: ClassName,
        DataType: "ftString",
        Dicionario: "DICTEXTO",
        DisplayLabel: "Linhas a Ignorar (CSV) Cabeçalho",
        DisplayWidth: 100,
        DisplayWidthColHtml: 300,
        Expressoes: {
            ForcarValor: false,
            PermiteAlterarValorCloud: false,
            ExpressaoEnabled: `IN(FORMATO, [${expressionValues}])`,
            ExpressaoRequired: `IN(FORMATO, [${expressionValues}])`,
        },
        Hint: "Linhas a Ignorar (CSV) Cabeçalho",
        InputType: "text",
        LinkDirecionaCadastro: false,
        LinkLookupMultiSelecao: false,
        Lookup: false,
        MaxLength: 300,
        Name: "LINHASIGNORARCABECALHO",
        NumeroColunas: 4,
        NumeroColunasLabel: 3,
        ReadOnly: false,
        Required: true,
        TabStop: true,
        TextCase: "",
        TipoControle: "ntcEdit",
        Valor: "",
        Visible: true,
        disabled: "",
        ntcEdit: true,
        oculto: false,
    }
};

const getFieldLinhasIgnorarRodape = ({ Accordion, ClassName, Definicao }: GetFieldsProps): Field => {
    const expressionValues = getCsvValueForExpression(Definicao.Formato);

    return {
        Accordion,
        AlignTH: "left",
        Alignment: "taLeftJustify",
        AutoIncrementar: false,
        BoldStyle: "normal",
        DMClassName: ClassName,
        DataType: "ftString",
        Dicionario: "DICTEXTO",
        DisplayLabel: "Linhas a Ignorar (CSV) Rodapé",
        DisplayWidth: 100,
        DisplayWidthColHtml: 300,
        Expressoes: {
            ForcarValor: false,
            PermiteAlterarValorCloud: false,
            ExpressaoEnabled: `IN(FORMATO, [${expressionValues}])`,
            ExpressaoRequired: `IN(FORMATO, [${expressionValues}])`,
        },
        Hint: "Linhas a Ignorar (CSV) Rodapé",
        InputType: "text",
        LinkDirecionaCadastro: false,
        LinkLookupMultiSelecao: false,
        Lookup: false,
        MaxLength: 300,
        Name: "LINHASIGNORARRODAPE",
        NumeroColunas: 4,
        NumeroColunasLabel: 3,
        ReadOnly: false,
        Required: true,
        TabStop: true,
        TextCase: "",
        TipoControle: "ntcEdit",
        Valor: "",
        Visible: true,
        disabled: "",
        ntcEdit: true,
        oculto: false,
    }
};

const getFieldCategoriaQColabore = ({ Accordion, ClassName, Definicao }: GetFieldsProps): Field => ({
    Accordion,
    AlignTH: "left",
    Alignment: "taLeftJustify",
    AutoIncrementar: false,
    BoldStyle: "bold",
    DMClassName: ClassName,
    DataType: "ftString",
    Dicionario: "",
    DisplayLabel: "Categoria Q-Colabore",
    DisplayWidth: 50,
    DisplayWidthColHtml: 150,
    Expressoes: { ForcarValor: false, PermiteAlterarValorCloud: false },
    InputType: "text",
    LinkDirecionaCadastro: false,
    LinkLookupMultiSelecao: false,
    Lookup: false,
    Name: "CATEGORIA",
    NumeroColunas: 4,
    NumeroColunasLabel: 3,
    ReadOnly: false,
    Required: true,
    TabStop: true,
    TextCase: "",
    TipoControle: "ntcComboBox",
    Valor: "",
    Visible: true,
    disabled: "",
    ntcComboBox: true,
    oculto: false,
    ...getComboBoxNameValue(Definicao.Categoria),
});

const getFieldAssunto = ({ Accordion, ClassName }: GetFieldsProps): Field => ({
    Accordion,
    AlignTH: "left",
    Alignment: "taLeftJustify",
    AutoIncrementar: false,
    BoldStyle: "normal",
    DMClassName: ClassName,
    DataType: "ftString",
    Dicionario: "DICTEXTO",
    DisplayLabel: "Assunto",
    DisplayWidth: 100,
    DisplayWidthColHtml: 300,
    Expressoes: { ForcarValor: false, PermiteAlterarValorCloud: false },
    Hint: "Assunto",
    InputType: "text",
    LinkDirecionaCadastro: false,
    LinkLookupDataSetNome: "nGemDCAtributosZen",
    LinkLookupKeyFields: "CODIGO;ATRIBUTO",
    LinkKeyFields: "CATEGORIA;TITULO",
    LinkLookupResultField: "ATRIBUTO",
    LinkLookupMultiSelecao: true,
    Lookup: false,
    MaxLength: 300,
    Name: "ASSUNTO",
    NumeroColunas: 4,
    NumeroColunasLabel: 3,
    ReadOnly: false,
    Required: true,
    TabStop: true,
    TextCase: "",
    TipoControle: "ntcLinkAnnotation",
    Valor: "",
    Visible: true,
    disabled: "",
    ntcLinkAnnotation: true,
    oculto: false,
});

const getFieldDescricao = ({ Accordion, ClassName }: GetFieldsProps): Field => ({
    Accordion,
    AlignTH: "left",
    Alignment: "taLeftJustify",
    AutoIncrementar: false,
    BoldStyle: "bold",
    ChavePrimaria: false,
    DMClassName: ClassName,
    DataType: "ftString",
    Dicionario: "",
    DisplayLabel: "Descrição",
    DisplayWidth: 4000,
    DisplayWidthColHtml: 350,
    Expressoes: { ForcarValor: false, PermiteAlterarValorCloud: false },
    Hint: "Descrição",
    InputType: "text",
    LinkDirecionaCadastro: false,
    LinkLookupDataSetNome: "nGemDCAtributosZen",
    LinkLookupKeyFields: "CODIGO;ATRIBUTO",
    LinkKeyFields: "CATEGORIA;DESCRICAO",
    LinkLookupResultField: "ATRIBUTO",
    LinkLookupMultiSelecao: true,
    Lookup: false,
    MaxLength: 4000,
    Name: "DESCRICAO",
    NumeroColunas: 12,
    NumeroColunasLabel: 3,
    ReadOnly: false,
    Required: true,
    TabStop: true,
    TextCase: "",
    TipoControle: "ntcMemoAnnotation",
    Visible: true,
    disabled: "",
    ntcMemoAnnotation: true,
    oculto: false,
});

const getComboBoxNameValue = (str: string) => {
    try {
        const options = str.split('\r\n');

        return {
            NomesValores: options.join(';'),
            NomesValoresList: options
                .filter(e => e)
                .map((option) => {
                    const [name = '', value = ''] = option.split('=');

                    return {
                        nvNome: name.trim(),
                        nvValor: value.trim(),
                    };
                }),
        };
    }
    catch (e) {
        console.error(e);

        return {
            NomesValores: '',
            NomesValoresList: [{
                nvNome: 'Nenhum Item Selecionado',
                nvValor: '0',
            }],
        };
    }
};

const getCsvValueForExpression = (str: string) => {
    const options = getComboBoxNameValue(str);

    const values = options.NomesValoresList
        .filter(({ nvNome }) => nvNome.match(/\*\.csv/))
        .map(({ nvValor }) => nvValor);

    return values.toString();
}