import dxDataGrid from "devextreme/ui/data_grid";
import SessionManager from "../../../js/SessionManager";
import { Formulario } from "../../../js/nFormulario";
import { ToolButton } from "../../components/fra-control/types";
import { ReportViewScopeConfig } from "../relatorio-exibir/relatorio-exibir";
import { getEnvioEdocButton, sendEdocHandler } from "./envio-edoc/send-action";
import { getSendQColaboreButton, sendQColaboreHandler } from "./envio-qcolabore/send-action";
import { fixRowValues } from "../../../js/nGrid";

export interface ReportProps {
    Params: {
        [key: string]: any;
    };
    ClassName: string;
    ReportName: string;
}

export const configureEnvioZenReportView = (scope: ReportViewScopeConfig): ToolButton[] => {
    const modal = createModalOnDocument(scope);
    const isFpa = SessionManager.moduloAtual === 'npmnFpa';
    const buttons = [ getEnvioEdocButton({ modal, scope }) ];
    
    if (isFpa) buttons.push( getSendQColaboreButton({ modal, scope }) );

    return buttons;
};

const createModalOnDocument = ({ DefinicaoObjetos, Params, fileNameNhz } :ReportViewScopeConfig) => {
    const modal = $(`
        <div class="modal fade" tabindex="-1" role="dialog" aria-labelledby="ModalLabel">
            <div class="modal-dialog modal-xl  modal-dialog-centered" role="document">
                <div class="modal-content">
                <div class="modal-header">
                    <h5 class="modal-title">Enviar e-Doc</h5>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                    <span aria-hidden="true">×</span>
                    </button>
                </div>
                <div class="modal-body iframe-modal">
                    <div class="kt-content" id="conteudo" style="margin-bottom: -5px">
                        <iframe class="iframe-modal" height=512 width=100% >			
                        </iframe>
                    </div>
                </div>
                <div class="modal-footer">
                    <button type="button" class="btn btn-text-dark btn-hover-transparent-dark" data-dismiss="modal">CANCELAR</button>        
                    <button id="sendButton" type="button" class="btn btn-dark ml-4 py-2">ENVIAR</button>
                </div>
                </div>
            </div>
        </div>
    `);

    $(document.body).append(modal);

    sendReportHandler(modal, {
        Params: Params!,
        ClassName: DefinicaoObjetos.Class,
        ReportName: fileNameNhz,
    });

    return modal;
};

const sendReportHandler = (modal: JQuery<HTMLElement>, reportProps :ReportProps) => {
    const iframe = modal.find('iframe').get(0);

    modal
        .find('#sendButton')
        .on('click', async () => {
            const iframeWidow = (iframe.contentWindow as any);
            const form: Formulario = iframeWidow.FormManager.itens.firstUsable();

            if (!form.validateRequiredFields()) return;

            const gridInstance = iframeWidow.gridManager.get('nClientDataSet');
            const formValues = form.getValues();
            const gridValues = await getGridValues(gridInstance);

            switch(formValues['TIPOENVIO']) {
                case '2': return sendEdocHandler({ formValues, gridValues, reportProps, modal });
                case '3': return sendQColaboreHandler({ formValues, gridValues, reportProps, modal });
            }
        });
};

const getGridValues = async (gridInstance?: dxDataGrid) => {
    if (!gridInstance) return [];
    if (gridInstance.hasEditData()) await gridInstance.saveEditData();

    return fixRowValues(gridInstance, gridInstance.getDataSource().items());
};
