import dxDataGrid from "devextreme/ui/data_grid";
import { fixRowValues } from "../../../js/nGrid";
import gridManager from "../../components/grid-manager/grid-manager";
import { TwCxGrid } from "../../components/widgets/tw-cx-grid/tw-cx-grid";
import { sendEdocHandler } from "./envio-edoc/send-action";
import { sendQColaboreHandler } from "./envio-qcolabore/send-action";
import { SendHandlerProps } from "./envio-zen";
import { ReportProps } from "./envio-zen-report-view";

interface KeyValObj {
    [key: string]: any;
};

export const handleFeedbackModal = ({
    formValues,
    reportProps,
    dataDestinatarios,
    gridsDocument,
}: {
    reportProps: ReportProps,
    dataDestinatarios: any[],
    gridsDocument: any[],
    formValues: KeyValObj,
}) => {
    const getGridDocuments = (gridConf: any[]) => {
        const gridName = gridConf[0].nomegrid;
        const gridInstance = gridManager.get(gridName)!;

        return {
            NomeDataset: gridName,
            Registros: fixRowValues(gridInstance, gridInstance.getDataSource().items()),
        }
    };

    const feedbackModal = createFeedbackModal({ 
        reportProps,
        grids: gridsDocument, 
    });

    feedbackModal
        .find('#reSendButton')
        .on('click', async () => {
            const sendProps: SendHandlerProps = {
                url: 'nHttpServicos/ExecutarReEnvioZen',
                formValues: {
                    ...formValues,
                    ['ListaFeedBackEnvio'.toUpperCase()]: getGridDocuments(gridsDocument),
                },
                gridValues: dataDestinatarios,
                modal: feedbackModal,
                reportProps,
            };

            switch(formValues['TIPOENVIO']) {
                case '2': return sendEdocHandler(sendProps);
                case '3': return sendQColaboreHandler(sendProps);
                default: console.error('Invalid TipoEnvio', formValues['TIPOENVIO']);
            }
        });
};

export const createFeedbackModal = ({ grids, reportProps }: { grids: any, reportProps: ReportProps }) => {
    const modal = $(`
        <div class="modal fade" tabindex="-1" role="dialog" aria-labelledby="ModalLabel">
            <div class="modal-dialog modal-xl  modal-dialog-centered" role="document">
                <div class="modal-content">
                    <div class="modal-header">
                        <h5 class="modal-title">Erro no Envio de Documento</h5>
                        <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                            <span aria-hidden="true">×</span>
                        </button>
                    </div>
                    <div class="modal-body">
                        <div class="kt-content">
                            <div class="mb-8">
                                <h1>STATUS DE ENVIO</h1>
                                <span class="d-block">Houve(ram) erro(s) no envio do(s) documento(s).</span>
                                <span class="d-block">Deseja reenviar o(s) documento(s) com erro(s) no envio?</span>
                            </div>
                            <div class="accordion accordion-solid accordion-toggle-arrow" id="main_accordion_Credito">
                                <div class="card">
                                    <div class="card-header" id="main_accordion_item_documentos-nao-enviados">
                                        <div class="card-title" data-toggle="collapse"
                                            data-target="#accordion-body-documentos-nao-enviados" aria-expanded="true">
                                            <div>
                                                <span>Ver Documentos Não Enviados</span>
                                            </div>
                                        </div>
                                    </div>
                                    <div id="accordion-body-documentos-nao-enviados" data-accordion="true" class="collapse show"
                                        aria-labelledby="Accordion Item"
                                        data-parent="#main_accordion_item_documentos-nao-enviados" style="">
                                        <div class="card-body ">
                                            <div class="card card-custom mb-0 mt-4" id="widget_GridDestinatario">
                                                <div class="card-body">
                                                    <div id="ConteudoGrid_wrapper">
                                                        <div id="gridTools"></div>
                                                        <div id="dataGridConteudonClientDataSet"> </div>
                                                    </div>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                    <div class="modal-footer">
                        <button type="button" class="btn btn-text-dark btn-hover-transparent-dark"
                            data-dismiss="modal">CANCELAR</button>
                        <button id="reSendButton" type="button" class="btn btn-dark ml-4 py-2">REENVIAR</button>
                    </div>
                </div>
            </div>
        </div>
    `);


    modal
        .find('[id*="dataGridConteudo"]')
        .replaceWith(TwCxGrid({
            widget: ({
                twcxGrid: true,
                Grid: {
                    ...grids[0],
                    editing: false,
                },
                Name: 'GridDestinatario',
            } as any),
            DefinicaoObjetos: {
                BodyCols: 12,
                    CadCons: "",
                    Caption: 'Reenvio documentos',
                    Class: reportProps.ClassName,
                    ClassForm: "TnFrmReport",
                    ClassParent: "TComponent.TDataModule.TnDM.TnDMPower",
                    ConsultaInativo: false,
                    Detalhes: [],
                    Fields: [],
                    IDRequest: 0,
                    JavaScript: "",
                    MultiColuna: false,
                    Name: "",
                    ProcessoPadrao: true,
                    TipoObjeto: "ntoRelatorio",
                    ToolButtons: [],
                    TopicoAjuda: reportProps.ClassName,
                    Widgets: false,
                    ChavePrimaria: [],
                    configextra: null,
            },
        }))

    const gridElement = modal.find(`[id*="dataGridConteudo"]`);

    gridElement.closest('.collapse').on('shown.bs.collapse', () => {
        let instance: dxDataGrid = (dxDataGrid.getInstance(gridElement) as dxDataGrid);
        
        instance.repaint();
        instance.refresh();
    });

    $(document.body).append(modal);

    modal.on('hidden.bs.modal', () => {
        let instance: dxDataGrid = (dxDataGrid.getInstance(gridElement) as dxDataGrid);
        
        instance.dispose();
        
        modal.remove();
    });
    
    // @ts-ignore
    modal.modal()
    
    return modal;
};