import dxDataGrid, { Column } from 'devextreme/ui/data_grid';
import jsPDF from 'jspdf'
import autoTable, { Styles, ColumnInput, RowInput } from 'jspdf-autotable'
import { formatWithDataType } from '../../../js/nGrid';

interface RowData {
    [key: string]: string
}

interface columnStyles {
    [key: string]: Partial<Styles>;
}

interface ExportPdfProps {
    instance: dxDataGrid;
    data: Array<RowData>;
    fileName?: string;
}

export const exportPdf = ({ data, instance, fileName = 'pdf' }: ExportPdfProps) => {
    const gridColumns = instance.getVisibleColumns().filter(column => !column.type);

    const doc = new jsPDF({
        orientation: 'landscape',
    });

    autoTable(doc, {
        columns: getColumns(gridColumns),
        body: getRows(gridColumns, data),
        columnStyles: getColumnStyles(gridColumns),
        headStyles:{
            fontStyle: 'bold',
            fontSize: 4,
            textColor: 'white',
            fillColor: '#2d4154',
            halign: 'center',
        },
        styles: {
            fontSize: 4,
            cellWidth: 'auto',
            textColor: 'black',
        },
    });

    doc.save(`${fileName}`);
};

const getColumns = (gridColumns: Array<Column>): Array<ColumnInput> => gridColumns.map(column => ({
    header: column.caption || '',
    dataKey: column.dataField,
}));

const getRows = (gridColumns: Array<Column>, data: Array<RowData>): Array<RowInput> => data.map((row) => formatRowForPdf(gridColumns, row));

const formatRowForPdf = (gridColumns: Array<Column>, row: RowData) => {
    const visibleRowValues = gridColumns.reduce((rowValues, column) => ({
        ...rowValues,
        [column.dataField!]: row[column.dataField!],
    }), {} as RowData);;

    Object
        .entries(visibleRowValues)
        .forEach(([columnName, columnValue]) => {
            const { dataType } = gridColumns.find(column => column.dataField == columnName) || {};

            if (!dataType) return;

            formatWithDataType(dataType!, visibleRowValues, columnName, columnValue);

            if (typeof columnValue === "boolean" || (columnValue as any) instanceof Boolean) {
                columnValue = (columnValue) ? "Ativo" : "Inativo";
            }

            // strip html from columns
            visibleRowValues[columnName] = visibleRowValues[columnName].replace(/<(?:.|\n)*?>/gm, "").trim();
        });

    return visibleRowValues;
};

const getColumnStyles = (gridColumns: Array<Column>) => {
    return gridColumns.reduce((styles, column) => ({
        ...styles,
        [column.dataField as string]: {
            halign: column.alignment,
        },
    }), {} as columnStyles);
};